/* test_aggregator.c - test aggregator */

/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <assert.h>
#include <cafe/ep/user_event.h>
#include <cafe/ep/aggregator.h>
#include <test/cafe_CUnitTest.h>


#define TEST_ITERS (43)

static bool setup_complete = false;	/* setup routine completed OK */


static
void setup(void)
{
	int rc;

	assert(!setup_complete);
	CAFE_LOCK();

	rc = cafe_aggregator_ep_register();
	if (rc == CAFE_OK)
		setup_complete = true;
}


static
void teardown(void)
{
	if (setup_complete)
		cafe_aggregator_ep_unregister();

	CAFE_UNLOCK();
	setup_complete = false;
}

static
int test_aggregator(void)
{
	struct cafe_instrument *session;
	struct cafe_ep_spec *spec;
	struct cafe_ep_inst *inst[2];
	struct cafe_value_type *type;
	struct cafe_value_type *type2;

	CUT_ASSERT_TRUE(setup_complete);

	CUT_ASSERT_CAFE_OK(cafe_instrument_create(
				"test_user_event", &session));

	spec = cafe_ep_spec_find("vxworks_kernel");
	CUT_ASSERT_NOT_NULL(spec);

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, spec,
					       0, NULL, &inst[0]));

	spec = cafe_ep_spec_find("cafe_aggregator");
	CUT_ASSERT_NOT_NULL(spec);

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, spec,
					       0, NULL, &inst[1]));

	/* Bypass threading compatibility checks */
	inst[0]->flags |= CAFE_EP_INST_FLAG_OUTPUT_SESSION_THREAD;

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_connect(inst[0], inst[1]));

	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, user_event_signed_VT_DEF, &type));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, "S[g,v[c1,L," cafe_SUM_VT_NAT "]v[s1,q," cafe_SUM_VT_NAT "]v[s2,t[cafe.setval_q]," cafe_SET_VT_NAT "]v[m1,t[cafe.setval_Q]," cafe_MIN_VT_NAT "]v[m2,t[cafe.setval_Q]," cafe_MAX_VT_NAT "]v[a1,t[cafe.avgval_Q]," cafe_AVG_VT_NAT "]]", &type));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, "S[k,k[k1,L]v[c1,Q," cafe_SUM_VT_NAT "]]", &type2));

	CUT_ASSERT_CAFE_OK(cafe_aggregator_ep_table_create(inst[1], type));
	CUT_ASSERT_CAFE_OK(cafe_aggregator_ep_table_create(inst[1], type2));

	CUT_ASSERT_CAFE_OK(cafe_aggregator_ep_action_add(
				inst[1], "user_event.signed", NULL,
				"g.c1", "count", NULL));

	CUT_ASSERT_CAFE_OK(cafe_aggregator_ep_action_add(
				inst[1], "user_event.signed", NULL,
				"g.s1", "sum", "user_event.signed.value"));

	CUT_ASSERT_CAFE_OK(cafe_aggregator_ep_action_add(
				inst[1], "user_event.signed", NULL,
				"g.s2", "set", "user_event.signed.value"));

	CUT_ASSERT_CAFE_OK(cafe_aggregator_ep_action_add(
				inst[1], "user_event.signed", NULL,
				"g.m1", "min", "user_event.signed.value"));

	CUT_ASSERT_CAFE_OK(cafe_aggregator_ep_action_add(
				inst[1], "user_event.signed", NULL,
				"g.m2", "max", "user_event.signed.value"));

	CUT_ASSERT_CAFE_OK(cafe_aggregator_ep_action_add(
				inst[1], "user_event.signed", NULL,
				"g.a1", "avg", "user_event.signed.value"));

	CUT_ASSERT_CAFE_OK(cafe_aggregator_ep_action_add(
				inst[1], "user_event.signed", NULL,
				"k[$1.value].c1", "count", NULL));

	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session));

	CUT_ASSERT_CAFE_OK(cafe_instrument_start(session));

	CAFE_UNLOCK();

	cafe_user_event_signed("l1", 100);
	cafe_user_event_signed("l1", 200);
	cafe_user_event_signed("l1", 300);
	cafe_user_event_signed("l1", 400);

	CAFE_LOCK();

	/* clean up, disconnect, and deallocate configurations... */
	CUT_ASSERT_CAFE_OK(cafe_instrument_stop(session));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_disconnect(inst[0], inst[1]));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_delete(inst[1]));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_delete(inst[0]));

	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session));

	return 0;
}


CUT_BEGIN_SUITE_F(aggregator_suite, &setup, &teardown)
	CUT_RUN_TEST(test_aggregator);
CUT_END_SUITE_F
