/* test_stacktrace.c -  */

/* Copyright (c) 2010-2011, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
15may14,sdt  fixed resource leak
*/

/*
DESCRIPTION
*/

#include <cafe/cafe.h>
#include <cafe/autogenerated/cafe.type.h>
#include <cafe/cafe_print.h>
#include <cafe/ep/print.h>
#include <test/cafe_CUnitTest.h>
#include <st_stacktrace.h>
#include "dummy_cfg.h"
#include "verify_cfg.h"
#if defined(__VXWORKS__)
#  include <st_vxutils.h>  /* for ST_get_thread_id() */
#endif


/**
 *  Verify that trivial stack tracing works correctly.
 */
static
int test_stacktrace(void)
{
#if defined(__linux__) && (__GNUC__ == 3 && __GNUC_MINOR__ == 4 && __GNUC_PATCHLEVEL__ == 6)
	/* Stacktrace currently fails on gcc-3.4.6.
	 *
	 * We pull out this version of the compiler specifically as
	 * this is what our auto build machines currently have as
	 * their default compiler.  If it doesn't work elsewhere we
	 * still want to run the tests even though the test may fail.
	 */
	return 0;
#endif
	REG_SET regs;
	ST_context ctx;
	void *stack[64];
	unsigned int stack_len;
	//extern int RtiTools_ContextDebuggingOutput;
#if defined(__linux__)
	MemoryMap map;
	CUT_ASSERT(getSelfMap(&map));
	ST_SetMmap(&map);
#endif
	ST_current_regs(&regs);
	CUT_ASSERT_NOT_NULL((void *)regs.reg_sp);
	//CUT_ASSERT_NOT_NULL(regs.lrReg);
	CUT_ASSERT_NOT_NULL(regs.pc);
	CUT_ASSERT(ST_context_create(&regs, ST_get_thread_id(), false, &ctx));
	stack_len = ST_perform_stack_trace(&ctx, stack, 64);
	(void)ST_context_delete(&ctx);
	CUT_ASSERT(stack_len > 2);

#if defined(__linux__)
	MemoryMapDestroy(&map);
	ST_SetMmap(0);
#endif

	//CUT_FAIL("need more verification of stack validity");
	return 0;
}


/**
 *  Verify we can successfully request and receive stack trace data as
 * orthogonal data.
 */
static struct cafe_event_descr *test_stack_event = NULL;

static void test_stack_odata_free(void)
{
	if (test_stack_event != NULL) {
	    free(test_stack_event);
	    test_stack_event = NULL;
	}
}

static
int test_stack_odata(void)
{
	static const unsigned int TEST_ITERS = 5;
	struct cafe_instrument *session;
	struct cafe_ep_spec *spec;
	struct cafe_ep_inst *inst[2];
	struct cafe_event_descr *dummy_event;
	unsigned int i;
#if defined(__linux__)
	MemoryMap map;
	CUT_ASSERT(getSelfMap(&map));
	ST_SetMmap(&map);
#endif

	CAFE_LOCK();
	CUT_ASSERT_CAFE_OK(dummy_cfg_register());
	CUT_ASSERT_CAFE_OK(cafe_print_configuration_register());
	//CUT_ASSERT_CAFE_OK(verify_cfg_register());

	CUT_ASSERT_CAFE_OK(cafe_instrument_create(__func__, &session));

	spec = cafe_ep_spec_find("dummy_source");
	CUT_ASSERT_NOT_NULL(spec);
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, spec,
					       0, NULL, &inst[0]));

	spec = cafe_ep_spec_find("print");
	CUT_ASSERT_NOT_NULL(spec);

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, spec,
					       0, NULL, &inst[1]));

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_connect(inst[0], inst[1]));

	/* set up the description for the dummy event with stacktrace odata */
	dummy_event = test_stack_event = malloc(sizeof(struct cafe_event_descr)
			     + sizeof(struct cafe_value_type *));
	CUT_ASSERT_NOT_NULL(dummy_event);
	dummy_event->num_types = 2;
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, DUMMY_EVT_VT_REF,
					   &dummy_event->types[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, cafe_stacktrace_VT_REF,
					   &dummy_event->types[1]));

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst[0],
							    dummy_event));

	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session));

	//CUT_ASSERT_CAFE_OK(cafe_ep_inst_print_chain(inst[1], CAFE_CFG_PRINT_INPUT | CAFE_CFG_PRINT_OUTPUT));
	CUT_ASSERT_CAFE_OK(cafe_instrument_start(session));

	for (i = 0; i < TEST_ITERS; ++i)
		CUT_ASSERT_CAFE_OK(send_dummy_event(inst[0], i));

	/* clean up, disconnect, and deallocate configurations... */
	CUT_ASSERT_CAFE_OK(cafe_instrument_stop(session));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_disconnect(inst[0], inst[1]));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_delete(inst[0]));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_delete(inst[1]));

	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session));
	verify_cfg_unregister();
	dummy_cfg_unregister();
	CAFE_UNLOCK();

	test_stack_odata_free();
#if defined(__linux__)
	MemoryMapDestroy(&map);
	ST_SetMmap(0);
#endif

	return 0;
}


CUT_BEGIN_SUITE(stacktrace_suite)
	CUT_RUN_TEST(test_stacktrace);
	CUT_RUN_TEST_F(test_stack_odata, NULL, test_stack_odata_free);
CUT_END_SUITE
