/* Copyright (c) 2011-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
14may14,sdt  Disable COPY_PASTE_ERROR warnings
24feb14,sdt  Disable unexpected NO_EFFECT warnings

DESCRIPTION

*/

#include <string.h>
#include <test/cafe_CUnitTest.h>
#include <cafe/atomic.h>
#include <cafe/autogenerated/vxworks.type.h>
#include <cafe/ep/vxworks_kernel.h>
#include "../vxworks_events.h"
#include <assert.h>

#ifdef __VXWORKS__
#include <private/eventP.h>
#endif

#define EP_NAME "vxworks_enable_disable_test"

#define WORKAREA_DEF "S[" EP_NAME ".workarea,v[dummy,q]]"
#define WORKAREA_REF "t[" EP_NAME ".workarea]"

#define ALL_TYPES				\
	WORKAREA_DEF				\
	vxWorks_semGive_VT_DEF	\
	vxWorks_memAlloc_VT_DEF

#if defined(__VXWORKS__)
static PART_ID mem_part_id;
static char *pool;
static unsigned int poolsize = 1000;
static SEM_ID sem_id;
#endif

#define SEMGIVE		0
#define MEMALLOC	1

#define SEMGIVE_EVENT_COUNT(INST_IDX)					\
	(((struct inst_data *)instance_list[(INST_IDX)]->client_data)->event_data[SEMGIVE].event_count)

#define MEMALLOC_EVENT_COUNT(INST_IDX)					\
	(((struct inst_data *)instance_list[(INST_IDX)]->client_data)->event_data[MEMALLOC].event_count)

static struct cafe_instrument *instrument[NUM_ROOT_INSTANCES];
static struct cafe_ep_inst *instance_list[NUM_ROOT_INSTANCES];

struct event_data {
	unsigned long event_count;
	bool enabled;
};

static const char *SEMGIVE_TYPE[] = {
	vxWorks_semGive_VT_REF
};

static const char *MEMALLOC_TYPE[] = {
	vxWorks_memAlloc_VT_REF
};

static struct cafe_ep_spec_input_evt input_events[] = {
	CAFE_INPEV_INIT2(SEMGIVE, (void *)0),
	CAFE_INPEV_INIT2(MEMALLOC, (void *)1)
};

struct inst_data {
	unsigned int index;
	struct event_data event_data[CAFE_NELEMENTS(input_events)];
	struct cafe_ep_inst *source;
};

static void semgive_action_handler(struct cafe_event_action_info *info, void **data)
{
#if defined(__VXWORKS__)
	struct vxWorks_semGive *e =
		(struct vxWorks_semGive *)data[0];
#endif
	struct inst_data *inst_data;
	unsigned long event_index;

#if defined(__VXWORKS__)
	if (e->semId != (unsigned long)sem_id)
		return;
#endif
	inst_data = (struct inst_data *)info->inst->client_data;
	event_index = (unsigned long)info->impl_data;
	inst_data->event_data[event_index].event_count++;
	assert(inst_data->event_data[event_index].enabled);
}

static void memalloc_action_handler(struct cafe_event_action_info *info, void **data)
{
#if defined(__VXWORKS__)
	struct vxWorks_memAlloc *e =
		(struct vxWorks_memAlloc *)data[0];
#endif
	struct inst_data *inst_data;
	unsigned long event_index;

#if defined(__VXWORKS__)
	if (e->partId != (unsigned long)mem_part_id)
		return;
	assert(e->nBytesRequested == 1);
#endif
	inst_data = (struct inst_data *)info->inst->client_data;
	event_index = (unsigned long)info->impl_data;
	inst_data->event_data[event_index].event_count++;
	assert(inst_data->event_data[event_index].enabled);
}

static int inst_create(struct cafe_ep_inst *inst)
{
	unsigned int i;
	struct inst_data *inst_data;

 	for (i = 0; i < CAFE_NELEMENTS(instance_list); i++)
		if (instance_list[i] == NULL)
			break;

	if (i == CAFE_NELEMENTS(instance_list))
		return CAFE_NOMEM;

	inst_data = CAFE_ALLOC(inst->allocator, sizeof *inst_data);

	if (inst_data == NULL)
		return CAFE_NOMEM;

	memset(&inst_data->event_data, 0, sizeof inst_data->event_data);
	inst_data->index = i;
	instance_list[i] = inst;
	inst->client_data = inst_data;

	return CAFE_OK;

}

static void inst_delete(struct cafe_ep_inst *inst)
{
	struct inst_data *inst_data =
		(struct inst_data *)inst->client_data;

	instance_list[inst_data->index] = NULL;
	CAFE_FREE(inst->allocator, inst_data);
}

static int get_input_event_actions(struct cafe_ep_inst *inst,
				   struct cafe_event_descr *event,
				   struct cafe_event_action_list *actions)
{
	int rc;
	struct inst_data *inst_data  = (struct inst_data *)inst->client_data;
	struct cafe_event_action_info *action;
	CAFE_ACTION_HANDLER action_handler = NULL;

	rc = cafe_ep_inst_event_action_info_get(inst, event, &action);
	if (action == NULL)
		return CAFE_ERROR;

	assert(inst_data->event_data[(unsigned long)action->impl_data].enabled);
	if (!inst_data->event_data[(unsigned long)action->impl_data].enabled)
		return CAFE_ERROR;
	if (rc != CAFE_OK)
		return rc;

	switch ((unsigned long)action->impl_data) {
	case 0:
		action_handler = semgive_action_handler;
		break;
	case 1:
		action_handler = memalloc_action_handler;
		break;
	default:
		/* coverity[array_null] */
		assert(!"unexpected value");
	}

	return cafe_add_event_action(inst, actions, action_handler, action);
}

static struct cafe_ep_spec spec = {
	EP_NAME,
	inst_create,
	inst_delete,
	NULL,			/* start */
	NULL,			/* stop */
	get_input_event_actions,
	NULL,			/* update outputs */
	NULL,			/* inject event */
	ALL_TYPES,
	WORKAREA_REF,
	CAFE_NELEMENTS(input_events),
	input_events
};

static bool test_ep_registered = false;

static void test_ep_unregister(void)
{
	if (test_ep_registered) {
		(void)cafe_ep_spec_unregister(&spec);
		test_ep_registered = false;
	}
}

static int test_ep_register(void)
{
	int rc = CAFE_OK;

	if (!test_ep_registered) {
		rc = cafe_ep_spec_register(&spec);
		if (rc == CAFE_OK)
			test_ep_registered = true;
	}

	return rc;
}

/*
 * I'm deliberately not asserting much in here: we have other tests
 * that verify instrument instantiation.
 */
static int build_instruments(void)
{
	size_t i;
	int rc = CAFE_OK;
	struct cafe_ep_spec *spec[2];

	CAFE_LOCK();

	if ((rc = test_ep_register()) != CAFE_OK)
		goto cafe_error;

	spec[0] = cafe_ep_spec_find("vxworks_kernel");
	spec[1] = cafe_ep_spec_find(EP_NAME);
	CUT_ASSERT_NOT_NULL(spec[0]);
	CUT_ASSERT_NOT_NULL(spec[1]);

	for (i = 0; i < NUM_ROOT_INSTANCES; i++) {
		static struct cafe_ep_inst *inst[2];
		struct inst_data *inst_data;

		rc = cafe_instrument_create(EP_NAME, &instrument[i]);
		if (rc != CAFE_OK)
			goto cafe_error;

		rc = cafe_ep_inst_create(instrument[i], spec[0],
					 0, NULL, &inst[0]);
		if (rc != CAFE_OK)
			goto cafe_error;

		rc = cafe_ep_inst_create(instrument[i], spec[1],
					 0, NULL, &inst[1]);
		if (rc != CAFE_OK)
			goto cafe_error;


		/* Bypass threading compatibility checks. */
		inst[0]->flags |= CAFE_EP_INST_FLAG_OUTPUT_SESSION_THREAD;
		inst_data = (struct inst_data *const)inst[1]->client_data;
		inst_data->source = inst[0];

		rc = cafe_ep_inst_connect(inst[0], inst[1]);
		if (rc != CAFE_OK)
			goto cafe_error;

		rc = cafe_instrument_refresh_output_event_actions(instrument[i]);
		if (rc != CAFE_OK)
			goto cafe_error;

		rc = cafe_instrument_start(instrument[i]);
		if (rc != CAFE_OK)
			goto cafe_error;
	}
	CUT_ASSERT_CAFE_OK(rc);
	CAFE_UNLOCK();
	return 0;

cafe_error:
	CAFE_UNLOCK();
	CUT_ASSERT_CAFE_OK(rc);
	return 1;
}

static int delete_instruments(void)
{
	size_t i;

	CAFE_LOCK();
	for (i = 0; i < NUM_ROOT_INSTANCES; i++) {
		if (instrument[i] != NULL) {
			CUT_ASSERT_CAFE_OK(cafe_instrument_delete(instrument[i]));
			instrument[i] = NULL;
		}
	}
	test_ep_unregister();
	CAFE_UNLOCK();
	return 0;
}

static void dispatch_event(int event_id, size_t nevents)
{
	size_t i;

	for (i = 0; i < nevents; i++) {
#if defined(__VXWORKS__)
		switch (event_id) {
		case EVENT_MEMALLOC: {
			void *p = memPartAlloc(mem_part_id, 1);
			(void)memPartFree(mem_part_id, p);
			break;
		}
		case EVENT_SEMGIVE:
			(void)semGive(sem_id);
			break;
		default:
			/* coverity[array_null] */
			assert(!"unexpected event_id");
		}
#else
		cafe_cfg_vxworks_ev_process_id(event_id, NULL, NULL);
#endif
	}
}

static int enable_event(struct cafe_ep_inst *inst, size_t event_index)
{
	int rc;
	struct cafe_event_descr *const type = inst->input_events[event_index];
	struct inst_data *const inst_data = (struct inst_data *const)inst->client_data;

	inst_data->event_data[event_index].enabled = 1;
	inst_data->event_data[event_index].event_count = 0;

	CAFE_LOCK();
	rc = cafe_ep_inst_enable_output_event(inst_data->source, type);
	CAFE_UNLOCK();

	return rc;
}

static int disable_event(struct cafe_ep_inst *inst, size_t event_index)
{
	int rc;
	struct cafe_event_descr *const type = inst->input_events[event_index];
	struct inst_data *const inst_data = (struct inst_data *const)inst->client_data;

	inst_data->event_data[event_index].enabled = 0;
	inst_data->event_data[event_index].event_count = 0;

	CAFE_LOCK();
	rc = cafe_ep_inst_disable_output_event(inst_data->source, type);
	CAFE_UNLOCK();

	return rc;
}

static int refresh_instance(size_t inst_index)
{
	int rc;

	CAFE_LOCK();
	rc = cafe_instrument_refresh_output_event_actions(instrument[inst_index]);
	CAFE_UNLOCK();

	return rc;
}

static int test(void)
{
	size_t i;

	/* disable all. */

	for (i = 0; i < NUM_ROOT_INSTANCES; i++) {
		CUT_ASSERT_CAFE_OK(disable_event(instance_list[i], SEMGIVE));
		CUT_ASSERT_CAFE_OK(disable_event(instance_list[i], MEMALLOC));
		CUT_ASSERT_CAFE_OK(refresh_instance(i));
	}

	dispatch_event(EVENT_SEMGIVE, 5);
	dispatch_event(EVENT_MEMALLOC, 5);

	for (i = 0; i < NUM_ROOT_INSTANCES; i++) {
		CUT_ASSERT_EQUAL(0, SEMGIVE_EVENT_COUNT(i));
		CUT_ASSERT_EQUAL(0, MEMALLOC_EVENT_COUNT(i));
	}

	/* enable all. */

	for (i = 0; i < NUM_ROOT_INSTANCES; i++) {
		CUT_ASSERT_CAFE_OK(enable_event(instance_list[i], SEMGIVE));
		CUT_ASSERT_CAFE_OK(enable_event(instance_list[i], MEMALLOC));
		CUT_ASSERT_CAFE_OK(refresh_instance(i));
	}

	dispatch_event(EVENT_SEMGIVE, 5);
	dispatch_event(EVENT_MEMALLOC, 5);

	for (i = 0; i < NUM_ROOT_INSTANCES; i++) {
		CUT_ASSERT_EQUAL(5, SEMGIVE_EVENT_COUNT(i));
		CUT_ASSERT_EQUAL(5, MEMALLOC_EVENT_COUNT(i));
	}

	/* enable MEMALLOC for even numbered instances. */

	for (i = 0; i < NUM_ROOT_INSTANCES; i++) {
		int is_even = (i % 2) == 0;

		CUT_ASSERT_CAFE_OK(disable_event(instance_list[i], SEMGIVE));
		CUT_ASSERT_CAFE_OK(disable_event(instance_list[i], MEMALLOC));
		CUT_ASSERT_CAFE_OK(refresh_instance(i));

		if (is_even) {
			CUT_ASSERT_CAFE_OK(enable_event(instance_list[i], MEMALLOC));
		}

		CUT_ASSERT_CAFE_OK(refresh_instance(i));
	}

	dispatch_event(EVENT_SEMGIVE, 5);
	dispatch_event(EVENT_MEMALLOC, 5);

	for (i = 0; i < NUM_ROOT_INSTANCES; i++) {
		int expected = (i % 2) == 0 ? 5 : 0;
		CUT_ASSERT_EQUAL(0, SEMGIVE_EVENT_COUNT(i));
		CUT_ASSERT_EQUAL(expected, MEMALLOC_EVENT_COUNT(i));
	}

	for (i = NUM_ROOT_INSTANCES / 2; i < NUM_ROOT_INSTANCES; i++) {
		CUT_ASSERT_CAFE_OK(disable_event(instance_list[i], SEMGIVE));
		CUT_ASSERT_CAFE_OK(disable_event(instance_list[i], MEMALLOC));
		CUT_ASSERT_CAFE_OK(refresh_instance(i));
	}

	for (i = 0; i < NUM_ROOT_INSTANCES / 2; i++) {
		CUT_ASSERT_CAFE_OK(disable_event(instance_list[i], SEMGIVE));
		CUT_ASSERT_CAFE_OK(enable_event(instance_list[i], MEMALLOC));
		CUT_ASSERT_CAFE_OK(refresh_instance(i));
	}

	dispatch_event(EVENT_SEMGIVE, 5);
	dispatch_event(EVENT_MEMALLOC, 5);

	for (i = 0; i < NUM_ROOT_INSTANCES; i++) {
		if (i >= NUM_ROOT_INSTANCES / 2) {
			CUT_ASSERT_EQUAL(0, SEMGIVE_EVENT_COUNT(i));
			CUT_ASSERT_EQUAL(0, MEMALLOC_EVENT_COUNT(i));
		} else {
			CUT_ASSERT_EQUAL(0, SEMGIVE_EVENT_COUNT(i));
			CUT_ASSERT_EQUAL(5, MEMALLOC_EVENT_COUNT(i));
		}
	}

	for (i = 0; i < NUM_ROOT_INSTANCES / 2; i++) {
		CUT_ASSERT_CAFE_OK(enable_event(instance_list[i], SEMGIVE));
		CUT_ASSERT_CAFE_OK(enable_event(instance_list[i], MEMALLOC));
		CUT_ASSERT_CAFE_OK(refresh_instance(i));
	}

	dispatch_event(EVENT_SEMGIVE, 5);
	dispatch_event(EVENT_MEMALLOC, 5);

	for (i = 0; i < NUM_ROOT_INSTANCES; i++) {
		if (i >= NUM_ROOT_INSTANCES / 2) {
			/* coverity[copy_paste_error] */
			CUT_ASSERT_EQUAL(0, SEMGIVE_EVENT_COUNT(i));
			CUT_ASSERT_EQUAL(0, MEMALLOC_EVENT_COUNT(i));
		} else {
			CUT_ASSERT_EQUAL(5, SEMGIVE_EVENT_COUNT(i));
			CUT_ASSERT_EQUAL(5, MEMALLOC_EVENT_COUNT(i));
		}
	}

	return 0;
}

static void setup(void)
{
	CAFE_LOCK();
	(void)cafe_cfg_vxworks_kernel_init_cafe();
	CAFE_UNLOCK();
#if defined(__VXWORKS__)
	pool = malloc(poolsize);
	assert(pool != NULL);
	mem_part_id = memPartCreate(pool, poolsize);
	assert(mem_part_id != NULL);
	sem_id = semBCreate(0, SEM_EMPTY);
	assert(sem_id != NULL);
#endif
}

static void teardown(void)
{
#if defined(__VXWORKS__)
	(void)memPartDelete(mem_part_id);
	free(pool);
	mem_part_id = NULL;
	(void)semDelete(sem_id);
	sem_id = NULL;
#endif
}

CUT_BEGIN_SUITE_F(vxw_disable_event_suite, setup, teardown)
	CUT_RUN_TEST(build_instruments);
	CUT_RUN_TEST(test);
	CUT_RUN_TEST(delete_instruments);
CUT_END_SUITE_F
