/* vxworks_eventtest.c - connect to a vxWorks instance and test its events */

/* Copyright (c) 2010-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
08jul14,sdt  Used CUT_PRINTF/CUT_PRINT_ERR macro to make clear output
30jun14,sdt  Used atomic variable to handle test_enabled flag
             Add support of PPC decrementer int enter - WB4-3797
11mar14,sdt  Fix 'Checked Return' warnings

DESCRIPTION
*/

#include <stdlib.h>
#include <assert.h>
#include <cafe/utils.h>
#include <cafe/value_type.h>

#include "../vxworks_events.h"
#include "vxworks_eventtest.h"
#include <test/cafe_CUnitTest.h>

static struct cafe_ep_spec *vxworks_spec;
static struct cafe_ep_inst *vxworks_inst;

static struct cafe_ep_spec *match_spec;
static struct cafe_ep_inst *match_inst;

#define EVENTS_TEST_TYPES ""

static struct cafe_instrument *session;


/* action handler only does anything if this is non-zero */
TEST_ENA_T test_enabled = 0;

/* filter for event types */
struct cafe_value_type *expected_type = NULL;

const char *taskDelay_TYPE[] = {
	vxWorks_taskDelay_VT_REF, cafe_timestamp_VT_REF};
const char *taskSpawn_TYPE[] = {
	vxWorks_taskSpawn_VT_REF, cafe_timestamp_VT_REF};
const char *memAlloc_TYPE[] = {
	vxWorks_memAlloc_VT_REF, cafe_timestamp_VT_REF};
const char *memFree_TYPE[] = {
	vxWorks_memFree_VT_REF, cafe_timestamp_VT_REF};
const char *memRealloc_TYPE[] = {
	vxWorks_memRealloc_VT_REF, cafe_timestamp_VT_REF};
const char *memPartCreate_TYPE[] = {
	vxWorks_memPartCreate_VT_REF, cafe_timestamp_VT_REF};

const char *intEnter_TYPE[] = {
	vxWorks_intEnter_VT_REF, cafe_timestamp_VT_REF};

#if defined(__VXWORKS__) && (_VX_CPU_FAMILY == _VX_PPC)
const char *intEnterPpcDec_TYPE[] = {
	vxWorks_intEnterPpcDec_VT_REF, cafe_timestamp_VT_REF};
#endif

const char *intExit_TYPE[] = {
	vxWorks_intExit_VT_REF, cafe_timestamp_VT_REF};
const char *intExitK_TYPE[] = {
	vxWorks_intExitK_VT_REF, cafe_timestamp_VT_REF};
const char *tickAnnounce_TYPE[] = {
	vxWorks_tickAnnounce_VT_REF, cafe_timestamp_VT_REF};
const char *taskLock_TYPE[] = {
	vxWorks_taskLock_VT_REF, cafe_timestamp_VT_REF};

#if defined(__VXWORKS__) && ((_WRS_VXWORKS_MAJOR > 6) || \
	((_WRS_VXWORKS_MAJOR == 6) && (_WRS_VXWORKS_MINOR > 8)) || \
    ((_WRS_VXWORKS_MAJOR == 6) && (_WRS_VXWORKS_MINOR == 8) && (_WRS_VXWORKS_MAINT >= 2)))
const char *taskUnlockZero_TYPE[] = {
	vxWorks_taskUnlockZero_VT_REF, cafe_timestamp_VT_REF};
const char *taskUnlockNonZero_TYPE[] = {
	vxWorks_taskUnlockNonZero_VT_REF, cafe_timestamp_VT_REF};
#else
const char *taskUnlock_TYPE[] = {
	vxWorks_taskUnlock_VT_REF, cafe_timestamp_VT_REF};
#endif
const char *windExitDispatch_TYPE[] = {
	vxWorks_windExitDispatch_VT_REF, cafe_timestamp_VT_REF};
const char *windExitDispatchPI_TYPE[] = {
	vxWorks_windExitDispatchPI_VT_REF, cafe_timestamp_VT_REF};
const char *windExitIdle_TYPE[] = {
	vxWorks_windExitIdle_VT_REF, cafe_timestamp_VT_REF};
const char *windDelete_TYPE[] = {
	vxWorks_windDelete_VT_REF, cafe_timestamp_VT_REF};
const char *taskName_TYPE[] = {
	vxWorks_taskName_VT_REF};
const char *timestampConfig_TYPE[] = {
	cafe_timestamp_config_VT_REF};
const char *numCores_TYPE[] = {
	cafe_num_cores_VT_REF};
const char *ioCreat_TYPE[] = {
	vxWorks_ioCreat_VT_REF, cafe_timestamp_VT_REF};
const char *ioOpen_TYPE[] = {
	vxWorks_ioOpen_VT_REF, cafe_timestamp_VT_REF};
const char *ioClose_TYPE[] = {
	vxWorks_ioClose_VT_REF, cafe_timestamp_VT_REF};
const char *ioRead_TYPE[] = {
	vxWorks_ioRead_VT_REF, cafe_timestamp_VT_REF};
const char *ioWrite_TYPE[] = {
	vxWorks_ioWrite_VT_REF, cafe_timestamp_VT_REF};
const char *ioIoctl_TYPE[] = {
	vxWorks_ioIoctl_VT_REF, cafe_timestamp_VT_REF};
const char *ioDelete_TYPE[] = {
	vxWorks_ioDelete_VT_REF, cafe_timestamp_VT_REF};
const char *windPrioritySetRaise_TYPE[] = {
	vxWorks_windPrioritySetRaise_VT_REF, cafe_timestamp_VT_REF};
const char *windPrioritySetLower_TYPE[] = {
	vxWorks_windPrioritySetLower_VT_REF, cafe_timestamp_VT_REF};
const char *semBCreate_TYPE[] = {
	vxWorks_semBCreate_VT_REF, cafe_timestamp_VT_REF};
const char *semCCreate_TYPE[] = {
	vxWorks_semCCreate_VT_REF, cafe_timestamp_VT_REF};
const char *semMCreate_TYPE[] = {
	vxWorks_semMCreate_VT_REF, cafe_timestamp_VT_REF};
const char *semDelete_TYPE[] = {
	vxWorks_semDelete_VT_REF, cafe_timestamp_VT_REF};
const char *semGive_TYPE[] = {
	vxWorks_semGive_VT_REF, cafe_timestamp_VT_REF};
const char *semTake_TYPE[] = {
	vxWorks_semTake_VT_REF, cafe_timestamp_VT_REF};
const char *semFlush_TYPE[] = {
	vxWorks_semFlush_VT_REF, cafe_timestamp_VT_REF};
const char *msgQSend_TYPE[] = {
	vxWorks_msgQSend_VT_REF, cafe_timestamp_VT_REF};
const char *msgQReceive_TYPE[] = {
	vxWorks_msgQRecv_VT_REF, cafe_timestamp_VT_REF};
const char *msgQCreate_TYPE[] = {
	vxWorks_msgQCreate_VT_REF, cafe_timestamp_VT_REF};
const char *msgQDelete_TYPE[] = {
	vxWorks_msgQDelete_VT_REF, cafe_timestamp_VT_REF};
const char *wdCreate_TYPE[] = {
	vxWorks_wdCreate_VT_REF, cafe_timestamp_VT_REF};
const char *wdDelete_TYPE[] = {
	vxWorks_wdDelete_VT_REF, cafe_timestamp_VT_REF};
const char *wdStart_TYPE[] = {
	vxWorks_wdStart_VT_REF, cafe_timestamp_VT_REF};
const char *wdCancel_TYPE[] = {
	vxWorks_wdCancel_VT_REF, cafe_timestamp_VT_REF};
const char *objSemGive_TYPE[] = {
	vxWorks_objSemGive_VT_REF, cafe_timestamp_VT_REF};
const char *objSemTake_TYPE[] = {
	vxWorks_objSemTake_VT_REF, cafe_timestamp_VT_REF};
const char *objSemFlush_TYPE[] = {
	vxWorks_objSemFlush_VT_REF, cafe_timestamp_VT_REF};
const char *objMsgSend_TYPE[] = {
	vxWorks_objMsgSend_VT_REF, cafe_timestamp_VT_REF};
const char *objMsgReceive_TYPE[] = {
	vxWorks_objMsgReceive_VT_REF, cafe_timestamp_VT_REF};
const char *objMsgDelete_TYPE[] = {
	vxWorks_objMsgDelete_VT_REF, cafe_timestamp_VT_REF};
const char *objSigPause_TYPE[] = {
	vxWorks_objSigPause_VT_REF, cafe_timestamp_VT_REF};
const char *objSigSuspend_TYPE[] = {
	vxWorks_objSigSuspend_VT_REF, cafe_timestamp_VT_REF};
const char *objSigKill_TYPE[] = {
	vxWorks_objSigKill_VT_REF, cafe_timestamp_VT_REF};
const char *windStop_TYPE[] = {
	vxWorks_windStop_VT_REF, cafe_timestamp_VT_REF};
const char *windCont_TYPE[] = {
	vxWorks_windCont_VT_REF, cafe_timestamp_VT_REF};
const char *windTickTimeout_TYPE[] = {
	vxWorks_windTickTimeout_VT_REF, cafe_timestamp_VT_REF};
const char *objSigWait_TYPE[] = {
	vxWorks_objSigWait_VT_REF, cafe_timestamp_VT_REF};
const char *objEventSend_TYPE[] = {
	vxWorks_objEventSend_VT_REF, cafe_timestamp_VT_REF};
const char *objEventReceive_TYPE[] = {
	vxWorks_objEventReceive_VT_REF, cafe_timestamp_VT_REF};
const char *objWindRotate_TYPE[] = {
	vxWorks_objWindRotate_VT_REF, cafe_timestamp_VT_REF};
const char *objSemRTake_TYPE[] = {
	vxWorks_objSemRTake_VT_REF, cafe_timestamp_VT_REF};
const char *objSemWTake_TYPE[] = {
	vxWorks_objSemWTake_VT_REF, cafe_timestamp_VT_REF};
const char *taskDestroy_TYPE[] = {
	vxWorks_taskDestroy_VT_REF, cafe_timestamp_VT_REF};
const char *taskPrioritySet_TYPE[] = {
	vxWorks_taskPrioritySet_VT_REF, cafe_timestamp_VT_REF};
const char *taskSuspend_TYPE[] = {
	vxWorks_taskSuspend_VT_REF, cafe_timestamp_VT_REF};
const char *taskResume_TYPE[] = {
	vxWorks_taskResume_VT_REF, cafe_timestamp_VT_REF};
const char *taskSafe_TYPE[] = {
	vxWorks_taskSafe_VT_REF, cafe_timestamp_VT_REF};
const char *taskUnsafe_TYPE[] = {
	vxWorks_taskUnsafe_VT_REF, cafe_timestamp_VT_REF};
const char *semMGiveForce_TYPE[] = {
	vxWorks_semMGiveForce_VT_REF, cafe_timestamp_VT_REF};
const char *signal_TYPE[] = {
	vxWorks_signal_VT_REF, cafe_timestamp_VT_REF};
const char *sigSuspend_TYPE[] = {
	vxWorks_sigSuspend_VT_REF, cafe_timestamp_VT_REF};
const char *pause_TYPE[] = {
	vxWorks_pause_VT_REF, cafe_timestamp_VT_REF};
const char *kill_TYPE[] = {
	vxWorks_kill_VT_REF, cafe_timestamp_VT_REF};
const char *sigWrapper_TYPE[] = {
	vxWorks_sigWrapper_VT_REF, cafe_timestamp_VT_REF};
const char *memAddToPool_TYPE[] = {
	vxWorks_memAddToPool_VT_REF, cafe_timestamp_VT_REF};
const char *taskStop_TYPE[] = {
	vxWorks_taskStop_VT_REF, cafe_timestamp_VT_REF};
const char *taskCont_TYPE[] = {
	vxWorks_taskCont_VT_REF, cafe_timestamp_VT_REF};
const char *eventSend_TYPE[] = {
	vxWorks_eventSend_VT_REF, cafe_timestamp_VT_REF};
const char *eventReceive_TYPE[] = {
	vxWorks_eventReceive_VT_REF, cafe_timestamp_VT_REF};
const char *objOwnerSet_TYPE[] = {
	vxWorks_objOwnerSet_VT_REF, cafe_timestamp_VT_REF};
const char *isrDispatcherInvoke_TYPE[] = {
	vxWorks_isrDispatcherInvoke_VT_REF, cafe_timestamp_VT_REF};
const char *isrDispatcherDone_TYPE[] = {
	vxWorks_isrDispatcherDone_VT_REF, cafe_timestamp_VT_REF};
const char *isrCreate_TYPE[] = {
	vxWorks_isrCreate_VT_REF, cafe_timestamp_VT_REF};
const char *isrDestroy_TYPE[] = {
	vxWorks_isrDestroy_VT_REF, cafe_timestamp_VT_REF};
const char *syscallReturnOk_TYPE[] = {
	vxWorks_syscallReturnOk_VT_REF, cafe_timestamp_VT_REF};
const char *syscallReturnError_TYPE[] = {
	vxWorks_syscallReturnError_VT_REF, cafe_timestamp_VT_REF};
const char *syscallBadId_TYPE[] = {
	vxWorks_syscallBadId_VT_REF, cafe_timestamp_VT_REF};
const char *rtpSpawn_TYPE[] = {
	vxWorks_rtpSpawn_VT_REF, cafe_timestamp_VT_REF};
const char *rtpDelete_TYPE[] = {
	vxWorks_rtpDelete_VT_REF, cafe_timestamp_VT_REF};
const char *rtpInitTask_TYPE[] = {
	vxWorks_rtpInitTask_VT_REF, cafe_timestamp_VT_REF};
const char *rtpLoad_TYPE[] = {
	vxWorks_rtpLoad_VT_REF, cafe_timestamp_VT_REF};
const char *edrErrorInject_TYPE[] = {
	vxWorks_edrErrorInject_VT_REF, cafe_timestamp_VT_REF};
const char *rtpKill_TYPE[] = {
	vxWorks_rtpKill_VT_REF, cafe_timestamp_VT_REF};
const char *rtpSignal_TYPE[] = {
	vxWorks_rtpSignal_VT_REF, cafe_timestamp_VT_REF};
const char *rtpSuspend_TYPE[] = {
	vxWorks_rtpSuspend_VT_REF, cafe_timestamp_VT_REF};
const char *rtpPause_TYPE[] = {
	vxWorks_rtpPause_VT_REF, cafe_timestamp_VT_REF};
const char *rtpTaskKill_TYPE[] = {
	vxWorks_rtpTaskKill_VT_REF, cafe_timestamp_VT_REF};
const char *rtpSigQueue_TYPE[] = {
	vxWorks_rtpSigQueue_VT_REF, cafe_timestamp_VT_REF};
const char *rtpTaskSigQueue_TYPE[] = {
	vxWorks_rtpTaskSigQueue_VT_REF, cafe_timestamp_VT_REF};
const char *rtpChildWait_TYPE[] = {
	vxWorks_rtpChildWait_VT_REF, cafe_timestamp_VT_REF};
const char *rtpSigTimedWait_TYPE[] = {
	vxWorks_rtpSigTimedWait_VT_REF, cafe_timestamp_VT_REF};
const char *rtpSigWrapper_TYPE[] = {
	vxWorks_rtpSigWrapper_VT_REF, cafe_timestamp_VT_REF};
const char *rtpSigWrapperDone_TYPE[] = {
	vxWorks_rtpSigWrapperDone_VT_REF, cafe_timestamp_VT_REF};
const char *rtpSigProcMask_TYPE[] = {
	vxWorks_rtpSigProcMask_VT_REF, cafe_timestamp_VT_REF};
const char *rtpSigPending_TYPE[] = {
	vxWorks_rtpSigPending_VT_REF, cafe_timestamp_VT_REF};
const char *taskRotate_TYPE[] = {
	vxWorks_taskRotate_VT_REF, cafe_timestamp_VT_REF};
const char *sdCreate_TYPE[] = {
	vxWorks_sdCreate_VT_REF, cafe_timestamp_VT_REF};
const char *sdDelete_TYPE[] = {
	vxWorks_sdDelete_VT_REF, cafe_timestamp_VT_REF};
const char *sdMap_TYPE[] = {
	vxWorks_sdMap_VT_REF, cafe_timestamp_VT_REF};
const char *sdUnmap_TYPE[] = {
	vxWorks_sdUnmap_VT_REF, cafe_timestamp_VT_REF};
const char *semRWCreate_TYPE[] = {
	vxWorks_semRWCreate_VT_REF, cafe_timestamp_VT_REF};
const char *semRTake_TYPE[] = {
	vxWorks_semRTake_VT_REF, cafe_timestamp_VT_REF};
const char *semWTake_TYPE[] = {
	vxWorks_semWTake_VT_REF, cafe_timestamp_VT_REF};
const char *taskCpuAffinitySet_TYPE[] = {
	vxWorks_taskCpuAffinitySet_VT_REF, cafe_timestamp_VT_REF};
const char *dprintf_TYPE[] = {
	vxWorks_dprintf_VT_REF, cafe_timestamp_VT_REF};
const char *timerRollover_TYPE[] = {
	vxWorks_timerRollover_VT_REF, cafe_timestamp_VT_REF};
const char *msgQRecv_TYPE[] = {
	vxWorks_msgQRecv_VT_REF, cafe_timestamp_VT_REF};
const char *isrInfo_TYPE[] = {
	vxWorks_isrInfo_VT_REF, cafe_timestamp_VT_REF};
const char *rtpName_TYPE[] = {
	vxWorks_rtpName_VT_REF, cafe_timestamp_VT_REF};
const char *exception_TYPE[] = {
	vxWorks_exception_VT_REF, cafe_timestamp_VT_REF};
const char *windSpawn_TYPE[] = {
	vxWorks_windSpawn_VT_REF, cafe_timestamp_VT_REF};
const char *windSuspend_TYPE[] = {
	vxWorks_windSuspend_VT_REF, cafe_timestamp_VT_REF};
const char *windResume_TYPE[] = {
	vxWorks_windResume_VT_REF, cafe_timestamp_VT_REF};
const char *windSemDelete_TYPE[] = {
	vxWorks_windSemDelete_VT_REF, cafe_timestamp_VT_REF};
const char *windTickAnnounceTmrSlc_TYPE[] = {
	vxWorks_windTickAnnounceTmrSlc_VT_REF, cafe_timestamp_VT_REF};
const char *windTickAnnounceTmrWd_TYPE[] = {
	vxWorks_windTickAnnounceTmrWd_VT_REF, cafe_timestamp_VT_REF};
const char *windDelay_TYPE[] = {
	vxWorks_windDelay_VT_REF, cafe_timestamp_VT_REF};
const char *windUndelay_TYPE[] = {
	vxWorks_windUndelay_VT_REF, cafe_timestamp_VT_REF};
const char *windWdStart_TYPE[] = {
	vxWorks_windWdStart_VT_REF, cafe_timestamp_VT_REF};
const char *windWdCancel_TYPE[] = {
	vxWorks_windWdCancel_VT_REF, cafe_timestamp_VT_REF};
const char *windPendQGet_TYPE[] = {
	vxWorks_windPendQGet_VT_REF, cafe_timestamp_VT_REF};
const char *windPendQFlush_TYPE[] = {
	vxWorks_windPendQFlush_VT_REF, cafe_timestamp_VT_REF};
const char *windPendQPut_TYPE[] = {
	vxWorks_windPendQPut_VT_REF, cafe_timestamp_VT_REF};
const char *windPendQTerminate_TYPE[] = {
	vxWorks_windPendQTerminate_VT_REF, cafe_timestamp_VT_REF};
const char *windTickUndelay_TYPE[] = {
	vxWorks_windTickUndelay_VT_REF, cafe_timestamp_VT_REF};
const char *moduleLoad_TYPE[] = {
	vxWorks_moduleLoad_VT_REF, cafe_timestamp_VT_REF};
const char *moduleUnload_TYPE[] = {
	vxWorks_moduleUnload_VT_REF, cafe_timestamp_VT_REF};
const char *rtpUnload_TYPE[] = {
	vxWorks_rtpUnload_VT_REF, cafe_timestamp_VT_REF};




struct cafe_ep_spec_input_evt input_events[] = {
	CAFE_INPEV_INIT(objSemGive),

    CAFE_INPEV_INIT(taskLock),
#if defined(__VXWORKS__) && ((_WRS_VXWORKS_MAJOR > 6) || \
	((_WRS_VXWORKS_MAJOR == 6) && (_WRS_VXWORKS_MINOR > 8)) || \
    ((_WRS_VXWORKS_MAJOR == 6) && (_WRS_VXWORKS_MINOR == 8) && (_WRS_VXWORKS_MAINT >= 2)))
	CAFE_INPEV_INIT(taskUnlockZero),
	CAFE_INPEV_INIT(taskUnlockNonZero),
#else
 	CAFE_INPEV_INIT(taskUnlock),
#endif

	CAFE_INPEV_INIT(dprintf),
	CAFE_INPEV_INIT(edrErrorInject),
	CAFE_INPEV_INIT(eventReceive),
	CAFE_INPEV_INIT(eventSend),
	CAFE_INPEV_INIT(exception),
	CAFE_INPEV_INIT(intEnter),
#if defined(__VXWORKS__) && (_VX_CPU_FAMILY == _VX_PPC)
	CAFE_INPEV_INIT(intEnterPpcDec),
#endif
	CAFE_INPEV_INIT(intExit),
	CAFE_INPEV_INIT(intExitK),
	CAFE_INPEV_INIT(ioClose),
	CAFE_INPEV_INIT(ioCreat),
	CAFE_INPEV_INIT(ioDelete),
	CAFE_INPEV_INIT(ioIoctl),
	CAFE_INPEV_INIT(ioOpen),
	CAFE_INPEV_INIT(ioRead),
	CAFE_INPEV_INIT(ioWrite),
	CAFE_INPEV_INIT(isrCreate),
	CAFE_INPEV_INIT(isrDestroy),
	CAFE_INPEV_INIT(isrDispatcherDone),
	CAFE_INPEV_INIT(isrDispatcherInvoke),
	CAFE_INPEV_INIT(isrInfo),
	CAFE_INPEV_INIT(kill),
	CAFE_INPEV_INIT(memAddToPool),
	CAFE_INPEV_INIT(memAlloc),
	CAFE_INPEV_INIT(memFree),
	CAFE_INPEV_INIT(memPartCreate),
	CAFE_INPEV_INIT(memRealloc),
	CAFE_INPEV_INIT(msgQCreate),
	CAFE_INPEV_INIT(msgQDelete),
	CAFE_INPEV_INIT(msgQReceive),
	CAFE_INPEV_INIT(msgQRecv),
	CAFE_INPEV_INIT(msgQSend),
	CAFE_INPEV_INIT(numCores),
	CAFE_INPEV_INIT(objEventReceive),
	CAFE_INPEV_INIT(objEventSend),
	CAFE_INPEV_INIT(objMsgDelete),
	CAFE_INPEV_INIT(objMsgReceive),
	CAFE_INPEV_INIT(objMsgSend),
	CAFE_INPEV_INIT(objOwnerSet),
	CAFE_INPEV_INIT(objSemFlush),
	CAFE_INPEV_INIT(objSemRTake),
	CAFE_INPEV_INIT(objSemTake),
	CAFE_INPEV_INIT(objSemWTake),
	CAFE_INPEV_INIT(objSigKill),
	CAFE_INPEV_INIT(objSigPause),
	CAFE_INPEV_INIT(objSigSuspend),
	CAFE_INPEV_INIT(objSigWait),
	CAFE_INPEV_INIT(objWindRotate),
	CAFE_INPEV_INIT(pause),
	CAFE_INPEV_INIT(rtpChildWait),
	CAFE_INPEV_INIT(rtpDelete),
	CAFE_INPEV_INIT(rtpInitTask),
	CAFE_INPEV_INIT(rtpKill),
	CAFE_INPEV_INIT(rtpLoad),
	CAFE_INPEV_INIT(rtpName),
	CAFE_INPEV_INIT(rtpPause),
	CAFE_INPEV_INIT(rtpSigPending),
	CAFE_INPEV_INIT(rtpSigProcMask),
	CAFE_INPEV_INIT(rtpSigQueue),
	CAFE_INPEV_INIT(rtpSigTimedWait),
	CAFE_INPEV_INIT(rtpSigWrapper),
	CAFE_INPEV_INIT(rtpSigWrapperDone),
	CAFE_INPEV_INIT(rtpSignal),
	CAFE_INPEV_INIT(rtpSpawn),
	CAFE_INPEV_INIT(rtpSuspend),
	CAFE_INPEV_INIT(rtpTaskKill),
	CAFE_INPEV_INIT(rtpTaskSigQueue),
	CAFE_INPEV_INIT(sdCreate),
	CAFE_INPEV_INIT(sdDelete),
	CAFE_INPEV_INIT(sdMap),
	CAFE_INPEV_INIT(sdUnmap),
	CAFE_INPEV_INIT(semBCreate),
	CAFE_INPEV_INIT(semCCreate),
	CAFE_INPEV_INIT(semDelete),
	CAFE_INPEV_INIT(semFlush),
	CAFE_INPEV_INIT(semGive),
	CAFE_INPEV_INIT(semMCreate),
	CAFE_INPEV_INIT(semMGiveForce),
	CAFE_INPEV_INIT(semRTake),
	CAFE_INPEV_INIT(semRWCreate),
	CAFE_INPEV_INIT(semTake),
	CAFE_INPEV_INIT(semWTake),
	CAFE_INPEV_INIT(sigSuspend),
	CAFE_INPEV_INIT(sigWrapper),
	CAFE_INPEV_INIT(signal),
	CAFE_INPEV_INIT(syscallBadId),
	CAFE_INPEV_INIT(syscallReturnError),
	CAFE_INPEV_INIT(syscallReturnOk),
	CAFE_INPEV_INIT(taskCont),
	CAFE_INPEV_INIT(taskCpuAffinitySet),
	CAFE_INPEV_INIT(taskDelay),
	CAFE_INPEV_INIT(taskDestroy),
	CAFE_INPEV_INIT(taskName),
	CAFE_INPEV_INIT(taskPrioritySet),
	CAFE_INPEV_INIT(taskResume),
	CAFE_INPEV_INIT(taskRotate),
	CAFE_INPEV_INIT(taskSafe),
	CAFE_INPEV_INIT(taskSpawn),
	CAFE_INPEV_INIT(taskStop),
	CAFE_INPEV_INIT(taskSuspend),
	CAFE_INPEV_INIT(taskUnsafe),
	CAFE_INPEV_INIT(tickAnnounce),
	CAFE_INPEV_INIT(timerRollover),
	CAFE_INPEV_INIT(timestampConfig),
	CAFE_INPEV_INIT(wdCancel),
	CAFE_INPEV_INIT(wdCreate),
	CAFE_INPEV_INIT(wdDelete),
	CAFE_INPEV_INIT(wdStart),
	CAFE_INPEV_INIT(wdStart),
	CAFE_INPEV_INIT(windCont),
	CAFE_INPEV_INIT(windDelay),
	CAFE_INPEV_INIT(windDelete),
	CAFE_INPEV_INIT(windExitDispatch),
	CAFE_INPEV_INIT(windExitDispatchPI),
	CAFE_INPEV_INIT(windExitIdle),
	CAFE_INPEV_INIT(windPendQFlush),
	CAFE_INPEV_INIT(windPendQGet),
	CAFE_INPEV_INIT(windPendQPut),
	CAFE_INPEV_INIT(windPendQTerminate),
	CAFE_INPEV_INIT(windPrioritySetLower),
	CAFE_INPEV_INIT(windPrioritySetRaise),
	CAFE_INPEV_INIT(windResume),
	CAFE_INPEV_INIT(windSemDelete),
	CAFE_INPEV_INIT(windSpawn),
	CAFE_INPEV_INIT(windStop),
	CAFE_INPEV_INIT(windSuspend),
	CAFE_INPEV_INIT(windTickAnnounceTmrSlc),
	CAFE_INPEV_INIT(windTickAnnounceTmrWd),
	CAFE_INPEV_INIT(windTickTimeout),
	CAFE_INPEV_INIT(windTickUndelay),
	CAFE_INPEV_INIT(windUndelay),
	CAFE_INPEV_INIT(windWdCancel),
	CAFE_INPEV_INIT(windWdStart),
	CAFE_INPEV_INIT(moduleLoad),
	CAFE_INPEV_INIT(moduleUnload),
	CAFE_INPEV_INIT(rtpUnload),
};


static void vx_eventtest_action_handler(struct cafe_event_action_info *event_info,
					void **event_data);

static int vx_eventtest_get_input_actions(struct cafe_ep_inst *inst,
					  struct cafe_event_descr *event,
					  struct cafe_event_action_list *actions);

static struct cafe_ep_spec spec = {
	"event_match",
	NULL,		/* create */
	NULL,		/* delete */
	NULL,		/* start */
	NULL,		/* stop */
	vx_eventtest_get_input_actions,
	NULL,           /* update_output_event_actions */
	NULL,
	cafe_input_any_ctx_VT_DEF
	EVENTS_TEST_TYPES,
	"S[event_match,," cafe_input_any_ctx_VT_NAT "]",	/* workarea */
	CAFE_NELEMENTS(input_events),
	input_events,
	0,		/* num output events */
	NULL,		/* output events */
	0,		/* num output orthogonal data */
	NULL,		/* output orthogonal data */
	/* cafe_list_head */
};


unsigned int logged_event_count = 0;
struct logged_event logged_events[MAX_STORED_EVENTS];

/*
  Save selected events into simple log for later analysis
 */
static int save_event_data(struct cafe_event_action_info *event_info,
			   void **event_data)
{
	struct logged_event *e;
	unsigned int type_index;
	struct cafe_vm_attr *attr;
	char *data;

	if (logged_event_count >= MAX_STORED_EVENTS)
		return CAFE_OK;

	e = logged_events + logged_event_count;
	e->num_types = event_info->event->num_types;

	for (type_index = 0; type_index < e->num_types; type_index++) {
		e->types[type_index] = event_info->event->types[type_index];

		if (event_info->event->types[type_index]->size >
		    sizeof(e->event_data[0].u))
			return CAFE_ERROR;

		memcpy(&e->event_data[type_index], event_data[type_index],
		       event_info->event->types[type_index]->size);

		/* we've copied the source binary data, now fix up strings and
		   blobs */
		data = (char *)&e->event_data[type_index];
		cafe_type_attr_foreach(attr, event_info->event->types[type_index]) {
			struct cafe_value_type *attrtype =
				cafe_base_type(cafe_attr_type(attr));

			if (attr->kind != CAFE_ATTR_VAR &&
			    attr->kind != CAFE_ATTR_KEY)
				continue;
			if (attrtype == NULL)
				continue;

			switch(attrtype->kind) {
			case CAFE_TYPE_STRING:
			{
				char *new_ptr;
				char **arg = (char **)(data + attr->offset);


				/* special case: If ptr == NULL, don't copy string data
				   to log */
				if (*arg == NULL)
					continue;

				new_ptr = strdup(*arg);
				if (new_ptr == NULL)
					return CAFE_NOMEM;

				/* replace the original */
				*(char **)(data + attr->offset) = new_ptr;
				break;
			} /* case CAFE_TYPE_STRING */

			case CAFE_TYPE_BLOB:
			{
				struct cafe_vm_blob *event_arg;
				char *blob_data;

				event_arg = (struct cafe_vm_blob*)(data + attr->offset);

				blob_data = malloc(event_arg->l);
				if (blob_data == NULL)
					return CAFE_NOMEM;

				/* copy the blob data into the log buffer */
				memcpy(blob_data, event_arg->p, event_arg->l);

				event_arg->p = blob_data;
				break;
			} /* case CAFE_TYPE_BLOB */


			case CAFE_TYPE_STRUCT:
			case CAFE_TYPE_DICT:
				CUT_PRINTF_ERR("Can't handle type %d!\n",
				               attrtype->kind);
			default:
				break;
			}
		}
	}

	logged_event_count++;
	return CAFE_OK;
}

static void free_event_data(struct logged_event *e)
{
	unsigned int type_index;
	struct cafe_vm_attr *attr;
	char *data;

	for (type_index = 0; type_index < e->num_types; type_index++) {

		data = (char *)&e->event_data[type_index];
		cafe_type_attr_foreach(attr, e->types[type_index]) {
			struct cafe_value_type *attrtype =
				cafe_base_type(cafe_attr_type(attr));

			if (attr->kind != CAFE_ATTR_VAR &&
			    attr->kind != CAFE_ATTR_KEY)
				continue;
			if (attrtype == NULL)
				continue;

			switch(attrtype->kind) {
			case CAFE_TYPE_STRING:
			{
				char **arg = (char **)(data + attr->offset);

				/* special case: If ptr == NULL, don't copy string data
				   to log */
				if (*arg == NULL)
					continue;

				free(*arg);
				break;
			} /* case CAFE_TYPE_STRING */

			case CAFE_TYPE_BLOB:
			{
				struct cafe_vm_blob *event_arg;
				event_arg = (struct cafe_vm_blob*)(data + attr->offset);

				free(event_arg->p);
				break;
			} /* case CAFE_TYPE_BLOB */


			case CAFE_TYPE_STRUCT:
			case CAFE_TYPE_DICT:
				CUT_PRINTF_ERR("Can't handle type %d!\n",
				               attrtype->kind);
			default:
				break;
			}
		}
	}
}

static void free_all_event_data(void)
{
	unsigned int i;
	for (i = 0; i < logged_event_count; i++)
		free_event_data(&logged_events[i]);
}

void reset_logged_events(void)
{
	free_all_event_data();
	logged_event_count = 0;
}

static void vx_eventtest_action_handler(struct cafe_event_action_info *event_info,
					void **event_data)
{
	if (IS_TEST_ENABLE && (event_info->event->types[0] == expected_type))
		(void)save_event_data(event_info, event_data);
}

static int vx_eventtest_get_input_actions(struct cafe_ep_inst *inst,
					  struct cafe_event_descr *event,
					  struct cafe_event_action_list *actions)
{
	int rc;
	unsigned int i;
	struct cafe_event_action_info *event_info;
	struct cafe_value_type *type;

	/* search list of input events */

	for (i = 0; i < inst->spec->n_input_events; i++) {
		rc = cafe_type_parse(inst->session->ns,
				     inst->spec->input_events[i].type[0],
				     &type);
		if (rc != CAFE_OK)
			return CAFE_ERROR;

		if (type == event->types[0])
			break;
	}

	/* if no match found, we don't handle this event */
	if (i >= inst->spec->n_input_events)
		return CAFE_OK;
	rc = cafe_ep_inst_event_action_info_get(inst, event, &event_info);
	if (rc != CAFE_OK || event_info == NULL)
		return rc;

	return cafe_add_event_action(inst, actions, vx_eventtest_action_handler,
				     event_info);
}


static int vxworks_eventtest_setup(void)
{
	struct cafe_value_type *types[16];
	struct cafe_event_descr *event;
	unsigned int i;
	int rc;

	CAFE_LOCK();
	CUT_ASSERT_CAFE_OK(cafe_instrument_create("vxworks_test", &session));
	rc = cafe_ep_spec_register(&spec);
	if (rc != CAFE_OK)
		CUT_PRINTF_ERR("%s (%u): Error registering spec: %s\n",
		               __FILE__, __LINE__, cafe_strerror(rc));

	/* locate event source */

	vxworks_spec = cafe_ep_spec_find("vxworks_kernel");
	CUT_ASSERT(vxworks_spec != NULL);

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, vxworks_spec,
					       0, NULL, &vxworks_inst));
	CUT_ASSERT_NOT_NULL(vxworks_inst);

	match_spec = cafe_ep_spec_find("event_match");
	CUT_ASSERT(match_spec != NULL);

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session, match_spec, 0, NULL,
					       &match_inst));
	CUT_ASSERT_NOT_NULL(match_inst);

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_connect(vxworks_inst, match_inst));

	/* for each input on the match spec, request the output from vxworks */
	for (i = 0; i < match_inst->spec->n_input_events; i++) {
		unsigned int ti;
		const unsigned int num_types =
			match_inst->spec->input_events[i].num_types;
		CUT_ASSERT(num_types <= CAFE_NELEMENTS(types));

		/* parse each input type into types[] */
		for (ti = 0; ti < num_types; ti++) {
			const char *fmt = match_inst->spec->input_events[i].type[ti];
			CUT_ASSERT_CAFE_OK(cafe_type_parse(session->ns, fmt, &types[ti]));
		}
		/* request event and odata from vxworks */
		CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session, num_types, types, &event));
		CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(vxworks_inst, event));
	}

	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session));

	TEST_ENABLE_SET(0);

	CUT_ASSERT_CAFE_OK(cafe_instrument_start(session));

	CAFE_UNLOCK();

	reset_logged_events();

	return 0;
}

static int do_intExit(void)
{
	return intExit_test(session, 0);
}

static int do_intExitK(void)
{
	return intExit_test(session, 1);
}

static int do_taskLock(void)
{
	return taskLockUnlock_test(session,1);	/* taskLock  */
}

static int do_taskUnlock(void)
{
	return taskLockUnlock_test(session,0);	/* taskUnlock  */
}

static int do_windExitDispatch(void)
{
	return windExitDispatch_test(session, 0);  /* windExitDispatch */
}

static int do_windExitDispatchPI(void)
{
	return windExitDispatch_test(session, 1);  /* windExitDispatchPI */
}

static int do_taskNameAtStart(void)
{
	return taskNameAtStart_test(session, vxworks_inst);
}

static int do_timestampConfig(void)
{
	return timestampConfig_test(session, vxworks_inst);
}

static int do_numCores(void)
{
	return numCores_test(session, vxworks_inst);
}

static int do_numCoresOnEnable(void)
{
	return numCoresOnEnableDisable_test(session, vxworks_inst, 1);
}

static int do_numCoresOnDisable(void)
{
	return numCoresOnEnableDisable_test(session, vxworks_inst, 0);
}

static int do_kernelOnStart(void)
{
	return kernelOnStart_test(session, vxworks_inst);
}


#define VANILLA_TEST(name)		\
static int do_##name(void)		\
{					\
	return name##_test(session);	\
}

VANILLA_TEST(taskDelay)
VANILLA_TEST(taskSpawn)
VANILLA_TEST(memAlloc)
VANILLA_TEST(memAddToPool)
VANILLA_TEST(memRealloc)
VANILLA_TEST(memPartCreate)
VANILLA_TEST(memFree)
VANILLA_TEST(intEnter)
VANILLA_TEST(tickAnnounce)
VANILLA_TEST(windDelete)
VANILLA_TEST(windExitIdle)
VANILLA_TEST(taskName)
VANILLA_TEST(ioOpen)
VANILLA_TEST(ioCreat)
VANILLA_TEST(ioRead)
VANILLA_TEST(ioWrite)
VANILLA_TEST(ioClose)
VANILLA_TEST(ioIoctl)
VANILLA_TEST(ioDelete)
VANILLA_TEST(semBCreate)
VANILLA_TEST(semMCreate)
VANILLA_TEST(semCCreate)
VANILLA_TEST(semDelete)
VANILLA_TEST(semGive)
VANILLA_TEST(semTake)
VANILLA_TEST(semFlush)
VANILLA_TEST(wdCreate)
VANILLA_TEST(wdStart)
VANILLA_TEST(wdCancel)
VANILLA_TEST(wdDelete)
VANILLA_TEST(msgQSend)
VANILLA_TEST(msgQRecv)
VANILLA_TEST(msgQCreate)
VANILLA_TEST(msgQDelete)
VANILLA_TEST(windPrioritySetRaise)
VANILLA_TEST(windPrioritySetLower)
VANILLA_TEST(objMsgReceive)
VANILLA_TEST(objMsgSend)
VANILLA_TEST(taskSuspend)
VANILLA_TEST(windSuspend)
VANILLA_TEST(taskPrioritySet)
VANILLA_TEST(objSigKill)

VANILLA_TEST(dprintf);
VANILLA_TEST(edrErrorInject);
VANILLA_TEST(eventSend);
VANILLA_TEST(isrCreate);
VANILLA_TEST(isrDestroy);
VANILLA_TEST(isrDispatcherDone);
VANILLA_TEST(isrDispatcherInvoke);

VANILLA_TEST(sdUnmap)
VANILLA_TEST(objSemFlush)
VANILLA_TEST(rtpTaskSigQueue)
VANILLA_TEST(rtpSignal)
VANILLA_TEST(windCont)
VANILLA_TEST(windTickUndelay)
VANILLA_TEST(exception)
VANILLA_TEST(rtpLoad)
VANILLA_TEST(rtpSigQueue)
VANILLA_TEST(objWindRotate)
VANILLA_TEST(taskCpuAffinitySet)
VANILLA_TEST(windPendQTerminate)
VANILLA_TEST(sdDelete)
VANILLA_TEST(kill)
VANILLA_TEST(windWdCancel)
VANILLA_TEST(objSigWait)
VANILLA_TEST(rtpSigTimedWait)
VANILLA_TEST(objMsgDelete)
VANILLA_TEST(objEventReceive)
VANILLA_TEST(pause)
VANILLA_TEST(signal)
VANILLA_TEST(rtpSigPending)
VANILLA_TEST(rtpChildWait)
VANILLA_TEST(syscallReturnOk)
VANILLA_TEST(objOwnerSet)
VANILLA_TEST(windResume)
VANILLA_TEST(sdMap)
VANILLA_TEST(syscallReturnError)
VANILLA_TEST(eventReceive)
VANILLA_TEST(windStop)
VANILLA_TEST(taskResume)
VANILLA_TEST(windUndelay)
VANILLA_TEST(objSigSuspend)
VANILLA_TEST(taskCont)
VANILLA_TEST(windSemDelete)
VANILLA_TEST(rtpName)
VANILLA_TEST(windPendQFlush)
VANILLA_TEST(rtpSigProcMask)
VANILLA_TEST(sdCreate)
VANILLA_TEST(windSpawn)
VANILLA_TEST(sigWrapper)
VANILLA_TEST(semRTake)
VANILLA_TEST(objSigPause)
VANILLA_TEST(taskSafe)
VANILLA_TEST(rtpKill)
VANILLA_TEST(windTickTimeout)
VANILLA_TEST(sigSuspend)
VANILLA_TEST(syscallBadId)
VANILLA_TEST(rtpDelete)
VANILLA_TEST(windWdStart)
VANILLA_TEST(windTickAnnounceTmrWd)
VANILLA_TEST(taskDestroy)
VANILLA_TEST(taskStop)
VANILLA_TEST(rtpSpawn)
VANILLA_TEST(rtpSuspend)
VANILLA_TEST(rtpSigWrapperDone)
VANILLA_TEST(semWTake)
VANILLA_TEST(isrInfo)
VANILLA_TEST(semRWCreate)
VANILLA_TEST(semMGiveForce)
VANILLA_TEST(rtpTaskKill)
VANILLA_TEST(windTickAnnounceTmrSlc)
VANILLA_TEST(taskUnsafe)
VANILLA_TEST(objEventSend)
VANILLA_TEST(timerRollover)
VANILLA_TEST(rtpPause)
VANILLA_TEST(rtpInitTask)
VANILLA_TEST(windDelay)
VANILLA_TEST(rtpSigWrapper)
VANILLA_TEST(taskRotate)
VANILLA_TEST(objSemGive)
VANILLA_TEST(objSemTake)
VANILLA_TEST(objSemRTake)
VANILLA_TEST(objSemWTake)
VANILLA_TEST(moduleLoad)
VANILLA_TEST(moduleUnload)
VANILLA_TEST(rtpUnload)

static int vxworks_eventtest_teardown(void)
{
	int rc;

	CAFE_LOCK();

	reset_logged_events();

	(void)cafe_instrument_stop(session);

	/* disconnect before delete */
	rc = cafe_ep_inst_disconnect(vxworks_inst, match_inst);
	if (rc != CAFE_OK)
		goto fail;

	rc = cafe_ep_inst_delete(match_inst);
	if (rc != CAFE_OK)
		goto fail;

	rc = cafe_ep_inst_delete(vxworks_inst);
	if (rc != CAFE_OK)
		goto fail;

	rc = cafe_instrument_delete(session);
	if (rc != CAFE_OK)
		goto fail;

	rc = cafe_ep_spec_unregister(&spec);
fail:
	CAFE_UNLOCK();

	CUT_ASSERT(rc == CAFE_OK);
	return 0;
}

CUT_BEGIN_SUITE(vxw_event_suite)
	if (vxworks_eventtest_setup() != 0)
		return;
	/* dataCollectorShowAll(1); */
	CUT_RUN_TEST(do_taskDelay);
	CUT_RUN_TEST(do_taskSpawn);
	CUT_RUN_TEST(do_memAlloc);
	CUT_RUN_TEST(do_memRealloc);
	CUT_RUN_TEST(do_memFree);
	CUT_RUN_TEST(do_memFree);
	CUT_RUN_TEST(do_memPartCreate);
	CUT_RUN_TEST(do_memAddToPool);
	CUT_RUN_TEST(do_intEnter);
	CUT_RUN_TEST(do_intEnter);
	CUT_RUN_TEST(do_intExit);
	CUT_RUN_TEST(do_intExitK);
	CUT_RUN_TEST(do_tickAnnounce);
	CUT_RUN_TEST(do_taskLock);
	CUT_RUN_TEST(do_taskUnlock);
	CUT_RUN_TEST(do_windDelete);
	CUT_RUN_TEST(do_windExitDispatch);
	CUT_RUN_TEST(do_windExitDispatchPI);
	CUT_RUN_TEST(do_windExitIdle);
	CUT_RUN_TEST(do_taskName);
	CUT_RUN_TEST(do_taskNameAtStart);
	CUT_RUN_TEST(do_timestampConfig);
	CUT_RUN_TEST(do_numCores);
	CUT_RUN_TEST(do_numCoresOnEnable);
	CUT_RUN_TEST(do_numCoresOnDisable);

	CUT_fixture_teardown = ioTests_treadown;
	CUT_RUN_TEST(do_ioOpen);
	CUT_RUN_TEST(do_ioCreat);
	CUT_RUN_TEST(do_ioClose);
	CUT_RUN_TEST(do_ioRead);
	CUT_RUN_TEST(do_ioWrite);
	CUT_RUN_TEST(do_ioIoctl);
	CUT_RUN_TEST(do_ioDelete);
	CUT_fixture_teardown = NULL;

	CUT_fixture_teardown = semTests_treadown;
	CUT_RUN_TEST(do_semTake);
	CUT_RUN_TEST(do_semGive);
	CUT_RUN_TEST(do_semFlush);
	CUT_fixture_teardown = NULL;

	CUT_RUN_TEST(do_msgQSend);
	CUT_RUN_TEST(do_msgQRecv);
	CUT_RUN_TEST(do_msgQCreate);
	CUT_RUN_TEST(do_msgQDelete);
	CUT_RUN_TEST(do_semBCreate);
	CUT_RUN_TEST(do_semCCreate);
	CUT_RUN_TEST(do_semMCreate);
	CUT_RUN_TEST(do_semDelete);
	CUT_RUN_TEST(do_wdCreate);
	CUT_RUN_TEST(do_wdStart);
	CUT_RUN_TEST(do_wdCancel);
	CUT_RUN_TEST(do_wdDelete);
	CUT_RUN_TEST(do_windPrioritySetRaise);
	CUT_RUN_TEST(do_windPrioritySetLower);
	CUT_RUN_TEST(do_objSemGive);
	CUT_RUN_TEST(do_objSemTake);
	CUT_RUN_TEST(do_objMsgSend);
	CUT_RUN_TEST(do_objMsgReceive);
	CUT_RUN_TEST(do_objSemRTake);
	CUT_RUN_TEST(do_objSemWTake);
	CUT_RUN_TEST(do_windSuspend);
	CUT_RUN_TEST(do_taskSuspend);
	CUT_RUN_TEST(do_taskPrioritySet);
	CUT_RUN_TEST(do_objSigKill);
	CUT_RUN_TEST(do_dprintf);
	CUT_RUN_TEST(do_edrErrorInject);
	CUT_RUN_TEST(do_eventReceive);
	CUT_RUN_TEST(do_eventSend);
	CUT_RUN_TEST(do_exception);
	CUT_RUN_TEST(do_isrCreate);
	CUT_RUN_TEST(do_isrDestroy);
	CUT_RUN_TEST(do_isrDispatcherDone);
	CUT_RUN_TEST(do_isrDispatcherInvoke);
	CUT_RUN_TEST(do_kill);
	CUT_RUN_TEST(do_sdUnmap);
	CUT_RUN_TEST(do_objSemFlush);
	CUT_RUN_TEST(do_rtpTaskSigQueue);
	CUT_RUN_TEST(do_rtpSignal);
	CUT_RUN_TEST(do_windCont);
	CUT_RUN_TEST(do_windTickUndelay);
	CUT_RUN_TEST(do_rtpLoad);
	CUT_RUN_TEST(do_rtpSigQueue);
	CUT_RUN_TEST(do_objWindRotate);
	CUT_RUN_TEST(do_taskCpuAffinitySet);
	CUT_RUN_TEST(do_windPendQTerminate);
	CUT_RUN_TEST(do_sdDelete);
	CUT_RUN_TEST(do_windWdCancel);
	CUT_RUN_TEST(do_objSigWait);
	CUT_RUN_TEST(do_rtpSigTimedWait);
	CUT_RUN_TEST(do_objMsgDelete);
	CUT_RUN_TEST(do_objEventReceive);
	CUT_RUN_TEST(do_pause);
	CUT_RUN_TEST(do_signal);
	CUT_RUN_TEST(do_rtpSigPending);
	CUT_RUN_TEST(do_rtpChildWait);
	CUT_RUN_TEST(do_syscallReturnOk);
	CUT_RUN_TEST(do_objOwnerSet);
	CUT_RUN_TEST(do_windResume);
	CUT_RUN_TEST(do_sdMap);
	CUT_RUN_TEST(do_syscallReturnError);
	CUT_RUN_TEST(do_eventReceive);
	CUT_RUN_TEST(do_windStop);
	CUT_RUN_TEST(do_taskResume);
	CUT_RUN_TEST(do_windUndelay);
	CUT_RUN_TEST(do_objSigSuspend);
	CUT_RUN_TEST(do_taskCont);
	CUT_RUN_TEST(do_windSemDelete);
	CUT_RUN_TEST(do_rtpName);
	CUT_RUN_TEST(do_windPendQFlush);
	CUT_RUN_TEST(do_rtpSigProcMask);
	CUT_RUN_TEST(do_sdCreate);
	CUT_RUN_TEST(do_windSpawn);
	CUT_RUN_TEST(do_sigWrapper);
	CUT_RUN_TEST(do_semRTake);
	CUT_RUN_TEST(do_objSigPause);
	CUT_RUN_TEST(do_taskSafe);
	CUT_RUN_TEST(do_rtpKill);
	CUT_RUN_TEST(do_windTickTimeout);
	CUT_RUN_TEST(do_sigSuspend);
	CUT_RUN_TEST(do_syscallBadId);
	CUT_RUN_TEST(do_rtpDelete);
	CUT_RUN_TEST(do_windWdStart);
	CUT_RUN_TEST(do_eventSend);
	CUT_RUN_TEST(do_windTickAnnounceTmrWd);
	CUT_RUN_TEST(do_taskDestroy);
	CUT_RUN_TEST(do_taskStop);
	CUT_RUN_TEST(do_rtpSpawn);
	CUT_RUN_TEST(do_rtpSuspend);
	CUT_RUN_TEST(do_rtpSigWrapperDone);
	CUT_RUN_TEST(do_semWTake);
	CUT_RUN_TEST(do_isrInfo);
	CUT_RUN_TEST(do_semRWCreate);
	CUT_RUN_TEST(do_semMGiveForce);
	CUT_RUN_TEST(do_rtpTaskKill);
	CUT_RUN_TEST(do_windTickAnnounceTmrSlc);
	CUT_RUN_TEST(do_taskUnsafe);
	CUT_RUN_TEST(do_objEventSend);
	CUT_RUN_TEST(do_timerRollover);
	CUT_RUN_TEST(do_rtpPause);
	CUT_RUN_TEST(do_rtpInitTask);
	CUT_RUN_TEST(do_windDelay);
	CUT_RUN_TEST(do_rtpSigWrapper);
	CUT_RUN_TEST(do_taskRotate);
	CUT_RUN_TEST(do_moduleLoad);
	CUT_RUN_TEST(do_moduleUnload);
	CUT_RUN_TEST(do_rtpUnload);
	CUT_RUN_TEST(do_kernelOnStart);
	vxworks_eventtest_teardown();
CUT_END_SUITE
