/* vxworks_multi_inst_test.c - VxWorks test for multiple root instance */

/* Copyright (c) 2010-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
20jun14,sdt  Add missing ep_spec_unregister that prevents to run test twice
21feb14,sdt  Fix compiler warnings

DESCRIPTION

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <cafe/utils.h>
#include <assert.h>
#include <cafe/value_type.h>

#include "../vxworks_events.h"
#include "vxworks_eventtest.h"
#include <test/cafe_CUnitTest.h>
#include <cafe/ep/vxworks_kernel.h>

#ifdef __VXWORKS__
#include <private/eventP.h>
#include <taskLib.h>
#endif

#define NUM_INST_CHAINS 2

static struct cafe_allocator *allocator;

static struct cafe_ep_spec *vxworks_spec;
static struct cafe_ep_inst *vxworks_inst[NUM_INST_CHAINS];

static const char *specnames[2] = {"event_match_multitestA", "event_match_multitestB"};

static struct cafe_ep_spec *match_spec[NUM_INST_CHAINS];
static struct cafe_ep_inst *match_inst[NUM_INST_CHAINS];


static struct cafe_instrument *session[NUM_INST_CHAINS];

static const char *taskDelay_m_TYPE[] = {"t[vxWorks.taskDelay]", cafe_timestamp_VT_REF};
static const char *taskLock_m_TYPE[] = {"t[vxWorks.taskLock]", cafe_timestamp_VT_REF};


static int handlerA_count = 0;

static void action_handlerA(struct cafe_event_action_info *event_info,
			    void **event_data)
{
	handlerA_count++;
}

static int handlerB_count = 0;

static void action_handlerB(struct cafe_event_action_info *event_info,
			    void **event_data)
{
	handlerB_count++;
}

static struct cafe_ep_spec_input_evt input_eventsA[] = {
	CAFE_INPEV_INIT2(taskDelay_m, action_handlerA),
};

static struct cafe_ep_spec_input_evt input_eventsB[] = {
	CAFE_INPEV_INIT2(taskLock_m, action_handlerB),
};

static int vx_eventtest_get_input_actions(struct cafe_ep_inst *inst,
					  struct cafe_event_descr *event,
					  struct cafe_event_action_list *actions)
{
	int rc;
	struct cafe_event_action_info *event_info;
	rc = cafe_ep_inst_event_action_info_get(inst, event, &event_info);
	if (rc != CAFE_OK || event_info == NULL)
		return rc;
	return cafe_add_event_action(inst, actions, 
	                             (CAFE_ACTION_HANDLER) event_info->impl_data,
	                             event_info);
}


static struct cafe_ep_spec specA = {
	"event_match_multitestA",
	NULL,		/* create */
	NULL,		/* delete */
	NULL,		/* start */
	NULL,		/* stop */
	vx_eventtest_get_input_actions,
	NULL,           /* update_output_event_actions */
	NULL,
	cafe_input_any_ctx_VT_DEF
	"",
	"S[event_match,," cafe_input_any_ctx_VT_NAT "]", /* workarea */
	CAFE_NELEMENTS(input_eventsA),
	input_eventsA,
	0,		/* num output events */
	NULL,		/* output events */
	0,		/* num output orthogonal data */
	NULL,		/* output orthogonal data */
	/* cafe_list_head */
};

static struct cafe_ep_spec specB = {
	"event_match_multitestB",
	NULL,		/* create */
	NULL,		/* delete */
	NULL,		/* start */
	NULL,		/* stop */
	vx_eventtest_get_input_actions,
	NULL,           /* update_output_event_actions */
	NULL,
	cafe_input_any_ctx_VT_DEF
	"",
	"S[event_match,," cafe_input_any_ctx_VT_NAT "]",		/* workarea */
	CAFE_NELEMENTS(input_eventsB),
	input_eventsB,
	0,		/* num output events */
	NULL,		/* output events */
	0,		/* num output orthogonal data */
	NULL,		/* output orthogonal data */
	/* cafe_list_head */
};


static int vxworks_multiinst_setup (void)
{
	struct cafe_value_type *types[16];
	struct cafe_event_descr *event;
	unsigned int i;
	unsigned int ic;

	allocator = cafe_stdlib_allocator;

	CAFE_LOCK();
	CUT_ASSERT_CAFE_OK(cafe_ep_spec_register(&specA));
	CUT_ASSERT_CAFE_OK(cafe_ep_spec_register(&specB));

	/* locate event source */

	vxworks_spec = cafe_ep_spec_find("vxworks_kernel");
	CUT_ASSERT(vxworks_spec != NULL);

	for(ic = 0; ic < NUM_INST_CHAINS; ic++) {

		match_spec[ic] = cafe_ep_spec_find(specnames[ic]);
		CUT_ASSERT(match_spec[ic] != NULL);

		CUT_ASSERT_CAFE_OK(cafe_instrument_create("vxworks_test",
							  &session[ic]));
		CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session[ic], vxworks_spec,
						       0, NULL, &vxworks_inst[ic]));
		CUT_ASSERT_NOT_NULL(vxworks_inst[ic]);

		CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(session[ic], match_spec[ic], 0, NULL,
						       &match_inst[ic]));
		CUT_ASSERT_NOT_NULL(match_inst[ic]);
	}

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_connect(vxworks_inst[0], match_inst[0]));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_connect(vxworks_inst[1], match_inst[1]));

	for (ic = 0; ic < NUM_INST_CHAINS; ic++) {
		/* for each input on the match spec, request the output from vxworks */
		for (i = 0; i < match_inst[ic]->spec->n_input_events; i++) {
			unsigned int ti;
			const unsigned int num_types =
				match_inst[ic]->spec->input_events[i].num_types;
			CUT_ASSERT(num_types <= CAFE_NELEMENTS(types));

			/* parse each input type into types[] */
			for (ti = 0; ti < num_types; ti++) {
				const char *fmt = match_inst[ic]->spec->input_events[i].type[ti];
				CUT_ASSERT_CAFE_OK(cafe_type_parse(session[ic]->ns, fmt, &types[ti]));
			}

			/* request event and odata from vxworks */
			CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(session[ic], num_types, types, &event));
			CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(vxworks_inst[ic], event));
		}
		CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session[ic]));
	}

	for (ic = 0; ic < NUM_INST_CHAINS; ic++)
		CUT_ASSERT_CAFE_OK(cafe_instrument_start(session[ic]));
	CAFE_UNLOCK();
	return 0;
}


static int vxworks_multiinst_teardown(void)
{
	int rc = CAFE_OK;
	int rc2;
	unsigned int ic;

	CAFE_LOCK();

	for (ic = 0; ic < NUM_INST_CHAINS; ic++) {
		/* disconnect before delete */
		rc = cafe_instrument_stop(session[ic]);
		if (rc != CAFE_OK)
			goto fail;

		rc = cafe_ep_inst_disconnect(vxworks_inst[ic], match_inst[ic]);
		if (rc != CAFE_OK)
			goto fail;

		rc = cafe_ep_inst_delete(match_inst[ic]);
		if (rc != CAFE_OK)
			goto fail;

		rc = cafe_ep_inst_delete(vxworks_inst[ic]);
		if (rc != CAFE_OK)
			goto fail;

		rc = cafe_instrument_delete(session[ic]);
		if (rc != CAFE_OK)
			goto fail;
	}
fail:
	rc2 = cafe_ep_spec_unregister(&specA);
	if ((rc2 != CAFE_OK) && (rc == CAFE_OK))
	    rc = rc;
	rc2 = cafe_ep_spec_unregister(&specB);
	if ((rc2 != CAFE_OK) && (rc == CAFE_OK))
	    rc = rc;

	CAFE_UNLOCK();

	CUT_ASSERT(rc == CAFE_OK);
	return 0;
}

#ifdef __VXWORKS__
static int do_testA(void)
{
	handlerA_count = 0;
	handlerB_count = 0;

	(void)taskDelay(0);

	CUT_ASSERT(handlerA_count == 1);

	return CAFE_OK;
}
#else
static int do_testA(void)
{
	struct vxWorks_taskDelay *e;
	e = malloc(sizeof (*e));
	CUT_ASSERT_NOT_NULL(e);
	e->objId = 1234;
	e->classId = 0x8000;
	e->ticks = 10;

	handlerA_count = 0;
	handlerB_count = 0;

	cafe_cfg_vxworks_ev_process_id(EVENT_TASKDELAY, e, NULL);

	free(e);
	CUT_ASSERT(handlerA_count == 1);

	return CAFE_OK;
}
#endif

#ifdef __VXWORKS__
static int do_testB(void)
{
	handlerA_count = 0;
	handlerB_count = 0;

	(void)taskCpuLock();
	(void)taskCpuUnlock();

	CUT_ASSERT(handlerB_count == 1);
	return CAFE_OK;
}
#else
static int do_testB(void)
{
	handlerA_count = 0;
	handlerB_count = 0;

	cafe_cfg_vxworks_ev_process_id(EVENT_TASKLOCK, NULL, 0);

	CUT_ASSERT(handlerB_count == 1);
	return CAFE_OK;
}
#endif
CUT_BEGIN_SUITE(vxw_multi_inst_suite)
	(void)vxworks_multiinst_setup();

	CUT_RUN_TEST(do_testA);

	CUT_RUN_TEST(do_testB);

	(void)vxworks_multiinst_teardown();
CUT_END_SUITE
