/* Copyright (c) 2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "bailout_allocator.h"

struct allocator {
	struct cafe_allocator base;
};

static void *bailout_alloc(const struct cafe_allocator *a, size_t size)
{
	void *p;

	if ((p = malloc(size)) == NULL) {
		fputs("Out of memory!\n", stderr);
		exit(EXIT_FAILURE);
	}

	return p;
}

static void *bailout_realloc(const struct cafe_allocator *a, void *ptr, size_t size)
{
	void *p;

	if ((p	= realloc(ptr, size)) == NULL) {
		fputs("Out of memory!\n", stderr);
		exit(EXIT_FAILURE);
	}

	return p;
}

static void bailout_free(const struct cafe_allocator *a, void *ptr)
{
	if (ptr != NULL)
		free(ptr);
}

static struct allocator _bailout_allocator = {
	{ bailout_alloc, bailout_realloc, bailout_free }
};

struct cafe_allocator *bailout_allocator_create(void)
{
	return &_bailout_allocator.base;
}

void bailout_allocator_delete(const struct cafe_allocator *allocator)
{
	assert(allocator == &_bailout_allocator.base);
}

struct cafe_allocator *bailout_allocator = &_bailout_allocator.base;
