/* Copyright (c) 2011,2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <cafe/cafe.h>
#include <cafe/list.h>
#include <assert.h>
#include "compiler.h"
#include "strbuf.h"

static unsigned long dotid(struct pt_node *n)
{
	assert(n != NULL);
	return (unsigned long)n;
}

static void pt_print_dot_label(struct pt_node *node, const char *filename, FILE *stream)
{
	const char *shape = pt_is_leaf(node) ? "none" : "ellipse";

	if (filename != NULL) {
		fprintf(stream, "\t%lu [label=\"file: %s\", shape=\"note\"];\n",
			dotid(node),
			filename);
	} else {
		fprintf(stream, "\t%lu [label=\"%s\", shape=\"%s\"];\n",
			dotid(node),
			node->istr ? node->istr->str : pt_type_str(node->type),
			shape);
	}
}

static void pt_print_dot_node(struct pt_node *node, const char *filename, FILE *stream)
{
	size_t i;

	if (node == NULL)
		return;

	if (node->left) {
		fprintf(stream, "\t%lu -> %lu;\n", dotid(node), dotid(node->left));
		pt_print_dot_node(node->left, NULL, stream);
		pt_print_dot_label(node, NULL, stream);
	}
	if (node->middle) {
		fprintf(stream, "\t%lu -> %lu;\n", dotid(node), dotid(node->middle));
		pt_print_dot_node(node->middle, NULL, stream);
		pt_print_dot_label(node, NULL, stream);
	}
	if (node->right) {
		fprintf(stream, "\t%lu -> %lu;\n", dotid(node), dotid(node->right));
		pt_print_dot_node(node->right, NULL, stream);
		pt_print_dot_label(node, NULL, stream);
	}
	if (node->arg4) {
		fprintf(stream, "\t%lu -> %lu;\n", dotid(node), dotid(node->arg4));
		pt_print_dot_node(node->arg4, NULL, stream);
		pt_print_dot_label(node, NULL, stream);
	}
	if (node->arg5) {
		fprintf(stream, "\t%lu -> %lu;\n", dotid(node), dotid(node->arg5));
		pt_print_dot_node(node->arg5, NULL, stream);
		pt_print_dot_label(node, NULL, stream);
	}
	if (node->children.list != NULL) {
		for (i = 0; i < node->children.count; i++) {
			assert(node->children.list[i]);
			pt_print_dot_node(node->children.list[i], NULL, stream);
			fprintf(stream, "\t%lu -> %lu;\n", dotid(node), dotid(node->children.list[i]));
		}
	}

	pt_print_dot_label(node, filename, stream);
}

void pt_print_dot(struct cafe_type_compiler *c, FILE *stream)
{
	struct cafe_list_head *item;

	fprintf(stream, "digraph PARSETREE {\n");
	fprintf(stream, "node [shape=plaintext, fontsize=12, height=.1];\n");
	fprintf(stream, "ranksep=.3;\n");
	fprintf(stream, "edge [arrowsize=0.5]\n");

	cafe_list_foreach(item, &c->parser_list_head) {
		struct parser *p = PARSER_LIST_ENTRY(item);
		fprintf(stream, "subgraph \"%s\" {\n", p->filename);
		pt_print_dot_node(p->root, p->filename, stream);
		fprintf(stream, "}\n");
	}

	fprintf(stream, "}\n");
}
