/*
 * Copyright (c) 2011,2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <assert.h>
#include <cafe/hashtbl.h>
#include "bailout_allocator.h"
#include "istr.h"
#include "lexer.h"
#include "token.h"

static const char *progname = "lexer-test";
static int silent;

static void strtbl_delval(void *k, void *client_data)
{
	istr_free((struct istr *)k, (const struct cafe_allocator *)client_data);
}

static void usage(void)
{
	fprintf(stderr, "Usage: %s [-s] [ <INPUT> ]\n", progname);
	fprintf(stderr, "\t-s -- silent\n");
}

static int parse_opts(int argc, char *argv[])
{
	int opt;
	int ind = 1;

	while (ind < argc) {
		char *s = argv[ind];

		if (*s++ != '-')
			break;

		ind++;

		if (s[0] == '-' && s[1] == '\0')
			break;

		while ((opt = *s++) != '\0') {
			switch (opt) {
			case 's':
				silent = 1;
				break;
			case 'h':
			case '?':
			default:
				usage();
				exit(EXIT_SUCCESS);
			}
		}
	}

	return ind;
}

int main(int argc, char *argv[])
{
	struct lexer l;
	FILE *stream = stdin;
	const char *filename = "<stdin>";
	int rc;
	int filename_index;
	struct cafe_hashtbl *strtbl = NULL;

	filename_index = parse_opts(argc, argv);

	if (filename_index < argc) {
		filename = argv[filename_index];
		stream = fopen(argv[filename_index], "rb");
		if (stream == NULL) {
			fprintf(stderr, "cannot open %s: %s\n",
				argv[1], strerror(errno));
			exit(EXIT_FAILURE);
		}
	}

	strtbl = cafe_hashtbl_new(1, bailout_allocator, istr_hashtbl_hash, istr_hashtbl_equals);
	cafe_hashtbl_autoresize_enable(strtbl, 75);
	lexer_init(&l, lexer_default_error_handler, filename, stream, strtbl, bailout_allocator);

	for (;;) {
		int tkid = lexer_next_token(&l);

		if (tkid == 0 || l.lexer_errors > 0) {
			continue;
		}

		if (tkid == TK_EOF) {
			rc = EXIT_SUCCESS;
			break;
		}

		assert(tkid > 0);
		assert(l.token.id > 0);

		if (!silent) {
			fprintf(stdout, "scanner: %s: "
				"line: %d, "
				"column: %d, "
				"name: %s",
				l.token.loc.filename,
				l.token.loc.lineno,
				l.token.loc.column,
				l.token.name);
			if (l.token.istr != NULL)
				fprintf(stdout, ", value: %s", l.token.istr->str);
			fprintf(stdout, "\n");
		}
	}

	fclose(stream);
	cafe_hashtbl_delete(strtbl, NULL, NULL, strtbl_delval, bailout_allocator);
	strbuf_release(&l.sb);

	return rc;
}
