/* atomic_solaris.c - atomic ops for solaris */

/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <atomic.h>
#include <cafe/atomic.h>

cafe_atomic_val cafe_atomic_val_add(volatile cafe_atomic_val *v,
				    cafe_atomic_val delta)
{
	return atomic_add_long_nv(v, delta) - delta;
}

cafe_atomic_val cafe_atomic_val_inc(volatile cafe_atomic_val *v)
{
	return atomic_add_long_nv(v, 1) - 1;
}

cafe_atomic_val cafe_atomic_val_dec(volatile cafe_atomic_val *v)
{
	return atomic_add_long_nv(v, -1) + 1;
}

cafe_atomic_val cafe_atomic_val_sub(volatile cafe_atomic_val *v,
				    cafe_atomic_val delta)
{
	return atomic_add_long_nv(v, -delta) + delta;
}

void *cafe_atomic_swap_ptr(volatile void *volatile *target, void *newval)
{
	return atomic_swap_ptr(target, newval);
}

cafe_atomic_val cafe_atomic_get(volatile cafe_atomic_val *target)
{
	return atomic_add_long_nv(target, 0);
}

cafe_atomic_val cafe_atomic_set(volatile cafe_atomic_val *target,
				cafe_atomic_val value)
{
	return atomic_swap_ulong(target, value);
}

bool cafe_atomic_cas(volatile cafe_atomic_val *target, cafe_atomic_val oldval,
		     cafe_atomic_val newval)
{
	cafe_atomic_val old = atomic_cas_ulong(target, oldval, newval);
	return old == oldval;
}
