/* atomic_unix_shim.c - atomic operations for unix based on vxworks primitives. */

/* Copyright (c) 2010-2011 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#if defined(HAVE_CAFE_CONFIG_H)
#include <cafe/config.h>
#endif

#include <cafe/atomic.h>
#include "atomic_vxw.h"

#if SIZEOF_CAFE_ATOMIC_VAL == 8
#define vxAtomicAdd		vxAtomic64Add
#define vxAtomicAnd		vxAtomic64And
#define vxAtomicDec		vxAtomic64Dec
#define vxAtomicInc		vxAtomic64Inc
#define vxAtomicNand		vxAtomic64Nand
#define vxAtomicOr		vxAtomic64Or
#define vxAtomicSub		vxAtomic64Sub
#define vxAtomicXor		vxAtomic64Xor
#define vxAtomicClear		vxAtomic64Clear
#define vxAtomicGet		vxAtomic64Get
#define vxAtomicSet		vxAtomic64Set
#define vxAtomicCas		vxAtomic64Cas
#endif

cafe_atomic_val cafe_atomic_val_inc(volatile cafe_atomic_val *v)
{
	return vxAtomicAdd((cafe_atomic_val *)v, 1);
}

cafe_atomic_val cafe_atomic_val_add(volatile cafe_atomic_val *v,
				    cafe_atomic_val addend)
{
	return vxAtomicAdd((cafe_atomic_val *)v, addend);
}

cafe_atomic_val cafe_atomic_val_dec(volatile cafe_atomic_val *v)
{
	return vxAtomicSub((cafe_atomic_val *)v, 1);
}

cafe_atomic_val cafe_atomic_val_sub(volatile cafe_atomic_val *v,
				    cafe_atomic_val addend)
{
	return vxAtomicSub((cafe_atomic_val *)v, addend);
}

void *cafe_atomic_swap_ptr(volatile void *volatile *target, void *newval)
{
	return (void *)vxAtomicSet((cafe_atomic_val *)target,
				   (cafe_atomic_val)newval);
}

cafe_atomic_val cafe_atomic_get(volatile cafe_atomic_val *target)
{
	return vxAtomicGet(target);
}

cafe_atomic_val cafe_atomic_set(volatile cafe_atomic_val *target,
				cafe_atomic_val value)
{
	return vxAtomicSet(target, value);
}

bool cafe_atomic_cas(volatile cafe_atomic_val *target,
		     cafe_atomic_val oldval, cafe_atomic_val newval)
{
	return vxAtomicCas(target, oldval, newval);
}
