/* atomic_vxworks.c - implementation of atomic operations for vxworks. */

/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#if CAFE_OS_VXWORKS

#include <vxWorks.h>
#include <vxAtomicLib.h>
#include <intLib.h>
#include <cafe/atomic.h>
#include "cafe_assert.h"

cafe_atomic_val cafe_atomic_val_inc(volatile cafe_atomic_val *v)
{
	return vxAtomicAdd((atomic_t *)v, 1);
}

cafe_atomic_val cafe_atomic_val_add(volatile cafe_atomic_val *v,
				    cafe_atomic_val addend)
{
	return vxAtomicAdd((atomic_t *)v, addend);
}

cafe_atomic_val cafe_atomic_val_dec(volatile cafe_atomic_val *v)
{
	return vxAtomicSub((atomic_t *)v, 1);
}

cafe_atomic_val cafe_atomic_val_sub(volatile cafe_atomic_val *v,
				    cafe_atomic_val addend)
{
	return vxAtomicSub((atomic_t *)v, addend);
}

void *cafe_atomic_swap_ptr(volatile void *volatile *target, void *newval)
{
	return (void *)vxAtomicSet((atomic_t *)target, (atomicVal_t)newval);
}

cafe_atomic_val cafe_atomic_get(volatile cafe_atomic_val *target)
{
	return vxAtomicGet((atomic_t *)target);
}

cafe_atomic_val cafe_atomic_set(volatile cafe_atomic_val *target,
				cafe_atomic_val value)
{
	return vxAtomicSet((atomic_t *)target, value);
}

bool cafe_atomic_cas(volatile cafe_atomic_val *target,
		     cafe_atomic_val oldval, cafe_atomic_val newval)
{
#if (_WRS_VXWORKS_MAJOR < 6)					\
	|| (_WRS_VXWORKS_MAJOR == 6) && (_WRS_VXWORKS_MINOR < 9)
	return vxCas((atomic_t *)target, oldval, newval);
#else
	return vxAtomicCas((atomic_t *)target, oldval, newval);
#endif
}

#endif	/* CAFE_OS_VXWORKS */
