/* Copyright (c) 2010-2011 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/* Implementations of OS primitives for Windows. */

#if CAFE_OS_WINDOWS

#include "cafe_os.h"
#include <cafe/cafe.h>
#include <cafe/ccdiff.h>
#include <stdio.h>
#include <process.h>
#include <fcntl.h>
#include <time.h>
#include "cafe_assert.h"
#include "time_utils.h"
#include "cafe_trace.h"

static pthread_mutex_t cafe_mutex;
static volatile pthread_t cafe_lock_thread_owner = 0;

static pthread_cond_t cafe_cond;

int cafe_lock_init(void)
{
	int mutex_init_result;
	int cond_init_result;
	mutex_init_result = pthread_mutex_init(&cafe_mutex, NULL);
	assert(mutex_init_result == 0);
	cond_init_result = pthread_cond_init(&cafe_cond, NULL);
	assert(cond_init_result == 0);
	return mutex_init_result == 0 && cond_init_result == 0 ? CAFE_OK : CAFE_ERROR;
}

bool inline cafe_islocked(void)
{
	return cafe_lock_thread_owner == pthread_self();
}

void __cafe_lock(const char *file, int line)
{
	int mutex_lock_result;

	TRACE(("cafe_lock(%s:%d): thr_id=%llx\n", file, line, (long long)pthread_self()));
	__cafe_assert(!cafe_islocked(), file, line, NULL);
	mutex_lock_result = pthread_mutex_lock(&cafe_mutex);
	assert(mutex_lock_result == 0);
	__cafe_assert(mutex_lock_result == 0, file, line, 0);
	cafe_lock_thread_owner = pthread_self();
}

void __cafe_unlock(const char *file, int line)
{
	int mutex_unlock_result;

	TRACE(("cafe_unlock(%s:%d): thr_id=%llx\n", file, line, (long long)pthread_self()));
	__cafe_assert(cafe_islocked(), file, line, NULL);
	__cafe_assert(pthread_self() == cafe_lock_thread_owner, file, line, "cafe_lock/unlock on same thread");

	cafe_lock_thread_owner = 0;
	mutex_unlock_result = pthread_mutex_unlock(&cafe_mutex);
	assert(mutex_unlock_result == 0);
}

int cafe_notify_signal(void)
{
	int cond_broadcast_result;

	CAFE_ASSERT_LOCKED();
	cond_broadcast_result = pthread_cond_broadcast(&cafe_cond);
	assert(cond_broadcast_result == 0);
	return CAFE_OK;
}

int cafe_notify_wait(void)
{
	int cond_wait_result;

	CAFE_ASSERT_LOCKED();
	cafe_lock_thread_owner = 0;
	cond_wait_result = pthread_cond_wait(&cafe_cond, &cafe_mutex);
	assert(cond_wait_result == 0);
	assert(cafe_lock_thread_owner == 0);
	cafe_lock_thread_owner = pthread_self();
	return CAFE_OK;
}

#endif	/* CAFE_OS_WINDOWS */
