/* cafe_os_windows.h - Cafe OS primitives for Windows. */

/*
 * Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

#ifndef INCcafe_os_windows_h
#define INCcafe_os_windows_h

#define WINDOWS_LEAN_AND_MEAN
#define _WIN32_WINNT 0x0501
#include <windows.h>
#include <cafe/ccdiff.h>

/* Suppress issues w.r.t. _CRT_SECURE_NO_WARNINGS */
#ifdef _MSC_VER
#pragma warning(disable: 4996)
#endif


struct timezone {
	int  tz_minuteswest;	/* minutes W of Greenwich */
	int  tz_dsttime;	/* type of dst correction */
};

#if !defined(HAVE_STRUCT_TIMESPEC) && !defined(_TIMESPEC_DEFINED)
struct timespec {
	time_t  tv_sec;         /* seconds */
	long    tv_nsec;        /* nanoseconds */
};
#define HAVE_STRUCT_TIMESPEC
#define _TIMESPEC_DEFINED
#endif

#define CLOCK_REALTIME 1

typedef int clockid_t;
extern CAFE_EXPORT int clock_gettime(clockid_t clock_id, struct timespec *tp);

#ifdef DISABLE_PTHREADS_WIN32
#  include <pthread.h>
#else
/*
 * PThreads emulation.
 */
#if defined(__CYGWIN__)
#  include <cygwin/types.h>
#else
typedef void * pthread_t;
typedef void * pthread_attr_t;
typedef void * pthread_mutex_t;
typedef void * pthread_cond_t;
typedef void * pthread_mutexattr_t;
typedef void * pthread_condattr_t;

#ifndef ETIMEDOUT
#define ETIMEDOUT 10060 /* Value from winsock.h. */
#endif
#endif

extern CAFE_EXPORT int pthread_attr_init(pthread_attr_t *attr);
extern CAFE_EXPORT int pthread_mutex_init(pthread_mutex_t *mutex,
					  const pthread_mutexattr_t *attr);
extern CAFE_EXPORT int pthread_mutex_destroy(pthread_mutex_t *mutex);

extern CAFE_EXPORT int pthread_cond_init(pthread_cond_t *cond,
					 const pthread_condattr_t *attr);
extern CAFE_EXPORT int pthread_cond_destroy(pthread_cond_t *cond);
extern CAFE_EXPORT int pthread_cond_signal(pthread_cond_t *cond);
extern CAFE_EXPORT int pthread_cond_broadcast(pthread_cond_t *cond);
extern CAFE_EXPORT int pthread_cond_wait(pthread_cond_t *cond,
					 pthread_mutex_t *mutex);
extern CAFE_EXPORT int pthread_cond_timedwait(pthread_cond_t *cond,
					      pthread_mutex_t *mutex,
					      const struct timespec *abstime);
extern CAFE_EXPORT int pthread_mutex_lock(pthread_mutex_t *mutex);
extern CAFE_EXPORT int pthread_mutex_unlock(pthread_mutex_t *mutex);
extern CAFE_EXPORT pthread_t pthread_self(void);
extern CAFE_EXPORT int pthread_create(pthread_t *thread,
				      const pthread_attr_t *attr,
				      void * (*start_routine)(void *),
				      void *arg);
extern CAFE_EXPORT int pthread_join(pthread_t thread, void **value_ptr);
extern CAFE_EXPORT int pthread_equal(pthread_t thread1, pthread_t thread2);
#endif

extern CAFE_EXPORT int getpagesize(void);
extern CAFE_EXPORT int cafe_sched_yield(void);
extern CAFE_EXPORT int nanosleep(struct timespec *tp, struct timespec *rmtp);

#endif /* INCcafe_os_windows_h */
