/* strtoll_test.c - */

/* Copyright (c) 2009, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
08jul14,sdt  Include cafe CUnitest header - WB4-3658
21may09,tcr  created
*/

/*
DESCRIPTION

Testcode for the strtoll() and strtoull() implmentations

*/

/* includes */

#ifndef FAST
#  define FAST
#endif

#include <test/cafe_CUnitTest.h>
#include <strtoll.h>
#include <strtoull.h>
#include <errno.h>

/* defines */

/* diab doesn't define these limits... */

#if(TOOL==diab)
#ifndef __LONG_LONG_MAX__
#define __LONG_LONG_MAX__ 9223372036854775807LL
#endif
#undef LONG_LONG_MIN
#define LONG_LONG_MIN (-LONG_LONG_MAX-1)
#undef LONG_LONG_MAX
#define LONG_LONG_MAX __LONG_LONG_MAX__

#undef ULONG_LONG_MAX
#define ULONG_LONG_MAX (LONG_LONG_MAX * 2ULL + 1)

#endif

/* typedefs */

/* globals */

/* locals */

/* forward declarations */

int decimal(void);
int udecimal(void);
int hex(void);
int uhex(void);

typedef struct test_data_signed {
	char *input;
	long long output;
	int errno_value;
	size_t offset;
} test_data_signed_t;

typedef struct test_data_unsigned {
	char *input;
	unsigned long long output;
	int errno_value;
	size_t offset;
} test_data_unsigned_t;

test_data_signed_t decimal_results[] = {
	{"", 0LL, 0, 0},
	{"+0", 0LL, 0, 2},
	{"0", 0LL, 0, 1},
	{"-0", 0LL, 0, 2},
	{"+1", 1LL, 0, 2},
	{"1", 1LL, 0, 1},
	{"-1", -1LL, 0, 2},
	{"+1234", 1234LL, 0, 5},
	{"1234", 1234LL, 0, 4},
	{"-1234", -1234LL, 0, 5},
	{"+4294967296", 4294967296LL, 0, 11},	/* 2^32 */
	{"4294967296", 4294967296LL, 0, 10},	/* 2^32 */
	{"-4294967296", -4294967296LL, 0, 11},	/* - 2^32 */
	{"4611686018427387904", 1LL << 62, 0, 19},	/* 2^62 */
	{"-4611686018427387904", -(1LL << 62), 0, 20},	/* - 2^62 */
	{"9223372036854775807", LONG_LONG_MAX, 0, 19},	/* 2^63 - 1 */
	{"-9223372036854775808", LONG_LONG_MIN, 0, 20},	/* - 2^63  */
	{" ", 0LL, 0, 0},
	{" +1", 1LL, 0, 3},
	{" 1", 1LL, 0, 2},
	{" -1", -1LL, 0, 3},
	{"9223372036854775808", LONG_LONG_MAX, ERANGE, 19},	/* 2^63 */
	{"-9223372036854775809", LONG_LONG_MIN, ERANGE, 20},	/* - 2^63 - 1 */

};

test_data_unsigned_t udecimal_results[] = {
	{"", 0ULL, 0, 0},
	{"+0", 0ULL, 0, 2},
	{"0", 0ULL, 0, 1},
	{"-0", 0ULL, 0, 2},
	{"+1", 1ULL, 0, 2},
	{"1", 1ULL, 0, 1},
	{"+1234", 1234ULL, 0, 5},
	{"1234", 1234ULL, 0, 4},
	{"+4294967296", 4294967296ULL, 0, 11},	/* 2^32 */
	{"4294967296", 4294967296ULL, 0, 10},	/* 2^32 */
	{"4611686018427387904", 1ULL << 62, 0, 19},	/* 2^62 */
	{"9223372036854775807", (1ULL << 63) - 1, 0, 19},	/* 2^63 - 1 */
	{"18446744073709551615", ULONG_LONG_MAX, 0, 20},	/* 2^64 - 1 */
	{" ", 0ULL, 0, 0},
	{" +1", 1ULL, 0, 3},
	{" 1", 1ULL, 0, 2},
	{"18446744073709551616", ULONG_LONG_MAX, ERANGE, 20},	/* 2^64 */

};

test_data_signed_t hex_results[] = {
	{"", 0LL, 0, 0},
	{"+0", 0LL, 0, 2},
	{"0", 0LL, 0, 1},
	{"-0", 0LL, 0, 2},
	{"+1", 1LL, 0, 2},
	{"1", 1LL, 0, 1},
	{"-1", -1LL, 0, 2},
	{"+1234", 0x1234LL, 0, 5},
	{"1234", 0x1234LL, 0, 4},
	{"-1234", -0x1234LL, 0, 5},
	{"+100000000", 4294967296LL, 0, 10},	/* 2^32 */
	{"100000000", 4294967296LL, 0, 9},	/* 2^32 */
	{"-100000000", -4294967296LL, 0, 10},	/* - 2^32 */
	{"4000000000000000", 1LL << 62, 0, 16},	/* 2^62 */
	{"-4000000000000000", -4611686018427387904LL, 0, 17},	/* 2^62 */
	{"7fffffffffffffff", LONG_LONG_MAX, 0, 16},	/* 2^63 - 1 */
	{"-8000000000000000", LONG_LONG_MIN, 0, 17},	/* - 2^63  */
	{" ", 0LL, 0, 0},
	{" +1", 1LL, 0, 3},
	{" 1", 1LL, 0, 2},
	{" -1", -1LL, 0, 3},
	{"8000000000000000", LONG_LONG_MAX, ERANGE, 16},	/* 2^63 */
	{"-8000000000000001", LONG_LONG_MIN, ERANGE, 17},	/* - 2^63 - 1 */

};

test_data_unsigned_t uhex_results[] = {
	{"", 0ULL, 0, 0},
	{"+0", 0ULL, 0, 2},
	{"0", 0ULL, 0, 1},
	{"-0", 0ULL, 0, 2},
	{"+1", 1ULL, 0, 2},
	{"1", 1ULL, 0, 1},
	{"+1234", 0x1234ULL, 0, 5},
	{"1234", 0x1234ULL, 0, 4},
	{"+100000000", 1ULL << 32, 0, 10},	/* 2^32 */
	{"100000000", 1ULL << 32, 0, 9},	/* 2^32 */
	{"4000000000000000", 1ULL << 62, 0, 16},	/* 2^62 */
	{"8000000000000000", 1ULL << 63, 0, 16},	/* 2^63 */
	{"ffffffffffffffff", ULONG_LONG_MAX, 0, 16},	/* 2^64 - 1 */
	{" ", 0ULL, 0, 0},
	{" +1", 1ULL, 0, 3},
	{" 1", 1ULL, 0, 2},
	{"10000000000000000", ULONG_LONG_MAX, ERANGE, 17},	/* 2^64 */

};

int decimal(void)
{
	int n;
	test_data_signed_t *p;
	long long result;
	char *final;

	for (n = 0; n < sizeof(decimal_results) / sizeof(decimal_results[0]);
	     n++) {
		p = decimal_results + n;

		errno = 0;
		result = strtoll(p->input, &final, 10);
/*         printf ("i: %s   r: %lld  off: %d   err: %d\n", p->input, result, final - p->input, errno); */
		CUT_ASSERT_EQUAL(result, p->output);
		CUT_ASSERT_EQUAL(errno, p->errno_value);
		CUT_ASSERT_EQUAL(final, p->input + p->offset);
	}

	return 0;
}

int udecimal(void)
{
	int n;
	test_data_unsigned_t *p;
	unsigned long long result;
	char *final;

	for (n = 0; n < sizeof(udecimal_results) / sizeof(udecimal_results[0]);
	     n++) {
		p = udecimal_results + n;

		errno = 0;
		result = strtoull(p->input, &final, 10);
/*         printf ("i: %s   r: %lld  off: %d   err: %d\n", p->input, result, final - p->input, errno); */
		CUT_ASSERT_EQUAL(result, p->output);
		CUT_ASSERT_EQUAL(errno, p->errno_value);
		CUT_ASSERT_EQUAL(final, p->input + p->offset);
	}

	return 0;
}

int hex(void)
{
	int n;
	test_data_signed_t *p;
	long long result;
	char *final;

	for (n = 0; n < sizeof(hex_results) / sizeof(hex_results[0]); n++) {
		p = hex_results + n;

		errno = 0;
		result = strtoll(p->input, &final, 16);
/*        printf ("i: %s   r: %lld  off: %d   err: %d\n", p->input, result, final - p->input, errno); */
		CUT_ASSERT_EQUAL(result, p->output);
		CUT_ASSERT_EQUAL(errno, p->errno_value);
		CUT_ASSERT_EQUAL(final, p->input + p->offset);
	}

	return 0;
}

int uhex(void)
{
	int n;
	test_data_unsigned_t *p;
	long long result;
	char *final;

	for (n = 0; n < sizeof(uhex_results) / sizeof(uhex_results[0]); n++) {
		p = uhex_results + n;

		errno = 0;
		result = strtoull(p->input, &final, 16);
		/* printf ("i: %s   r: %llu  off: %d   err: %d\n", p->input, result, final - p->input, errno);  */
		CUT_ASSERT_EQUAL(result, p->output);
		CUT_ASSERT_EQUAL(errno, p->errno_value);
		CUT_ASSERT_EQUAL(final, p->input + p->offset);
	}

	return 0;
}

CUT_BEGIN_SUITE(suite_strtoll)
	CUT_RUN_TEST(decimal);
	CUT_RUN_TEST(udecimal);
	CUT_RUN_TEST(hex);
	CUT_RUN_TEST(uhex);
CUT_END_SUITE

CUT_BEGIN_TEST_HARNESS
	CUT_RUN_SUITE_TESTS(suite_strtoll);
CUT_END_TEST_HARNESS
