/* test_array.c - cafe dynamic array unit tests */

/* Copyright (c) 2010-2011 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <test/cafe_CUnitTest.h>
#include <string.h>
#include <cafe/array.h>

struct test_array_struct {
	int foo;
	unsigned long long bar;
	double baz;
};

static int test_array(void)
{
	size_t i;
	struct cafe_array a;
	struct test_array_struct values[8];

	cafe_array_init(&a, cafe_stdlib_allocator, sizeof(values[0]));
	CUT_ASSERT_EQUAL(0, cafe_array_len(&a));
	CUT_ASSERT_EQUAL(0, cafe_array_capacity(&a));
	CUT_ASSERT_EQUAL(sizeof(values[0]), cafe_array_element_size(&a));

	for (i = 0; i < CAFE_NELEMENTS(values); i++) {
		values[i].foo = i;
		values[i].bar = ~i;
		values[i].baz = 0.001 - i;
		CUT_ASSERT_CAFE_OK(cafe_array_append(&a, &values[i]));
	}

	CUT_ASSERT_EQUAL(CAFE_NELEMENTS(values), cafe_array_len(&a));
	CUT_ASSERT_EQUAL(8, cafe_array_capacity(&a));

	for (i = 0; i < cafe_array_len(&a); i++) {
		struct test_array_struct v;
		struct test_array_struct *p;
		v = cafe_array_val(&a, struct test_array_struct, i);
		p = cafe_array_val_ptr(&a, struct test_array_struct, i);
		CUT_ASSERT_EQUAL(values[i].foo, v.foo);
		CUT_ASSERT_EQUAL(values[i].bar, v.bar);
		CUT_ASSERT_EQUAL(values[i].baz, v.baz);
		CUT_ASSERT_EQUAL(values[i].foo, p->foo);
		CUT_ASSERT_EQUAL(values[i].bar, p->bar);
		CUT_ASSERT_EQUAL(values[i].baz, p->baz);
	}

	cafe_array_release(&a);
	return 0;
}

static int test_int_array(void)
{
	size_t i;
	struct cafe_array a;
	int values[8];

	cafe_array_init(&a, cafe_stdlib_allocator, sizeof(values[0]));
	CUT_ASSERT_EQUAL(0, cafe_array_len(&a));
	CUT_ASSERT_EQUAL(0, cafe_array_capacity(&a));
	CUT_ASSERT_EQUAL(sizeof(values[0]), cafe_array_element_size(&a));

	for (i = 0; i < CAFE_NELEMENTS(values); i++) {
		values[i] = i;
		CUT_ASSERT_CAFE_OK(cafe_array_append(&a, &values[i]));
	}

	CUT_ASSERT_EQUAL(CAFE_NELEMENTS(values), cafe_array_len(&a));
	CUT_ASSERT_EQUAL(8, cafe_array_capacity(&a));

	for (i = 0; i < cafe_array_len(&a); i++) {
		int v;
		int *p;
		v = cafe_array_val(&a, int, i);
		p = cafe_array_val_ptr(&a, int, i);
		CUT_ASSERT_EQUAL(values[i], v);
		CUT_ASSERT_EQUAL(values[i], *p);
	}

	cafe_array_release(&a);
	return 0;
}

CUT_BEGIN_TEST_HARNESS(array_suite)
CUT_RUN_TEST(test_array);
CUT_RUN_TEST(test_int_array);
CUT_END_TEST_HARNESS
