/* Copyright (c) 2010-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <string.h>		/* memset */
#include <cafe/expr.h>
#include <cafe/value_type.h>
#include <test/cafe_CUnitTest.h>

static int test_expr_const(void)
{
	struct cafe_namespace *ns;
	struct cafe_value_type *type;
	struct cafe_value_data *data;
	struct s1 {
		char v1;
		unsigned char v2;
		short v3;
		unsigned short v4;
		int v5;
		unsigned int v6;
		long v7;
		unsigned long v8;
		long long v9;
		unsigned long long v10;
		int v11[5];
		struct cafe_vm_dynarray v12;
		char *v13;
		struct cafe_vm_blob v14;
	} *s1;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns,
					"S[s1,v[v1,c,,1]v[v2,C,,2]v[v3,h,,3]"
					"v[v4,H,,4]v[v5,i,,5]v[v6,I,,6]v[v7,l,,7]"
					"v[v8,L,,8]v[v9,q,,-9]v[v10,Q,,10]"
					"v[v11,a[i,5],,[1,2,-3,4,5]]"
					"v[v12,a[i],,[-101,102,103]]"
					"v[v13,s,,\"string for v13\"]"
					"v[v14,b,,\"string for v14\"]]",
					&type));
	CUT_ASSERT_TRUE(type->size == sizeof *s1);
	CUT_ASSERT_CAFE_OK(cafe_value_create(type, &data));
	s1 = (struct s1 *)data->u.data;
	CUT_ASSERT_EQUAL(s1->v1, 1);
	CUT_ASSERT_EQUAL(s1->v2, 2);
	CUT_ASSERT_EQUAL(s1->v3, 3);
	CUT_ASSERT_EQUAL(s1->v4, 4);
	CUT_ASSERT_EQUAL(s1->v5, 5);
	CUT_ASSERT_EQUAL(s1->v6, 6);
	CUT_ASSERT_EQUAL(s1->v7, 7);
	CUT_ASSERT_EQUAL(s1->v8, 8);
	CUT_ASSERT_EQUAL(s1->v9, -9);
	CUT_ASSERT_EQUAL(s1->v10, 10);
	CUT_ASSERT_EQUAL(s1->v11[0], 1);
	CUT_ASSERT_EQUAL(s1->v11[1], 2);
	CUT_ASSERT_EQUAL(s1->v11[2], -3);
	CUT_ASSERT_EQUAL(s1->v11[3], 4);
	CUT_ASSERT_EQUAL(s1->v11[4], 5);
	CUT_ASSERT_EQUAL(s1->v12.l, 3);
	CUT_ASSERT_EQUAL(((int *)s1->v12.p)[0], -101);
	CUT_ASSERT_EQUAL(((int *)s1->v12.p)[1], 102);
	CUT_ASSERT_EQUAL(((int *)s1->v12.p)[2], 103);
	CUT_ASSERT_TRUE(strcmp(s1->v13, "string for v13") == 0);
	CUT_ASSERT_EQUAL(s1->v14.l, 14);
	CUT_ASSERT_TRUE(memcmp(s1->v14.p, "string for v14", 14) == 0);
	cafe_value_delete(data);

	cafe_namespace_delete(ns);
	return 0;
}

static int test_expr_new(void)
{
	struct cafe_namespace *ns;
	struct cafe_vm_expr *e;
	const char *fmt;
	struct cafe_value_type *t;
	struct cafe_value_context *context;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	context = cafe_value_context_create(ns->allocator);
	assert(context != NULL);
	context->ns = ns;

	fmt = "S[S1,v[a,i]v[b,i]v[c,i]]";
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, fmt, &t));
	context->type = t->ident;
	fmt = "1+2*3";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	CUT_ASSERT_EQUAL(e->kind, CAFE_EXPR_ADD);
	CUT_ASSERT_EQUAL(e->u.bin.r->kind, CAFE_EXPR_MUL);
	cafe_expr_free(ns->allocator, e);

	fmt = "a ? b : c";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	CUT_ASSERT_EQUAL(e->kind, CAFE_EXPR_QUEST);
	CUT_ASSERT_EQUAL(e->u.bin.r->kind, CAFE_EXPR_COLON);
	cafe_expr_free(ns->allocator, e);

	fmt = "S[S2,v[a,b]v[b,b]v[c,i]v[d,i]v[e,i]]";
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, fmt, &t));
	context->type = t->ident;
	fmt = "a ? b ? c : d : e";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	CUT_ASSERT_EQUAL(e->kind, CAFE_EXPR_QUEST);
	CUT_ASSERT_EQUAL(e->u.bin.l->kind, CAFE_EXPR_IDENT);
	CUT_ASSERT_EQUAL(e->u.bin.r->kind, CAFE_EXPR_COLON);
	CUT_ASSERT_EQUAL(e->u.bin.r->u.bin.l->kind, CAFE_EXPR_QUEST);
	CUT_ASSERT_EQUAL(e->u.bin.r->u.bin.r->kind, CAFE_EXPR_IDENT);
	cafe_expr_free(ns->allocator, e);

	fmt = "S[S3,v[a,b]v[b,i]v[c,b]v[d,i]v[e,i]]";
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, fmt, &t));
	context->type = t->ident;
	fmt = "a ? b : c ? d : e";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	CUT_ASSERT_EQUAL(e->kind, CAFE_EXPR_QUEST);
	CUT_ASSERT_EQUAL(e->u.bin.l->kind, CAFE_EXPR_IDENT);
	CUT_ASSERT_EQUAL(e->u.bin.r->kind, CAFE_EXPR_COLON);
	CUT_ASSERT_EQUAL(e->u.bin.r->u.bin.l->kind, CAFE_EXPR_IDENT);
	CUT_ASSERT_EQUAL(e->u.bin.r->u.bin.r->kind, CAFE_EXPR_QUEST);
	cafe_expr_free(ns->allocator, e);

	fmt = ("S[S6,v[d,i]]"
	       "S[S5,v[b,a[t[S6]]]]"
	       "S[S4,v[a,t[S5]]v[c,i]v[e,i]]");
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, fmt, &t));
	context->type = t->ident;
	fmt = "a.b[c].d(e)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	CUT_ASSERT_EQUAL(e->kind, CAFE_EXPR_CALL);
	CUT_ASSERT_EQUAL(e->u.bin.l->kind, CAFE_EXPR_FIELD);
	CUT_ASSERT_EQUAL(e->u.bin.r->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(e->u.bin.r->u.elem.e->kind, CAFE_EXPR_IDENT);
	CUT_ASSERT_EQUAL(e->u.bin.r->u.elem.next, NULL);
	CUT_ASSERT_EQUAL(e->u.bin.l->u.bin.l->kind, CAFE_EXPR_INDEX);
	CUT_ASSERT_EQUAL(e->u.bin.l->u.bin.r->kind, CAFE_EXPR_IDENT);
	CUT_ASSERT_EQUAL(e->u.bin.l->u.bin.l->u.bin.l->kind, CAFE_EXPR_FIELD);
	CUT_ASSERT_EQUAL(e->u.bin.l->u.bin.l->u.bin.r->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(e->u.bin.l->u.bin.l->u.bin.r->u.elem.e->kind, CAFE_EXPR_IDENT);
	CUT_ASSERT_EQUAL(e->u.bin.l->u.bin.l->u.bin.l->u.bin.l->kind, CAFE_EXPR_IDENT);
	CUT_ASSERT_EQUAL(e->u.bin.l->u.bin.l->u.bin.l->u.bin.r->kind, CAFE_EXPR_IDENT);
	cafe_expr_free(ns->allocator, e);

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[S7,v[b,i]]", &t));
	context->type = t->ident;
	fmt = "{t[S7]}.b";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	CUT_ASSERT_EQUAL(e->kind, CAFE_EXPR_FIELD);
	CUT_ASSERT_EQUAL(e->u.bin.l->kind, CAFE_EXPR_TYPE);
	CUT_ASSERT_EQUAL(e->u.bin.r->kind, CAFE_EXPR_IDENT);
	cafe_expr_free(ns->allocator, e);

	fmt = "({i}){t[S7]}.b";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	CUT_ASSERT_EQUAL(e->kind, CAFE_EXPR_CAST);
	CUT_ASSERT_EQUAL(e->u.bin.l->kind, CAFE_EXPR_FIELD);
	CUT_ASSERT_EQUAL(e->u.bin.r->kind, CAFE_EXPR_TYPE);
	CUT_ASSERT_EQUAL(e->u.bin.l->u.bin.l->kind, CAFE_EXPR_TYPE);
	CUT_ASSERT_EQUAL(e->u.bin.l->u.bin.r->kind, CAFE_EXPR_IDENT);
	cafe_expr_free(ns->allocator, e);

	cafe_value_context_delete(context);
	cafe_namespace_delete(ns);
	return 0;
}

static int test_expr_resolve(void)
{
	struct cafe_namespace *ns;
	struct cafe_value_type *type;
	struct cafe_vm_attr *a1;
	struct cafe_vm_expr *e1;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns,
					"S[s1,"
					"K[e1,i,,{t[s1]}.v1]"
					"v[v1,i]"
					"]",
					&type));

	a1 = cafe_type_attr_by_name(type, "e1");
	CUT_ASSERT_NOT_NULL(a1);
	CUT_ASSERT_EQUAL(a1->kind, CAFE_ATTR_CONST);
	e1 = a1->ident->expr;
	CUT_ASSERT_NOT_NULL(e1);
	CUT_ASSERT_EQUAL(e1->kind, CAFE_EXPR_ATTR);

	cafe_namespace_delete(ns);
	return 0;
}

static int test_expr_eval(void)
{
	struct cafe_namespace *ns;
	struct cafe_value_context *context;
	struct cafe_expr_eval_context eval_context;
	const char *fmt;
	struct cafe_vm_expr *e;
	cafe_any_value result;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	context = cafe_value_context_create(ns->allocator);
	CUT_ASSERT_NOT_NULL(context);
	context->ns = ns;
	cafe_expr_eval_context_create(&eval_context, ns->allocator);

	fmt = "bucket(123,10)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 120);
	cafe_expr_free(ns->allocator, e);

	fmt = "bucket(({q})-234,10)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, -230);
	cafe_expr_free(ns->allocator, e);

	fmt = "bucket_log2(500,1)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 256);
	cafe_expr_free(ns->allocator, e);

	fmt = "bucket_log2(({q})-500,1)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, -512);
	cafe_expr_free(ns->allocator, e);

	fmt = "bucket_log10(1234,1)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 1000);
	cafe_expr_free(ns->allocator, e);

	fmt = "bucket_log10(({q})-1234,1)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, -10000);
	cafe_expr_free(ns->allocator, e);

	fmt = "bucket_log2(400,8)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 384);
	cafe_expr_free(ns->allocator, e);

	fmt = "bucket_log2(({q})-400,8)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, -416);
	cafe_expr_free(ns->allocator, e);

	fmt = "bucket_log10(2345,10)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 1900);
	cafe_expr_free(ns->allocator, e);

	fmt = "bucket_log10(({q})-2345,10)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, -2800);
	cafe_expr_free(ns->allocator, e);

	fmt = "minmax(150,100,200)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 150);
	cafe_expr_free(ns->allocator, e);

	fmt = "minmax(50,100,200)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 100);
	cafe_expr_free(ns->allocator, e);

	fmt = "minmax(250,100,200)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 200);
	cafe_expr_free(ns->allocator, e);

	fmt = "minmax(({q})-50,-100,200)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, -50);
	cafe_expr_free(ns->allocator, e);

	fmt = "minmax(({q})50,-100,200)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 50);
	cafe_expr_free(ns->allocator, e);

	fmt = "minmax(({q})-150,-100,200)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, -100);
	cafe_expr_free(ns->allocator, e);

	fmt = "minmax(({q})-25,-100,-50)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, -50);
	cafe_expr_free(ns->allocator, e);

	fmt = "minmax(({q})25,-100,-50)";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, -50);
	cafe_expr_free(ns->allocator, e);

	fmt = "\"foo\"==\"foo\"";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 1);
	cafe_expr_free(ns->allocator, e);

	fmt = "\"foo\"==\"bar\"";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 0);
	cafe_expr_free(ns->allocator, e);

	fmt = "\"foo\"!=\"foo\"";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 0);
	cafe_expr_free(ns->allocator, e);

	fmt = "\"foo\"!=\"bar\"";
	CUT_ASSERT_CAFE_OK(cafe_expr_parse(context, &fmt, &e));
	e = cafe_expr_resolve(context, e);
	CUT_ASSERT_NOT_NULL(e);
	CUT_ASSERT_NOT_NULL(e->type);
	CUT_ASSERT_CAFE_OK(cafe_expr_eval_value(&eval_context, e, &result));
	CUT_ASSERT_EQUAL(result.ll, 1);
	cafe_expr_free(ns->allocator, e);

	cafe_expr_eval_context_delete(&eval_context);
	cafe_value_context_delete(context);
	cafe_namespace_delete(ns);
	return 0;
}

CAFE_BEGIN_SUITE_HARNESS(expr_suite)
	CUT_RUN_TEST(test_expr_const);
	CUT_RUN_TEST(test_expr_new);
	CUT_RUN_TEST(test_expr_resolve);
	CUT_RUN_TEST(test_expr_eval);
CAFE_END_SUITE_HARNESS
