/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <string.h>		/* memset */
#include <cafe/dict.h>
#include <test/cafe_CUnitTest.h>

static int nature_reg_unreg(void)
{
	struct cafe_namespace *ns;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "N[percent,v[scale,I]]", NULL));
	CUT_ASSERT(cafe_type_parse(ns, "N[percent,v[notScale,I]]", NULL) != CAFE_OK);
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "N[slice,v[id,I]]", NULL));
	cafe_namespace_delete(ns);
	return 0;
}

static int nature_use_in_type(void)
{
	struct cafe_namespace *ns;
	struct cafe_value_type *type;
	struct cafe_vm_attr *attr;
	struct cafe_vm_expr *expr;
	struct cafe_type_attr_iter attr_iter;

	CUT_ASSERT_CAFE_OK(cafe_namespace_create(cafe_stdlib_allocator, &ns));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "N[percent,v[scale,I]]", NULL));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "N[slice,v[id,I]]", NULL));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "N[label,v[id,s]]", NULL));

	CUT_ASSERT_CAFE_OK(cafe_type_parse(ns, "S[s2,k[key1,i,n[percent,100]]v[m2,i]]S[s1,v[system,i,n[slice,1]]v[user,i,n[label,\"foo bar\"]n[percent,100]]v[other,D[t[s2]]]]", &type));
	CUT_ASSERT_EQUAL(type->kind, CAFE_TYPE_STRUCT);
	CUT_ASSERT_EQUAL(type->elements, 3);

	attr = cafe_type_first_attr(type, &attr_iter);
	CUT_ASSERT_NOT_NULL(attr->ident->name);
	CUT_ASSERT_EQUAL(strcmp(attr->ident->name, "system"), 0);
	CUT_ASSERT_EQUAL(cafe_attr_type(attr)->kind, CAFE_TYPE_INT);
	CUT_ASSERT_NOT_NULL(attr->ident->natures);
	CUT_ASSERT_NOT_NULL(attr->ident->natures->def);
	CUT_ASSERT_NOT_NULL(attr->ident->natures->def->name);
	CUT_ASSERT_EQUAL(strcmp(attr->ident->natures->def->name, "slice"), 0);
	CUT_ASSERT_NOT_NULL(attr->ident->natures->args);
	CUT_ASSERT_EQUAL(attr->ident->natures->args->kind, CAFE_EXPR_LIST);
	expr = attr->ident->natures->args->u.e;
	CUT_ASSERT_EQUAL(expr->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(expr->u.elem.e->kind, CAFE_EXPR_UVALUE);
	CUT_ASSERT_EQUAL(expr->u.elem.e->u.ull, 1);
	CUT_ASSERT_NULL(attr->ident->natures->next);

	attr = cafe_type_next_attr(&attr_iter);
	CUT_ASSERT_NOT_NULL(attr->ident->name);
	CUT_ASSERT_EQUAL(strcmp(attr->ident->name, "user"), 0);
	CUT_ASSERT_EQUAL(cafe_attr_type(attr)->kind, CAFE_TYPE_INT);
	CUT_ASSERT_NOT_NULL(attr->ident->natures);
	CUT_ASSERT_NOT_NULL(attr->ident->natures->def);
	CUT_ASSERT_NOT_NULL(attr->ident->natures->def->name);
	CUT_ASSERT_EQUAL(strcmp(attr->ident->natures->def->name, "label"), 0);
	CUT_ASSERT_NOT_NULL(attr->ident->natures->next);
	CUT_ASSERT_NOT_NULL(attr->ident->natures->next->def);
	CUT_ASSERT_NOT_NULL(attr->ident->natures->next->def->name);
	CUT_ASSERT_EQUAL(strcmp(attr->ident->natures->next->def->name, "percent"), 0);
	CUT_ASSERT_NOT_NULL(attr->ident->natures->next->args);
	CUT_ASSERT_EQUAL(attr->ident->natures->next->args->kind, CAFE_EXPR_LIST);
	expr = attr->ident->natures->next->args->u.e;
	CUT_ASSERT_EQUAL(expr->kind, CAFE_EXPR_ELEMENT);
	CUT_ASSERT_EQUAL(expr->u.elem.e->kind, CAFE_EXPR_UVALUE);
	CUT_ASSERT_EQUAL(expr->u.elem.e->u.ull, 100);
	CUT_ASSERT_NULL(attr->ident->natures->next->next);

	attr = cafe_type_next_attr(&attr_iter);
	CUT_ASSERT_EQUAL(cafe_attr_type(attr)->kind, CAFE_TYPE_DICT);
	CUT_ASSERT_NULL(attr->ident->natures);
	cafe_namespace_delete(ns);

	return 0;
}


CAFE_BEGIN_SUITE_HARNESS(nature_suite)
	CUT_RUN_TEST(nature_reg_unreg);
	CUT_RUN_TEST(nature_use_in_type);
CAFE_END_SUITE_HARNESS
