/* test_ptr_array.c - cafe dynamic array unit tests */

/* Copyright (c) 2010-2011 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <test/cafe_CUnitTest.h>
#include <string.h>
#include <cafe/ptr_array.h>

#define STREQ(A, B) (strcmp((A), (B)) == 0)

static int test_ptr_array(void)
{
	size_t i;
	struct cafe_ptr_array a;
	const char *values[] = {
		"one", "two", "three", "four", "five",
		"six", "seven", "eight", "nine", "ten"
	};

	cafe_ptr_array_init(&a, cafe_stdlib_allocator);
	CUT_ASSERT_EQUAL(0, cafe_ptr_array_len(&a));
	CUT_ASSERT_EQUAL(0, cafe_ptr_array_capacity(&a));

	for (i = 0; i < CAFE_NELEMENTS(values); i++) {
		CUT_ASSERT_CAFE_OK(cafe_ptr_array_append(&a, (void *)values[i]));
	}

	CUT_ASSERT_EQUAL(CAFE_NELEMENTS(values), cafe_ptr_array_len(&a));

	for (i = 0; i < cafe_ptr_array_len(&a); i++) {
		const char *v;
		v = cafe_ptr_array_val(&a, i);
		CUT_ASSERT_TRUE(STREQ(values[i], v));
	}

	cafe_ptr_array_release(&a);
	return 0;
}

CUT_BEGIN_TEST_HARNESS(ptr_array_suite)
CUT_RUN_TEST(test_ptr_array);
CUT_END_TEST_HARNESS
