;;; A major mode for editing Cafe Event Processor files.

(require 'cc-mode)

;; Define several classes of keywords.

(defconst cafe-ep-keywords
  '("as" "const" "else" "extends" "for" "from" "if" "import" "in"
    "key" "nature" "on" "processor" "return" "struct" "with")
  "Cafe EP keywords.")

(defconst cafe-ep-types
  '("array" "blob" "bool" "boolean" "dict"
    "int" "long" "longlong"
    "string"
    "uint" "ulong" "ulonglong")
  "Cafe EP types.")

(defconst cafe-ep-constants
  '("null")
  "Cafe EP constants.")

(defconst cafe-ep-functions
  '("send")
  "Cafe EP functions.")

;; Create the regex string for each class of keywords
;;
;; Below we generate the regex for each keyword class, using the
;; built-in function “regexp-opt”. We gave regexp-opt a second
;; optional argument “'words”. This will create a regex that match
;; only if it is a complete word; if word is contained inside a longer
;; word it will not be highlighted.

(defconst cafe-ep-keywords-regexp (regexp-opt cafe-ep-keywords 'words))
(defconst cafe-ep-type-regexp (regexp-opt cafe-ep-types 'words))
(defconst cafe-ep-constant-regexp (regexp-opt cafe-ep-constants 'words))
(defconst cafe-ep-functions-regexp (regexp-opt cafe-ep-functions 'words))

;; Create the list for font-lock.  Each class of keyword is given a
;; particular face

(setq cafe-ep-font-lock-keywords
      `(
	("?=" . font-lock-builtin-face)
	("\\(\\w+@\\w+\\)" . font-lock-constant-face)
	(,cafe-ep-type-regexp . font-lock-type-face)
	(,cafe-ep-constant-regexp . font-lock-builtin-face)
	(,cafe-ep-functions-regexp . font-lock-function-name-face)
	(,cafe-ep-keywords-regexp . font-lock-keyword-face)
	;; note: order above matters. “cafe-ep-keywords-regexp” goes
	;; last because otherwise the keyword “state” in the function
	;; “state_entry” would be highlighted.
	;; java-font-lock-keywords-1
	;; java-font-lock-keywords-2
	;; java-font-lock-keywords-3
	))

(define-derived-mode cafe-ep-mode java-mode
  "Cafe-EP"
  "Major mode for editing Cafe Event Processor code.

This mode is derived from `java-mode'; see its documentation for
further information.

\\{cafe-ep-mode-map}"
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults
  	'((cafe-ep-font-lock-keywords)
  	  nil nil ((?_ . "w") (?$ . "w")) nil))
  (easy-menu-define c-cafe-ep-menu cafe-ep-mode-map
    "Cafe-EP Mode Commands" (c-mode-menu "Cafe-EP")))

(setq auto-mode-alist (cons '("\\.cafeconfig$" . cafe-ep-mode) auto-mode-alist))
(setq auto-mode-alist (cons '("\\.cafetype$" . cafe-ep-mode) auto-mode-alist))

(defun cafe-ep-mode-untabify ()
  (save-excursion
    (goto-char (point-min))
    (while (re-search-forward "[ \t]+$" nil t)
      (delete-region (match-beginning 0) (match-end 0)))
    (goto-char (point-min))
    (if (search-forward "\t" nil t)
	(untabify (1- (point)) (point-max))))
  nil)

(add-hook 'cafe-ep-mode-hook
	  '(lambda ()
	     (make-local-variable 'write-contents-hooks)
	     (add-hook 'write-contents-hooks 'cafe-ep-mode-untabify)))

(provide 'cafe-ep-mode)
