#!/usr/bin/env perl

use strict;
use IO::File;
use Text::ParseWords;

my @data = <DATA>;

my $header_file = new IO::File ">hpc_e500.type"
  or die "cannot open file: $!";

my $host_file = new IO::File ">hpc_e500.host"
  or die "cannot open file: $!";

my $header_comment1 = <<END_COMMENT;
#include "cafe.type"

END_COMMENT

# Example output:
# #define HPC_E500___HPC___E500_PROCESSOR_CYCLES_VT_DEF "S[hpc.e500_processor_cycles],,n[cafe.COUNTER_METRIC_CLASS,1]n[cafe.LABEL,\"TOT CYC#\"]]"
# #define HPC_E500___HPC___E500_PROCESSOR_CYCLES_VT_REF "t[hpc.e500_processor_cycles]"

sub print_header_file_entry {
  my (@words) = @_;

  my $s1 = sprintf("struct hpc.e500_%s\n\t:cafe.COUNTER_METRIC_CLASS(1)\n\t:cafe.LABEL(\"%s\") {\n}\n",
    uc($words[0]), $words[3]);

  print $header_file "$s1\n";
}

sub print_host_entry {
  my (@words) = @_;
  my $s1 = sprintf("set hpc.e500_%s_count n[category.%s]n[cafeui.TOOLTIP,\"%s\"]n[formatter.big]n[cafe.ADVANCED_METRIC]", uc($words[0]), $words[2], $words[4]);
  print $host_file "$s1\n";
}

print $header_file $header_comment1;

foreach (@data) {
  my @words = shellwords($_);
  print_header_file_entry(@words);
}

foreach (@data) {
  my @words = shellwords("$_");
  print_host_entry(@words);
}

# VT name, min reload value, UI metric category, UI metric name (string), UI tooltip (string)

__DATA__
PROCESSOR_CYCLES			1000	hpc_e500_general	"Processor CYCs"		""
INSTRUCTIONS_COMPLETED			1000	hpc_e500_general	"INST Completed"		""
MICRO_OPS_COMPLETED			1000	hpc_e500_general	"Micro-ops Completed"		""
INSTRUCTIONS_FETCHED			1000	hpc_e500_general	"Instructions Fetched"		""
MICRO_OPS_DECODED			1000	hpc_e500_general	"Micro-ops Decoded"		""
PM_EVENT_TRANSITIONS			1000	hpc_e500_general	"PM_EVENT Transitions"		""
PM_EVENT_CYCLES				1000	hpc_e500_general	"PM_EVENT CYCs"			""
BRANCH_INSTRUCTIONS_COMPLETED		1000	hpc_e500_inst_types_completed	"BR INST Completed"		""
LOAD_MICRO_OPS_COMPLETED		1000	hpc_e500_inst_types_completed	"LD Micro-ops Completed"	""
STORE_MICRO_OPS_COMPLETED		1000	hpc_e500_inst_types_completed	"ST Micro-ops Completed"	""
NUMBER_OF_CQ_REDIRECTS			1000	hpc_e500_inst_types_completed	"NUM of CQ Redirects"		""
BRANCHES_FINISHED						1000	hpc_e500_br_pred_and_exec	"BR Finished"			""
TAKEN_BRANCHES_FINISHED						1000	hpc_e500_br_pred_and_exec	"Taken BR Finished"		""
FINISHED_UNCONDITIONAL_BRANCHES_THAT_MISS_THE_BTB		1000	hpc_e500_br_pred_and_exec	"Finished UN-COND BR That Miss the BTB" ""
BRANCHES_MISPREDICTED						1000	hpc_e500_br_pred_and_exec	"BR Miss"			""
BRANCHES_IN_THE_BTB_MISPREDICTED_DUE_TO_DIRECTION_PREDICTION	1000	hpc_e500_br_pred_and_exec	"BR in the BTB Mispredicted due to direction prediction" ""
BTB_HITS_AND_PSEUDO_HITS					1000	hpc_e500_br_pred_and_exec	"BTB hits and pseudo-hits"	""
CYCLES_DECODE_STALLED			1000	hpc_e500_pipe_stalls	"CYC Decode Stalled"		""
CYCLES_ISSUE_STALLED			1000	hpc_e500_pipe_stalls	"CYC Issue Stalled"		""
CYCLES_BRANCH_ISSUE_STALLED		1000	hpc_e500_pipe_stalls	"CYC BR Issue Stalled"		""
CYCLES_SU1_SCHEDULE_STALLED		1000	hpc_e500_pipe_stalls	"CYC SU1 SCHD Stalled"		""
CYCLES_SU2_SCHEDULE_STALLED		1000	hpc_e500_pipe_stalls	"CYC SU2 SCHD Stalled"		""
CYCLES_MU_SCHEDULE_STALLED		1000	hpc_e500_pipe_stalls	"CYC MU SCHD Stalled"		""
CYCLES_LRU_SCHEDULE_STALLED		1000	hpc_e500_pipe_stalls	"CYC LRU SCHD Stalled"		""
CYCLES_BU_SCHEDULE_STALLED		1000	hpc_e500_pipe_stalls	"CYC BU SCHD Stalled"		""
TOTAL_TRANSLATED_SPEC				1000	hpc_e500_DLFB	"TOT Translated"			""
LOADS_TRANSLATED				1000	hpc_e500_DLFB	"LDs Thranslted"			""
STORES_TRANSLATED				1000	hpc_e500_DLFB	"STs Translated"			""
TOUCHES_TRANSLATED				1000	hpc_e500_DLFB	"Touches Translated"			""
CACHEOPS_TRANSLATED				1000	hpc_e500_DLFB	"Cache-Ops Translated"			""
CACHE_INHIBITED_ACCESSES_TRANSLATED		1000	hpc_e500_DLFB	"Cache Inhibited Access Translated"	""
GUARDED_LOADS_TRANSLATED			1000	hpc_e500_DLFB	"Guarded LDs translated"		""
WRITE_THROUGH_STORES_TRANSLATED			1000	hpc_e500_DLFB	"WR Thro STs Translated"		""
MISALIGNED_LOAD_OR_STORE_ACCESSES_TRANSLATED	1000	hpc_e500_DLFB	"Misaligned LD or ST Access Translated"	""
TOTAL_ALLOCATED_TO_DLFB				1000	hpc_e500_DLFB	"TOT Alloc to DLFB"			""
LOADS_TRANSLATED_AND_ALLOCATED_TO_DLFB		1000	hpc_e500_DLFB	"LDs Translated and Alloc to DLFB"	""
STORES_COMPLETED_AND_ALLOCATED_TO_DLFB		1000	hpc_e500_DLFB	"STs Translated and Alloc to DLFB"	""
TOUCHES_TRANSLATED_AND_ALLOCATED_TO_DLFB	1000	hpc_e500_DLFB	"Touches Translated and Alloc to DLFB"	""
STORES_COMPLETED				1000	hpc_e500_DLFB	"STs Completed"				""
DATA_L1_CACHE_LOCKS				1000	hpc_e500_DLFB	"L1 D-C Locks"				""
DATA_L1_CACHE_RELOADS				1000	hpc_e500_DLFB	"L1 D-C Reloads"			""
DATA_L1_CACHE_CASTOUTS				1000	hpc_e500_DLFB	"L1 D-C Castouts"			""
LOAD_MISS_WITH_DLFB_FULL_TD						1000	hpc_e500_DSRC_times_detected	"LD Miss DLFB Full"		""
LOAD_MISS_WITH_LOAD_QUEUE_FULL_TD					1000	hpc_e500_DSRC_times_detected	"LD Miss with LD Queue Full"	""
LOAD_GUARDED_MISS_WHEN_THE_LOAD_IS_NOT_YET_AT_THE_BOTTOM_OF_THE_CQ_TD	1000	hpc_e500_DSRC_times_detected	"LD Guarded Miss When the LD in not yet at the bottom of the CQ" ""
TRANSLATE_A_STORE_WHEN_THE_STORE_QUEUE_IS_FULL_TD			1000	hpc_e500_DSRC_times_detected	"Translate a ST when the ST Queue is Full"	""
ADDRESS_COLLISION_TD							1000	hpc_e500_DSRC_times_detected	"Address Collision"		""
DATA_MMU_MISS_TD							1000	hpc_e500_DSRC_times_detected	"Data MMU Miss"			""
DATA_MMU_BUSY_TD							1000	hpc_e500_DSRC_times_detected	"Data MMU Busy"			""
SECOND_PART_OF_MISALIGNED_ACCESS_WHEN_FIRST_PART_MISSED_IN_CACHE_TD	1000	hpc_e500_DSRC_times_detected	"2nd part of Misaligned Access When 1st Part Missed in Cache" ""
LOAD_MISS_WITH_DLFB_FULL_CS						1000	hpc_e500_DSRC_cycles_stalled	"LD Miss DLFB Full"		""
LOAD_MISS_WITH_LOAD_QUEUE_FULL_CS					1000	hpc_e500_DSRC_cycles_stalled	"LD Miss with LD Queue Full"	""
LOAD_GUARDED_MISS_WHEN_THE_LOAD_IS_NOT_YET_AT_THE_BOTTOM_OF_THE_CQ_CS	1000	hpc_e500_DSRC_cycles_stalled	"LD Guarded Miss When the LD in not yet at the bottom of the CQ" ""
TRANSLATE_A_STORE_WHEN_THE_STORE_QUEUE_IS_FULL_CS			1000	hpc_e500_DSRC_cycles_stalled	"Translate a ST when the ST Queue is Full"	""
ADDRESS_COLLISION_CS							1000	hpc_e500_DSRC_cycles_stalled	"Address Collision"		""
DATA_MMU_MISS_CS							1000	hpc_e500_DSRC_cycles_stalled	"Data MMU Miss"			""
DATA_MMU_BUSY_CS							1000	hpc_e500_DSRC_cycles_stalled	"Data MMU Busy"			""
SECOND_PART_OF_MISALIGNED_ACCESS_WHEN_FIRST_PART_MISSED_IN_CACHE_CS	1000	hpc_e500_DSRC_cycles_stalled	"2nd part of Misaligned Access When 1st Part Missed in Cache" ""
INSTRUCTION_L1_CACHE_LOCKS		1000	hpc_e500_F_IC_ILFB_IPE	"INST L1 C Locks"			""
INSTRUCTION_L1_CACHE_RELOADS_FROM_FETCH	1000	hpc_e500_F_IC_ILFB_IPE	"INST L1 C Reloads from Fetches"	""
NUMBER_OF_FETCHES			1000	hpc_e500_F_IC_ILFB_IPE	"Num of Fetches"			""
INSTRUCTION_MMU_TLB4K_RELOADS		1000	hpc_e500_InstMMU_DataMMU_L2MMU	"INST MMU TLB4k Reloads"	""
INSTRUCTION_MMU_VSP_RELOADS		1000	hpc_e500_InstMMU_DataMMU_L2MMU	"INST MMU VSP Reloads"		""
DATA_MMU_TLB4K_RELOADS			1000	hpc_e500_InstMMU_DataMMU_L2MMU	"Data MMU TLB4k Reloads"	""
DATA_MMU_VSP_RELOADS			1000	hpc_e500_InstMMU_DataMMU_L2MMU	"Data MMU VSP Reloads"		""
L2_MMU_MISSES				1000	hpc_e500_InstMMU_DataMMU_L2MMU	"L2 MMU Miss"			""
BIU_MASTER_REQUESTS			1000	hpc_e500_BIU	"Master Req"			""
BIU_MASTER_INSTRUCTION_SIDE_REQUESTS	1000	hpc_e500_BIU	"Master INST Side Req"		""
BIU_MASTER_DATA_SIDE_REQUESTS		1000	hpc_e500_BIU	"Master Data Side Req"		""
BIU_MASTER_DATA_SIDE_CASTOUT_REQUESTS	1000	hpc_e500_BIU	"Master Data Side Castout Req"	""
BIU_MASTER_RETRIES			1000	hpc_e500_BIU	"Master Retries"		""
SNOOP_REQUESTS				1000	hpc_e500_snoop	"Snoop Req"	""
SNOOP_HITS				1000	hpc_e500_snoop	"Snoop Hits"	""
SNOOP_PUSHES				1000	hpc_e500_snoop	"Snoop Pushes"	""
SNOOP_RETRIES				1000	hpc_e500_snoop	"Snoop Retries"	""
INTERRUPTS_TAKEN			1000	hpc_e500_interrupts	"Ints Taken"			""
EXTERNAL_INPUT_INTERRUPTS_TAKEN		1000	hpc_e500_interrupts	"Ext Ints Taken"		""
CRITICAL_INPUT_INTERRUPTS_TAKEN		1000	hpc_e500_interrupts	"Cri Ints Taken"		""
SYSTEM_CALL_AND_TRAP_INTERRUPTS		1000	hpc_e500_interrupts	"Sys Call and Trap Ints"	""
V2_TRANSITIONS_OF_TBL_BIT_SELECTED_BY_PMGC0_TBSEL	1000	hpc_e500v2	"Transitions of TLB bit Selected by PMGC0 TBSEL"	""
