#!/usr/bin/env perl

use strict;
use IO::File;
use Text::ParseWords;

my @data = <DATA>;

my $header_file = new IO::File ">hpc_e500mc.type"
  or die "cannot open file: $!";

my $host_file = new IO::File ">hpc_e500mc.host"
  or die "cannot open file: $!";

my $header_comment1 = <<END_COMMENT;
#include "cafe.type"

END_COMMENT

# Example output:
# #define HPC_E500MC___HPC___E500MC_PROCESSOR_CYCLES_VT_DEF "S[hpc.e500mc_processor_cycles],,n[cafe.COUNTER_METRIC_CLASS,1]n[cafe.LABEL,\"TOT CYC#\"]]"
# #define HPC_E500MC___HPC___E500MC_PROCESSOR_CYCLES_VT_REF "t[hpc.e500mc_processor_cycles]"

sub print_header_file_entry {
  my (@words) = @_;

  my $s1 = sprintf("struct hpc.e500mc_%s\n\t:cafe.COUNTER_METRIC_CLASS(1)\n\t:cafe.LABEL(\"%s\") {\n}\n",
    uc($words[0]), $words[3]);

  print $header_file "$s1\n";
}

sub print_host_entry {
  my (@words) = @_;
  my $s1 = sprintf("set hpc.e500mc_%s_count n[category.%s]n[cafeui.TOOLTIP,\"%s\"]n[formatter.big]n[cafe.ADVANCED_METRIC]", uc($words[0]), $words[2], $words[4]);
  print $host_file "$s1\n";
}

print $header_file $header_comment1;

foreach (@data) {
  my @words = shellwords($_);
  print_header_file_entry(@words);
}

foreach (@data) {
  my @words = shellwords("$_");
  print_host_entry(@words);
}

# VT name, min reload value, UI metric category, UI metric name (string), UI tooltip (string)

__DATA__
PROCESSOR_CYCLES			1000	hpc_e500mc_general	"Processor CYCs"		""
INSTRUCTIONS_COMPLETED			1000	hpc_e500mc_general	"INST Completed"		""
MICRO_OPS_COMPLETED			1000	hpc_e500mc_general	"Micro-ops Completed"		""
INSTRUCTIONS_FETCHED			1000	hpc_e500mc_general	"Instructions Fetched"		""
MICRO_OPS_DECODED			1000	hpc_e500mc_general	"Micro-ops Decoded"		""
PM_EVENT_TRANSITIONS			1000	hpc_e500mc_general	"PM_EVENT Transitions"		""
PM_EVENT_CYCLES				1000	hpc_e500mc_general	"PM_EVENT CYCs"			""
BRANCH_INSTRUCTIONS_COMPLETED		1000	hpc_e500mc_inst_types_completed	"BR INST Completed"		""
LOAD_MICRO_OPS_COMPLETED		1000	hpc_e500mc_inst_types_completed	"LD Micro-ops Completed"	""
STORE_MICRO_OPS_COMPLETED		1000	hpc_e500mc_inst_types_completed	"ST Micro-ops Completed"	""
NUMBER_OF_CQ_REDIRECTS			1000	hpc_e500mc_inst_types_completed	"NUM of CQ Redirects"		""
BRANCHES_FINISHED						1000	hpc_e500mc_br_pred_and_exec	"BR Finished"			""
TAKEN_BRANCHES_FINISHED						1000	hpc_e500mc_br_pred_and_exec	"Taken BR Finished"		""
FINISHED_UNCONDITIONAL_BRANCHES_THAT_MISS_THE_BTB		1000	hpc_e500mc_br_pred_and_exec	"Finished UN-COND BR That Miss the BTB" ""
BRANCHES_MISPREDICTED						1000	hpc_e500mc_br_pred_and_exec	"BR Miss"			""
BRANCHES_IN_THE_BTB_MISPREDICTED_DUE_TO_DIRECTION_PREDICTION	1000	hpc_e500mc_br_pred_and_exec	"BR in the BTB Mispredicted due to direction prediction" ""
BTB_HITS_AND_PSEUDO_HITS					1000	hpc_e500mc_br_pred_and_exec	"BTB hits and pseudo-hits"	""
CYCLES_DECODE_STALLED			1000	hpc_e500mc_pipe_stalls	"CYC Decode Stalled"		""
CYCLES_ISSUE_STALLED			1000	hpc_e500mc_pipe_stalls	"CYC Issue Stalled"		""
CYCLES_BRANCH_ISSUE_STALLED		1000	hpc_e500mc_pipe_stalls	"CYC BR Issue Stalled"		""
CYCLES_SFX0_SCHEDULE_STALLED		1000	hpc_e500mc_pipe_stalls	"Cycles SFX0 schedule stalled"		"Cycles SFX0 is not empty but 0 instructions scheduled"
CYCLES_SFX1_SCHEDULE_STALLED		1000	hpc_e500mc_pipe_stalls	"Cycles SFX1 schedule stalled"		"Cycles SFX1 is not empty but 0 instructions scheduled"
CYCLES_MU_SCHEDULE_STALLED		1000	hpc_e500mc_pipe_stalls	"CYC MU SCHD Stalled"		""
CYCLES_LRU_SCHEDULE_STALLED		1000	hpc_e500mc_pipe_stalls	"CYC LRU SCHD Stalled"		""
CYCLES_BU_SCHEDULE_STALLED		1000	hpc_e500mc_pipe_stalls	"CYC BU SCHD Stalled"		""
TOTAL_TRANSLATED				1000	hpc_e500mc_DLFB	"TOT Translated"			""
LOADS_TRANSLATED				1000	hpc_e500mc_DLFB	"LDs Thranslted"			""
STORES_TRANSLATED				1000	hpc_e500mc_DLFB	"STs Translated"			""
TOUCHES_TRANSLATED				1000	hpc_e500mc_DLFB	"Touches Translated"			""
CACHEOPS_TRANSLATED				1000	hpc_e500mc_DLFB	"Cache-Ops Translated"			""
CACHE_INHIBITED_ACCESSES_TRANSLATED		1000	hpc_e500mc_DLFB	"Cache Inhibited Access Translated"	""
GUARDED_LOADS_TRANSLATED			1000	hpc_e500mc_DLFB	"Guarded LDs translated"		""
WRITE_THROUGH_STORES_TRANSLATED			1000	hpc_e500mc_DLFB	"WR Thro STs Translated"		""
MISALIGNED_LOAD_OR_STORE_ACCESSES_TRANSLATED	1000	hpc_e500mc_DLFB	"Misaligned LD or ST Access Translated"	""
TOTAL_ALLOCATED_TO_DLFB				1000	hpc_e500mc_DLFB	"TOT Alloc to DLFB"			""
LOADS_TRANSLATED_AND_ALLOCATED_TO_DLFB		1000	hpc_e500mc_DLFB	"LDs Translated and Alloc to DLFB"	""
STORES_COMPLETED_AND_ALLOCATED_TO_DLFB		1000	hpc_e500mc_DLFB	"STs Translated and Alloc to DLFB"	""
TOUCHES_TRANSLATED_AND_ALLOCATED_TO_DLFB	1000	hpc_e500mc_DLFB	"Touches Translated and Alloc to DLFB"	""
STORES_COMPLETED				1000	hpc_e500mc_DLFB	"STs Completed"				""
DATA_L1_CACHE_LOCKS				1000	hpc_e500mc_DLFB	"L1 D-C Locks"				""
DATA_L1_CACHE_RELOADS				1000	hpc_e500mc_DLFB	"L1 D-C Reloads"			""
DATA_L1_CACHE_CASTOUTS				1000	hpc_e500mc_DLFB	"L1 D-C Castouts"			""
LOAD_MISS_WITH_DLFB_FULL_TD						1000	hpc_e500mc_DSRC_times_detected	"LD Miss DLFB Full"		""
LOAD_MISS_WITH_LOAD_QUEUE_FULL_TD					1000	hpc_e500mc_DSRC_times_detected	"LD Miss with LD Queue Full"	""
LOAD_GUARDED_MISS_WHEN_THE_LOAD_IS_NOT_YET_AT_THE_BOTTOM_OF_THE_CQ_TD	1000	hpc_e500mc_DSRC_times_detected	"LD Guarded Miss When the LD in not yet at the bottom of the CQ" ""
TRANSLATE_A_STORE_WHEN_THE_STORE_QUEUE_IS_FULL_TD			1000	hpc_e500mc_DSRC_times_detected	"Translate a ST when the ST Queue is Full"	""
ADDRESS_COLLISION_TD							1000	hpc_e500mc_DSRC_times_detected	"Address Collision"		""
DATA_MMU_MISS_TD							1000	hpc_e500mc_DSRC_times_detected	"Data MMU Miss"			""
DATA_MMU_BUSY_TD							1000	hpc_e500mc_DSRC_times_detected	"Data MMU Busy"			""
SECOND_PART_OF_MISALIGNED_ACCESS_WHEN_FIRST_PART_MISSED_IN_CACHE_TD	1000	hpc_e500mc_DSRC_times_detected	"2nd part of Misaligned Access When 1st Part Missed in Cache" ""
LOAD_MISS_WITH_DLFB_FULL_CS						1000	hpc_e500mc_DSRC_cycles_stalled	"LD Miss DLFB Full"		""
LOAD_MISS_WITH_LOAD_QUEUE_FULL_CS					1000	hpc_e500mc_DSRC_cycles_stalled	"LD Miss with LD Queue Full"	""
LOAD_GUARDED_MISS_WHEN_THE_LOAD_IS_NOT_YET_AT_THE_BOTTOM_OF_THE_CQ_CS	1000	hpc_e500mc_DSRC_cycles_stalled	"LD Guarded Miss When the LD in not yet at the bottom of the CQ" ""
TRANSLATE_A_STORE_WHEN_THE_STORE_QUEUE_IS_FULL_CS			1000	hpc_e500mc_DSRC_cycles_stalled	"Translate a ST when the ST Queue is Full"	""
ADDRESS_COLLISION_CS							1000	hpc_e500mc_DSRC_cycles_stalled	"Address Collision"		""
DATA_MMU_MISS_CS							1000	hpc_e500mc_DSRC_cycles_stalled	"Data MMU Miss"			""
DATA_MMU_BUSY_CS							1000	hpc_e500mc_DSRC_cycles_stalled	"Data MMU Busy"			""
SECOND_PART_OF_MISALIGNED_ACCESS_WHEN_FIRST_PART_MISSED_IN_CACHE_CS	1000	hpc_e500mc_DSRC_cycles_stalled	"2nd part of Misaligned Access When 1st Part Missed in Cache" ""
INSTRUCTION_L1_CACHE_LOCKS		1000	hpc_e500mc_F_IC_ILFB_IPE	"INST L1 C Locks"			""
INSTRUCTION_L1_CACHE_RELOADS_FROM_FETCH	1000	hpc_e500mc_F_IC_ILFB_IPE	"INST L1 C Reloads from Fetches"	""
NUMBER_OF_FETCHES			1000	hpc_e500mc_F_IC_ILFB_IPE	"Num of Fetches"			""
INSTRUCTION_MMU_TLB4K_RELOADS		1000	hpc_e500mc_InstMMU_DataMMU_L2MMU	"INST MMU TLB4k Reloads"	""
INSTRUCTION_MMU_VSP_RELOADS		1000	hpc_e500mc_InstMMU_DataMMU_L2MMU	"INST MMU VSP Reloads"		""
DATA_MMU_TLB4K_RELOADS			1000	hpc_e500mc_InstMMU_DataMMU_L2MMU	"Data MMU TLB4k Reloads"	""
DATA_MMU_VSP_RELOADS			1000	hpc_e500mc_InstMMU_DataMMU_L2MMU	"Data MMU VSP Reloads"		""
L2_MMU_MISSES				1000	hpc_e500mc_InstMMU_DataMMU_L2MMU	"L2 MMU Miss"			""
BIU_MASTER_REQUESTS			1000	hpc_e500mc_BIU	"Master Req"			""
BIU_MASTER_INSTRUCTION_SIDE_REQUESTS	1000	hpc_e500mc_BIU	"Master INST Side Req"		""
BIU_MASTER_DATA_SIDE_REQUESTS		1000	hpc_e500mc_BIU	"Master Data Side Req"		""
BIU_MASTER_DATA_SIDE_CASTOUT_REQUESTS	1000	hpc_e500mc_BIU	"Master Data Side Castout Req"	""
SNOOP_REQUESTS				1000	hpc_e500mc_snoop	"Snoop Req"	""
SNOOP_HITS				1000	hpc_e500mc_snoop	"Snoop Hits"	""
SNOOP_PUSHES				1000	hpc_e500mc_snoop	"Snoop Pushes"	""
INTERRUPTS_TAKEN			1000	hpc_e500mc_interrupts	"Ints Taken"			""
EXTERNAL_INPUT_INTERRUPTS_TAKEN		1000	hpc_e500mc_interrupts	"Ext Ints Taken"		""
CRITICAL_INPUT_INTERRUPTS_TAKEN		1000	hpc_e500mc_interrupts	"Cri Ints Taken"		""
SYSTEM_CALL_AND_TRAP_INTERRUPTS		1000	hpc_e500mc_interrupts	"Sys Call and Trap Ints"	""
TRANSITIONS_OF_TBL_BIT_SELECTED_BY_PMGC0_TBSEL	1000	hpc_e500mc_misc	"Transitions of TLB bit Selected by PMGC0[TBSEL]" "Counts transitions of the TBL bit selected by PMGC0[TBSEL]"
L2_LINEFILL_BUFFER				1000	hpc_e500mc_misc	"L2 Linefill Buffer"	"Linefill requests to L2"
L2_VS						1000	hpc_e500mc_misc	"L2 VS"			"Victim selects to L2"
CASTOUTS_RELEASED				1000	hpc_e500mc_misc	"Castouts Released"	"Speculative reservations in castout buffer that are not needed"
INTV_ALLOCATIONS				1000	hpc_e500mc_misc	"INTV Allocs"		"Allocations to INTV queue"
STORE_RETRIES_TO_MBAR				1000	hpc_e500mc_misc	"ST Retries to MBAR"	"DLFB retries to MBAR"
STORE_RETRIES_DUE_TO_MISC			1000	hpc_e500mc_misc	"ST Retries due to Misc" "Retries to store queue, excluding MBAR case"
STASH_L1_HIT			1000	hpc_e500mc_stashing	"Stash L1 Hit"			"Stash hits in L1"
STASH_L2_HIT			1000	hpc_e500mc_stashing	"Stash L2 Hit"			"Stash hits in L2"
STASH_BUSY_1			1000	hpc_e500mc_stashing	"Stash Busy 1"			"Cycles stash 1 DLFB busy"
STASH_BUSY_2			1000	hpc_e500mc_stashing	"Stash Busy 2"			"Cycles stash 2 DLFB’s busy"
STASH_BUSY_3			1000	hpc_e500mc_stashing	"Stash Busy 3"			"Cycles stash 3 DLFB’s busy"
STASH_HIT			1000	hpc_e500mc_stashing	"Stash Hit"			"Access hits on stash DLFB"
STASH_HIT_DLFB			1000	hpc_e500mc_stashing	"Stash Hit DLFB"		"Stash hits to a DLFB"
STASH_DOWNGRADE_1		1000	hpc_e500mc_stashing	"Stash Downgrade 1"		"Condition 1 downgrades"
STASH_DOWNGRADE_2		1000	hpc_e500mc_stashing	"Stash Downgrade 2"		"Condition 2 downgrades"
STASH_REQUESTS			1000	hpc_e500mc_stashing	"Stash Requests"		"Stash requests"
STASHES_TO_L1_DATA_CACHE	1000	hpc_e500mc_stashing	"Stashes to L1 D-C"		"Stash requests to L1"
STASHES_TO_BACKSIDE_L2		1000	hpc_e500mc_stashing	"Stashes to Backside L2"	"Stash requests to L2"
STALLS_DUE_TO_NO_CAQ_OR_COB	1000	hpc_e500mc_stashing	"Stalls due to no CAQ or COB"	"Stalls due to no CAQ or COB"
L2_CACHE_ACCESSES			1000	hpc_e500mc_backside_l2	"L2-C accesses"			"L2 cache accesses, which include the following: load, store, fetch, dcba*, dcbz*, dcblc CT = 1, icblc CT=1, dcblc CT=2, icblc CT=2,dcbf*, dcbst*, dcbi, CI store, icbi*, lwarx, stwcx., write-thoughstore, CI stwcx., mbar, sync, tlbsync, tlbivax, tlbilx, prefetch requests(dcbt, dcbtst, dcbtls, dcbtstls CT = 0,1,2 & icbt CT=1, 2 & icbtls CT = 0,1,2), L2 cache allocation"
L2_HIT_CACHE_ACCESSES			1000	hpc_e500mc_backside_l2	"L2-C Hit accesses"		"L2 cache accesses, which include the following: load, store, fetch, dcba*, dcbz*, dcblc CT = 1, icblc CT=1, dcblc CT=2, icblc CT=2, dcbf*, dcbst*, dcbi, CI store, icbi*, lwarx, stwcx., write-though store, CI stwcx., mbar, sync, tlbsync, tlbivax, tlbilx, prefetch requests(dcbt, dcbtst, dcbtls, dcbtstls CT = 0,1,2 & icbt CT=1, 2 & icbtls CT = 0,1,2) && L2_hit"
L2_CACHE_DATA_ACCESSES			1000	hpc_e500mc_backside_l2	"L2 D-C accesses"		"L2 cache data accesses"
L2_CACHE_DATA_HITS			1000	hpc_e500mc_backside_l2	"L2 D-C hits"			"L2 cache data hits"
L2_CACHE_INSTRUCTION_ACCESSES		1000	hpc_e500mc_backside_l2	"L2 I-C accesses"		"L2 cache instruction accesses"
L2_CACHE_INSTRUCTION_HITS		1000	hpc_e500mc_backside_l2	"L2 I-C hits"			"L2 cache instruction hits"
L2_CACHE_ALLOCATIONS			1000	hpc_e500mc_backside_l2	"L2-C Allocations"		"L2 cache allocations: castouts from L1 (clean or modified), dcbt, dcbtst CT=2, dcbtls, dcbtstls CT=2, icbt CT=2, icbtls CT=2, fetch"
L2_CACHE_DATA_ALLOCATIONS		1000	hpc_e500mc_backside_l2	"L2 D-C Allocations"		"L2 cache data allocations castouts from L1, dcbt/dcbtst CT=2, dcbtls/dcbtstls CT=2"
L2_CACHE_DIRTY_DATA_ALLOCATIONS		1000	hpc_e500mc_backside_l2	"L2-C Dirty Data Allocs" 	"L2 cache dirty data allocations"
L2_CACHE_INSTRUCTION_ALLOCATIONS	1000	hpc_e500mc_backside_l2	"L2 I-C Allocs"	"L2 cache instruction allocations fetch, icbtls CT=2, icbt CT=2"
L2_CACHE_UPDATES			1000	hpc_e500mc_backside_l2	"L2-C Updates"			"L2 cache updates"
L2_CACHE_CLEAN_UPDATES			1000	hpc_e500mc_backside_l2	"L2-C Clean Updates"		"L2 cache clean updates"
L2_CACHE_DIRTY_UPDATES			1000	hpc_e500mc_backside_l2	"L2-C Dirty Updates"		"L2 cache dirty updates"
L2_CACHE_CLEAN_REDUNDANT_UPDATES	1000	hpc_e500mc_backside_l2	"L2-C Clean Redundant Updates"	"L2 cache clean redundant updates"
L2_CACHE_DIRTY_REDUNDANT_UPDATES	1000	hpc_e500mc_backside_l2	"L2-C Dirty Redundant Updates"	"L2 cache dirty redundant updates"
L2_CACHE_LOCKS				1000	hpc_e500mc_backside_l2	"L2-C Locks"			"L2 cache locks"
L2_CACHE_CASTOUTS			1000	hpc_e500mc_backside_l2	"L2-C Castouts"			"L2 cache castouts"
L2_CACHE_DATA_DIRTY_HITS		1000	hpc_e500mc_backside_l2	"L2 D-C Dirty Hits"		"L2 cache data dirty hits"
INSTRUCTION_LFB_WENT_HIGH_PRIORITY	1000	hpc_e500mc_backside_l2	"Inst lfb went High Pri"	"Instruction lfb went high priority"
SNOOP_THROTTLING_TURNED_ON		1000	hpc_e500mc_backside_l2	"Snoop Throttling Turned On"	"Snoop throttling turned on"
L2_INVALIDATION_OF_CLEAN_LINES		1000	hpc_e500mc_backside_l2	"L2 Inval of Clean Lines"	"L2 invalidation of clean lines"
L2_INVALIDATION_OF_INCOHERENT_LINE	1000	hpc_e500mc_backside_l2	"L2 Inval of Incoherent Lines"	"L2 invalidation of Incoherent line"
L2_INVALIDATION_OF_COHERENT_LINE	1000	hpc_e500mc_backside_l2	"L2 Inval of Coherent lines"	"L2 invalidation of coherent line"
COHERENT_LOOKUP_MISS_DUE_TO_VALID_BUT_INCOHERENT_ADDRESS_MATCHES 1000	hpc_e500mc_backside_l2	"Coherent lookup miss due to valid but incoherent" "Coherent lookup miss due to valid but incoherent (address matches)"
IAC1S_DETECTED			1000	hpc_e500mc_iac_dac	"IAC1s Detected"	"Every valid IAC1 detection"
IAC2S_DETECTED			1000	hpc_e500mc_iac_dac	"IAC2s Detected"	"Every valid IAC2 detection"
DAC1S_DETECTED			1000	hpc_e500mc_iac_dac	"DAC1s Detected"	"Every valid DAC1 detection"
DAC2S_DETECTED			1000	hpc_e500mc_iac_dac	"DAC2s Detected"	"Every valid DAC2 detection"
DVT0_DETECTED			1000	hpc_e500mc_dvt		"DVT0 Detected"		"Detection of a write to DEVENT SPR with DVT0 set"
DVT1_DETECTED			1000	hpc_e500mc_dvt		"DVT0 Detected"		"Detection of a write to DEVENT SPR with DVT1 set"
DVT2_DETECTED			1000	hpc_e500mc_dvt		"DVT0 Detected"		"Detection of a write to DEVENT SPR with DVT2 set"
DVT3_DETECTED			1000	hpc_e500mc_dvt		"DVT0 Detected"		"Detection of a write to DEVENT SPR with DVT3 set"
DVT4_DETECTED			1000	hpc_e500mc_dvt		"DVT0 Detected"		"Detection of a write to DEVENT SPR with DVT4 set"
DVT5_DETECTED			1000	hpc_e500mc_dvt		"DVT0 Detected"		"Detection of a write to DEVENT SPR with DVT5 set"
DVT6_DETECTED			1000	hpc_e500mc_dvt		"DVT0 Detected"		"Detection of a write to DEVENT SPR with DVT6 set"
DVT7_DETECTED			1000	hpc_e500mc_dvt		"DVT0 Detected"		"Detection of a write to DEVENT SPR with DVT7 set"
CYCLES_COMPLETION_STALLED_NEXUS	1000	hpc_e500mc_dvt		"DVT0 Detected"		"Number of completion cycles stalled due to Nexus FIFO full"
FPU_DOUBLE_PUMP		1000	hpc_e500mc_fpu		"FPU Double Pump"	"Double pump penalized ops finished through the pipe. Counts once for every multiply family double pump operation"
FPU_FINISH		1000	hpc_e500mc_fpu		"FPU Finish"		"Number of FPU ops finished through the pipe. Also, includes fsel, fneg, mtf*, and mff*"
FPU_DIVIDE_CYCLES	1000	hpc_e500mc_fpu		"FPU Divide Cycles"	"Counts once for every cycle of divide execution (fdivs and fdiv)"
FPU_DENORM_INPUT	1000	hpc_e500mc_fpu		"FPU Denorm Input"	"Counts extra cycles delay due to denormalized inputs. If there is one, this is incremented 4 times, Two operands increments it 5 times. This shows the real penalty due to denorms, not just how often they occur"
FPU_RESULT_STALL	1000	hpc_e500mc_fpu		"FPU Result Stall"	"Counts extra cycles due to denorm results, overflow, mass cancellation, zero results, carry-in mispredict, exponent range check"
FPU_FPSCR_FULL_STALL	1000	hpc_e500mc_fpu		"FPU FPSCR Full Stall"	"Number of cycles stalled due to FPSCR full issue"
FPU_PIPE_SYNC_STALL	1000	hpc_e500mc_fpu		"FPU Pipe Sync Stall"	"Synchronization-op stalls: count once for each cycle that a break-before FPU is in the RS/issue stage but cannot issue. Also count once for each cycle that an FPU op is in the RS/issue stage but cannot issue due to “break-after”: of an FPU op currently in progress"
FPU_INPUT_DATA_STALL	1000	hpc_e500mc_fpu		"FPU Input Data Stall"	"FPU data-ready stall: cycles in which there is an op in the RS/issue stage that cannot issue because one or more of its operands is not yet available"
DECORATED_LOADS		1000	hpc_e500mc_ext_ld_st	"Decorated Loads"	"Number of decorated loads"
DECORATED_STORES	1000	hpc_e500mc_ext_ld_st	"Decorated Stores"	"Number of decorated stores"
LOAD_RETRIES		1000	hpc_e500mc_ext_ld_st	"Load Retries"		"Number of load retries"
STWCX_SUCCESSES		1000	hpc_e500mc_ext_ld_st	"stwcx. Successes"	"Number of successful stwcx. instructions"
STWCX_UNSUCCESSFUL	1000	hpc_e500mc_ext_ld_st	"stwcx. Unsuccessful"	"Number of unsuccessful stwcx. instructions"
