/* hpcI86PerfIsr.c - i86 performance monitor interrupt handler  */

/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/i86/hpcI86vxw.h>
#include "../../hpc_utils.h"
#include "hpcI86PerfIsr.h"

static int initialised;
static int (*callback0)(void * pRegs, int cpu);
static int (*callback1)(void * pRegs, int cpu);

static int defaultCallback
    (
    void * pRegs,
    int cpu
    )
    {
    return 0;
    }

/*
 * HPC perf ISR is not reentrant, so ensure that interrupts are locked.
 */
int hpcI86PerfMonHandler
    (
    void * pRegs
    )
    {
    int p; /* processed */
    int cpu = vxCpuIndexGet();
    int key = intCpuLock();

    hpcVxwI86PerfMonIsrAck();

    p = (*callback0)(pRegs, cpu);
    p += (*callback1)(pRegs, cpu);

    intCpuUnlock(key);
    return p;
    }

void hpcI86PerfIsrHandler
    (
    int id,
    int (*callback)(void * pRegs, int cpu)
    )
    {
    if (initialised == 0)
        {
        initialised = 1;
        callback0 = callback1 = defaultCallback;
        }
    if (id == 0)
        {
        if (callback == NULL)
            callback0 = defaultCallback;
        else
            callback0 = callback;
        }
    else
        {
        if (callback == NULL)
            callback1 = defaultCallback;
        else
            callback1 = callback;
        }
    }
