/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#define _ASMLANGUAGE
#include "vxWorks.h"
#include "asm.h"
#include "regs.h"

#if (_VX_CPU_FAMILY == _VX_I80X86)

#include "arch/i86/pentiumLib.h"

	/* globals */
	.globl	FUNC(i86cpuid)
	.globl	GTEXT(hpcI86PerfMonVector)

	.globl	FUNC(hpcI86PerfMonHandler)

	.extern	FUNC(intEnt)	    /* interrupt enter routine */
	.extern	FUNC(intExit)	    /* interrupt exit routine */

/*****************************************************************************
* void pm_pent_do_cpuid (unsigned int * eax, unsigned int * ebx,
*				unsigned int * ecx, unsigned int * edx)
*
* eax: in:  set to required cpuid functin number. Value is loaded to
*	    eax register before cpuid insytrction is executed
*      out: contents of eax register after cpuid executed
* ebx  out: contents of ebx register after cpuid executed
* ecx  out: contents of ecx register after cpuid executed
* edx  out: contents of edx register after cpuid executed
*
* Stack layout:
*
*     |             | high mem
*     +-------------+
*     |param4 (&edx)|
*     |param3 (&ecx)|
*     |param2 (&ebx)|
*     |param1 (&eax)|
*     |return addrs | <-esp, on entry to this function
*     +-------------+
*     |reg ebx      | <-esp, on entry, non vol ebx is preserved
*     |reg eax      | <-esp, save eax value returned by cpuid
*     |reg edx      | <-esp, save edx value returned by cpuid
*     +-------------+
*     |             | low mem
*/
#if !defined(_WRS_CONFIG_LP64)
        .balign 16,0x90
FUNC_LABEL(i86cpuid)
	pushl	%ebx			/* preserve non-vol register */

	movl	0x8(%esp), %eax		/* Get value of input param */
	movl	(%eax), %eax

	cpuid
	pushl	%eax		/* save copy of eax returned by cpuid */
	pushl	%edx		/* save copy of edx returned by cpuid */

	/* save eax output param */
	movl	0x10(%esp), %eax
	movl	%eax, %edx
	movl	0x4(%esp), %eax
	movl	%eax, (%edx)

	/* save ebx output param */
	movl	0x14(%esp), %eax
	movl	%eax, %edx
	movl	%ebx, (%edx)

	/* save ecx output param */
	movl	0x18(%esp), %eax
	movl	%eax, %edx
	movl	%ecx, (%edx)

	/* save edx output param */
	movl	0x1C(%esp), %eax
	movl	%eax, %edx
	movl	(%esp), %eax
	movl	%eax, (%edx)

	/* clean up stack and return */
	add	$0x8, %esp
	popl	%ebx			/* restore ebx */
	ret

#else /* _WRS_CONFIG_LP64 version */

/*****************************************************************************
* void pm_pent_do_cpuid (unsigned int * eax, unsigned int * ebx,
*				unsigned int * ecx, unsigned int * edx)
* sizeof(unsigned int) = 4
* eax: passed in register rdi
* ebx: passed in register rsi
* ecx: passed in register rcx
* edx: passed in register rdx
*/
        .balign 16,0x90
FUNC_LABEL(i86cpuid)
	push	%rbx			/* preserve non-vol register */

	mov	(%rdi), %rax		/* Get *eax parameter to eax register */
	mov	%rcx, %r8		/* save rcx as cpuid uses it */
	mov	%rdx, %r9		/* save rdx as cpuid uses it */
	cpuid
	mov	%eax, (%rdi)		/* return eax */
	mov	%ebx, (%rsi)		/* return ebx */
	mov	%ecx, (%r9)		/* return ecx */
	mov	%edx, (%r8)		/* return edx */

	pop	%rbx			/* restore ebx */
	ret
#endif

/*****************************************************************************
* pm_pent_int_handler - Performance monitor interrupt handler.
*/

#if !defined(_WRS_CONFIG_LP64)
	.balign 16,0x90
FUNC_LABEL(hpcI86PerfMonVector)
	/* If this code changes, ensure the 597-102 code below is at the
	 * correct offset as described */
	call	intEnt
	pushal
	pushl	%esp 			/* push param1 of pmP4intHandler() */
	call	FUNC(hpcI86PerfMonHandler) /* Call performance counter handler */
	addl	$4, %esp
	popal
	jmp	intExit

	/* System viewer int enter event looks up an interrupt number at an
	 * offset of 28 bytes following the intEnt call above. See
	 * INT_CONNECT_CODE29 in System Viewer code.
	 */
	.balign 8
	.byte 0x00
	.byte 0x00
	.byte 0x00
	.byte 0x00
	.byte 0x00
	.long 597-102-102 /* INT_LEVEL(597 - EVENT_INT_ENTER): value must be */
				/* 28 bytes after call to intEnt above */

#else /* _WRS_CONFIG_LP64 */

	.balign 16,0x90
FUNC_LABEL(hpcI86PerfMonVector)
	/* If this code changes, ensure the 597-102 code below is at the
	 * correct offset as described */
	call	intEnt
	jmp	hpcI86PerfMonVector2
	/* System viewer int enter event looks up an interrupt number at an
	 * offset of 47 bytes following the intEnt call above. See
	 * ICC_INT_ENT_RETURN in System Viewer code.
	 */
	.quad 0x0 /* 8 bytes */
	.quad 0x0
	.quad 0x0
	.quad 0x0
	.quad 0x0
	.byte 0x0
	.byte 0x0
	.byte 0x0
	.byte 0x0
	.byte 0x0
	.long 597-102-102 /* INT_LEVEL(597 - EVENT_INT_ENTER): value must be */
				/* 47 bytes after call to intEnt above */

	.balign 16
hpcI86PerfMonVector2:
	pushal			/* this gets expanded to multiple MOVs */
	mov	%rsp, %rdi	/* Pass SP as param to hpcI86PerfMonHandler */
	call	FUNC(hpcI86PerfMonHandler) /* Call performance counter handler */
	popal
	jmp	intExit


#endif /* _WRS_CONFIG_LP64 */
#endif /* (_VX_CPU_FAMILY == _VX_I80X86) */
