/* Copyright (c) 2011 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdlib.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/hpc_drv/ppc/e500Drv.h>
#include <hpc/drv_evt_cfg/ppc/e500vxw.h>
#include <hpc/drv_evt_cfg/ppc/e500DrvSet.h>
#include <hpc/drv_evt_cfg/ppc/e500DrvEvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500DeviceDrvCfg.h>

#include "e500DeviceDrvCfgP.h"
#include "../../hpc_utils.h"

static void * (*pMalloc)(size_t);
static void (*pFree)(void *);
static int e500PerfIsr(void * pEsf);
static struct hpcDriverConfig e500DeviceHpcDriverConfig;

/******************************************************************************
*/

static int defaultIsrHandler
    (
    struct driverInstE500 * i,
    void * pContext,
    int cpu
    )
    {
    return 0;
    }

/******************************************************************************
*/

static int (*pIsrHandler)(struct driverInstE500 *, void *, int) = defaultIsrHandler;

/******************************************************************************
*/

static STATUS enablePerfIsr
    (
    int (*pHandler)(struct driverInstE500 *, void *, int)
    )
    {
    pIsrHandler = pHandler;
    return hpcVxwDevicePerfIsrEnable (e500PerfIsr);
    }

/******************************************************************************
*/

static STATUS disablePerfIsr (void)
    {
    STATUS status;

    if ((status = hpcVxwDevicePerfIsrDisable (e500PerfIsr)) != OK)
        {
        return status;
        }

    pIsrHandler = defaultIsrHandler;
    return status;
    }

/******************************************************************************
*/

struct hpcDriverConfig * hpcCafeE500DeviceDrvCfgGet(void)
    {
    return &e500DeviceHpcDriverConfig;
    }

/******************************************************************************
*/

STATUS hpcCafeE500DeviceDrvCfgInit
    (
    void * (*pMallocFunc)(size_t),
    void   (*pFreeFunc)(void *),
    struct e500EventDescription *pEvtDesc,
    const unsigned int numEvents,
    struct driverInstE500 *pDrvInst,
    EVENT_SETTING_GET_FUNC pSettingGetFunc
    )
    {
    size_t size;

    pMalloc = pMallocFunc;
    pFree = pFreeFunc;

    pDrvInst->pmcRegSet = devPmcRegSet;
    pDrvInst->pmcRegGet = devPmcRegGet;
    pDrvInst->pmlcaRegSet = devPmlcaRegSet;
    pDrvInst->pmlcaRegGet = devPmlcaRegGet;
    pDrvInst->pmlcbRegSet = devPmlcbRegSet;
    pDrvInst->pmlcbRegGet = devPmlcbRegGet;
    pDrvInst->pmgc0RegSet = devPmgc0RegSet;
    pDrvInst->pmgc0RegGet = devPmgc0RegGet;
    pDrvInst->enablePerfIsr = enablePerfIsr;
    pDrvInst->disablePerfIsr = disablePerfIsr;

    e500DeviceHpcDriverConfig.numCpuCores = 1;
    e500DeviceHpcDriverConfig.numEvents = numEvents;
    e500DeviceHpcDriverConfig.pDrvInst = pDrvInst;
    e500DeviceHpcDriverConfig.pFuncs = &hpcHwDrvE500Rtns;
    e500DeviceHpcDriverConfig.pEventSettingsGet = pSettingGetFunc;
    e500DeviceHpcDriverConfig.eventSettingSize = sizeof(struct e500EventDescription);
    e500DeviceHpcDriverConfig.pEventDesc = pEvtDesc;

    size = sizeof(struct hpcHwCtrData *) * pDrvInst->numCounters;
    pDrvInst->ppDrvHwCtrData = (*pMalloc)(size);

    if (pDrvInst->ppDrvHwCtrData == NULL)
	return ERROR;

    memset(pDrvInst->ppDrvHwCtrData, 0, size);

    return OK;
    }

/******************************************************************************
*/

STATUS hpcCafeE500DeviceDrvCfgFinit
    (
    struct hpcDriverConfig *pHpcDriverConfig
    )
    {
    struct driverInstE500 *pDrvInst = pHpcDriverConfig->pDrvInst;

    if (pDrvInst->ppDrvHwCtrData == NULL)
        return OK;

    (*pFree)(pDrvInst->ppDrvHwCtrData);
    pDrvInst->ppDrvHwCtrData = NULL;

    return OK;
    }

/******************************************************************************
* The perf ISR is not reentrant, so this routine locks interrupts before
* calling it.
* For device counters there is no interrupted register context provided
* by the OS, so orthogonal data items that rely on those being provided like
* stacktrace are not be available.
* Since device HPC interrupts always originate on the core where the external
* interrupts are handled, orthogonal data core id parameter is set to 0.
*/

static int e500PerfIsr
    (
    void * pEsf
    )
    {
    int p; /* processed */
    int key = intCpuLock();
    p = (*pIsrHandler)(e500DeviceHpcDriverConfig.pDrvInst, NULL, 0);
    intCpuUnlock(key);
    return p;
    }

/******************************************************************************
*/

struct e500DeviceEventsPriv e500DeviceEventsPrivTest =
    {
    e500PerfIsr
    };
