/* Copyright (c) 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
15aug14,wzc  fix jenkins test failure.
16jul14,wzc  fix counter wrap value.
28mar14,sdt  fixed build issue in tcf-tools env.
13mar14,wzc  support arm hpc(US17715)
*/

/*
DESCRIPTION
*/

#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/hpc_drv/arm/armDrv.h>

/* Array index for a 2 dimensional array */
#define A2D(ROW,COLUMN,NCOLUMNS) (ROW * NCOLUMNS + COLUMN)
#define COUNTER_WRAP_32	(0xffffffff)

struct hpcHwCtrData
    {
    void *		pClient;
    HPC_COUNTER		countValue;
    int			       counter;
    HPC_COUNTER		reloadValue;
    HPC_COUNTER		counterWrap;
    };

static STATUS hpcHwDriverShow
    (
    void * pDrvInst
    )
    {
    struct driverInstArm * pInst = pDrvInst;
    int counter;
    ARM_PMU_INFO info;
    int state, interrupt, overflow;
    unsigned int count;

    pInst->getArmPmuInfo(&info);
    printf ("ARM PMU IMP 0x%x, IDCODE 0x%x, Counters Number %d,  State %d\n", 
            info.imp, info.idcode, info.numCounters, info.state);

    for (counter = 0; counter < pInst->numCounters; counter++) {
        pInst->getCounterState(counter, &state);
        pInst->getEvtCount(counter, &count);
        pInst->getCounterIntState(counter, &interrupt);
        pInst->getCounterOverflow(counter, &overflow);
        if (state) {
            int evtCode;
            pInst->getCounterEvent(counter, &evtCode);
            printf ("Counter %02d State %01d, evtCode %d, Count 0x%08x, Int %d, Overflow %d\n",
                 counter, state, evtCode, count, interrupt, overflow);
        } else
            printf ("Counter %02d State %01d, evtCode 0, Count 0x%08x, Int %d, Overflow %d\n",
                 counter, state, count, interrupt, overflow);
    }
    return OK;
    }

/******************************************************************************
*/

static void counterResetAll
    (
    struct driverInstArm * pInst
    )
    {
    pInst->resetArmPmuAllCounters();
    }

/******************************************************************************
*/
static int pmuCounterIntHandler
    (
    struct driverInstArm * pInst,
    void * pRegs,
    int cpu
    )
    {
    int eventsProcessed = 0;
    int c; /* counter */
    HPC_COUNTER eventCount;
    struct hpcHwCtrData * pCtr;

    for (c = 0; c < pInst->numCounters; c++)
    {
        size_t idx2d = A2D(cpu, c, pInst->numCounters);
        unsigned int value;
        int overflow;
        /* check if the counter is running */
        if ((pCtr = pInst->ppDrvHwCtrData[idx2d]) == NULL)
            continue;
        pInst->getCounterOverflow(c, &overflow);
        if (overflow)
        {            
            /* stop the counter */
            pInst->disableCounter(pCtr->counter);
            pInst->getEvtCount(c, &value);
            if (value < pCtr->reloadValue)
                eventCount = pCtr->counterWrap - pCtr->reloadValue + value + 1;
            else
                eventCount = value - pCtr->reloadValue;
            (*pInst->pHandler) (pCtr->pClient, pRegs, eventCount, cpu);

            /* reload the counter */
            pInst->setEvtCount(c, (unsigned int)(pCtr->reloadValue));

            /* restart the counter */
            pInst->clearCounterOverflow(c);
            pInst->enableCounter(pCtr->counter);

            eventsProcessed = 1;
        }
    }
    
    return eventsProcessed;
}

/******************************************************************************
*/

static STATUS hpcHwCtrStart
    (
    void *		  pDrvInst,
    struct hpcHwCtrData * pCtr
    )
    {
    struct driverInstArm * pInst = pDrvInst;
    int value;
    size_t idx2d = A2D(vxCpuIndexGet(), pCtr->counter, pInst->numCounters);

    pInst->getCounterState(pCtr->counter, &value);

    if (value == 1)
    {
        if (!intCount())
            printf ("counter %u is already running, %s\n",
                    pCtr->counter, pInst->pDriverName);
        return ERROR;
    }

    pInst->ppDrvHwCtrData[idx2d] = pCtr;

    pInst->setEvtCount(pCtr->counter, (unsigned int)(pCtr->countValue));
    pInst->clearCounterOverflow(pCtr->counter);
    pInst->enableCounterInt(pCtr->counter);
    pInst->enableCounter(pCtr->counter);
    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwCtrStop
    (
    void *		  pDrvInst,
    struct hpcHwCtrData * pCtr,
    HPC_COUNTER *	  pCount
    )
    {
    struct driverInstArm * pInst = pDrvInst;
    int value;
    unsigned int count;
    size_t idx2d = A2D(vxCpuIndexGet(), pCtr->counter, pInst->numCounters);
    HPC_COUNTER cnt;

    pInst->getCounterState(pCtr->counter, &value);

    if (value == 0)
    {
        if (!intCount()) printf("counter is already stopped %u", pCtr->counter);
            return ERROR;
    }

    /* stop the counter */
    pInst->disableCounterInt(pCtr->counter);
    pInst->disableCounter(pCtr->counter);
   
    /* save the counters value */
    pInst->getEvtCount(pCtr->counter, &count);

    pCtr->countValue = count;

    pInst->ppDrvHwCtrData[idx2d] = NULL;

    if (pCtr->countValue < pCtr->reloadValue)
        cnt = pCtr->countValue + pCtr->counterWrap - pCtr->reloadValue;
    else
        cnt = pCtr->countValue - pCtr->reloadValue;
    
    if (pCount != NULL)
        *pCount = cnt;

    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwCtrRead
    (
    void *		  pDrvInst,
    struct hpcHwCtrData * pCtr,
    HPC_COUNTER	*	  pValue
    )
    {
    struct driverInstArm * pInst = pDrvInst;
    unsigned int value;
    HPC_COUNTER cnt;

    pInst->getEvtCount(pCtr->counter, &value);
    if (value < pCtr->reloadValue)
        cnt = value + pCtr->counterWrap -pCtr->reloadValue;
    else
        cnt = value - pCtr->reloadValue;

    *pValue = cnt;
    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwDeleteCtrForEvent
    (
    void *		  pDrvInst,
    struct hpcHwCtrData * pHwConfig
    )
    {
    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwUpdateCtrReload
    (
    void * 		  pDrvInst,
    struct hpcHwCtrData * pHwConfig,
    HPC_COUNTER *	  pReload
    )
    {
    if (*pReload > pHwConfig->counterWrap || *pReload == 0)
        {
        /*
         * set a value that will cause maximum number of perf counter increments
         * before perf rollover interrupt occurs.
         */
        pHwConfig->reloadValue = 0;
        *pReload = pHwConfig->counterWrap;
        return OK;
        }

    pHwConfig->reloadValue = pHwConfig->counterWrap - *pReload;
    pHwConfig->countValue = pHwConfig->reloadValue;

    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwCreateCtrForEvent
    (
    void *		  pDrvInst,
    void *		  pEvtSettings,
    HPC_COUNTER *	  pReload,
    struct hpcHwCtrData * pHwConfig,
    void *		  pClient,
    HPC_EVENT_ITER_FUNC	  pEventIterFunc,
    void *		  pIterArg
    )
    {
    STATUS status;
    struct driverInstArm * pInst = pDrvInst;
    struct armSettings * pSettings = pEvtSettings;
    struct hpcHwCtrData * pHwCfgIter;
    unsigned int ctrMask, ctrInUseMask = 0;

    if (pDrvInst == NULL || pEvtSettings == NULL)
        return ERROR;

    while ((pHwCfgIter = (*pEventIterFunc)(pIterArg)) != NULL)
    {
        ctrInUseMask |= 1 << pHwCfgIter->counter;
    }

    ctrMask = pSettings->ctrMask;
    ctrMask &= NOT(ctrInUseMask);

    if (ctrMask & 0x80000000)
         pHwConfig->counter = pInst->numCounters - 1;
    else
          pHwConfig->counter = hpcGetFirstBitSet(ctrMask);

    if (pHwConfig->counter >= pInst->numCounters)
    {
        if (!intCount())
            printf("Unable to get counter in use 0x%08x, can use 0x%08x %s\n",
                   ctrInUseMask, ctrMask, pInst->pDriverName);
        return ERROR;
    }

    pInst->configCounterEvent(pHwConfig->counter, pSettings->evtCode);
    pHwConfig->pClient = pClient;
    pHwConfig->counterWrap = COUNTER_WRAP_32;

    if ((status = hpcHwUpdateCtrReload(pInst, pHwConfig, pReload)) != OK)
        {
        if (!intCount())
            printf("unable to get hw reload value %s\n", pInst->pDriverName);
            return status;
        }
    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwDriverInit
    (
    void *			 pDrvInst,
    struct hpcHwDriverInitData * pDrvInstInitData,
    size_t *			 sizeOfHwPrivData
    )
    {
    struct driverInstArm * pInst = pDrvInst;
    STATUS rc;
    pInst->pHandler = pDrvInstInitData->pPerfIntCallback;

    *sizeOfHwPrivData = sizeof(struct hpcHwCtrData);

    if ((rc = pInst->enablePmuIsr(pmuCounterIntHandler)) != OK)
        {
        if (!intCount()) printf("failed to enable arm pmu isr %u\n", rc);
        }

    return rc;
    }

/******************************************************************************
*/

static STATUS hpcHwDriverFinit
    (
    void * pDrvInst
    )
    {
    struct driverInstArm * pInst = pDrvInst;
    STATUS rc = OK;

    if ((rc = pInst->disablePmuIsr(pmuCounterIntHandler)) != OK)
        {
        if (!intCount()) 
            printf("failed to disable arm pmu isr %d, %s\n", rc,
                   pInst->pDriverName);
        }

    return rc;
    }

/******************************************************************************
*/

static STATUS hpcHwPerCoreInit
    (
    void * pDrvInst
    )
    {
    struct driverInstArm * pInst = pDrvInst;
    counterResetAll (pInst);
    pInst->enableArmPmu();
    return OK;
    }

/******************************************************************************
*/

static STATUS hpcHwPerCoreFinit
    (
    void * pDrvInst,
    int    force
    )
    {
    struct driverInstArm * pInst = pDrvInst;
    counterResetAll (pInst);
    pInst->disableArmPmu();
    return OK;
    }

/******************************************************************************
*/

STATUS hpcHwArmDriverInit(void)
    {
    return OK;
    }

/******************************************************************************
*/

STATUS hpcHwArmDriverFinit(void)
    {
    return OK;
    }

/******************************************************************************
*/

struct hpcHwDriverFuncs hpcHwDrvArmRtns =
    {
    hpcHwCtrStart,
    hpcHwCtrStop,
    hpcHwCtrRead,
    hpcHwUpdateCtrReload,
    hpcHwCreateCtrForEvent,
    hpcHwDeleteCtrForEvent,
    hpcHwDriverInit,
    hpcHwDriverFinit,
    hpcHwPerCoreInit,
    hpcHwPerCoreFinit,
    hpcHwDriverShow
    };
