/* Copyright (c) 2010-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#ifndef INChpc_log_h
#define INChpc_log_h

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __VXWORKS__
#ifndef HPC_LOG_ERROR
#define HPC_LOG_ERROR		1
#endif
#ifndef HPC_LOG_DEBUG
#define HPC_LOG_DEBUG		1
#endif
#ifndef HPC_LOG_DEBUG_ISR
#define HPC_LOG_DEBUG_ISR	1
#endif
#ifndef HPC_DEBUG_VERBOSITY
#define HPC_DEBUG_VERBOSITY	5
#endif
#ifndef HPC_LOG_USE_LOGMSG
#define HPC_LOG_USE_LOGMSG	0
#endif
#ifndef HPC_LOG_USE_PRINTF
#define HPC_LOG_USE_PRINTF	1
#endif
#else /* LINUX */
#ifndef HPC_LOG_ERROR
  #ifdef CAFE_DEBUG
   #define HPC_LOG_ERROR	1
  #else
   #define HPC_LOG_ERROR	0
  #endif
#endif
#ifndef HPC_LOG_DEBUG
#define HPC_LOG_DEBUG		1
#endif
#ifndef HPC_LOG_DEBUG_ISR
#define HPC_LOG_DEBUG_ISR	0
#endif
#ifndef HPC_DEBUG_VERBOSITY
#define HPC_DEBUG_VERBOSITY	5
#endif
#ifndef HPC_LOG_USE_PRINTF
#define HPC_LOG_USE_PRINTF	1
#endif
#endif /* __VXWORKS__ */


#if HPC_LOG_USE_LOGMSG && HPC_LOG_USE_PRINTF
#error unsupported
#endif

#if (HPC_LOG_DEBUG > 0) || (HPC_LOG_ERROR > 0) || (HPC_LOG_DEBUG_ISR > 0)
#define HPC_SAFE(C) do {C} while (0)
#endif

#if HPC_LOG_DEBUG
#if HPC_LOG_USE_LOGMSG
extern void hpc_log_debug (int verbosity, const char *format, ...);
#define hpc_debug(V, FORMAT, ...) HPC_SAFE(hpc_log_debug(V, FORMAT, ##__VA_ARGS__);)
#elif HPC_LOG_USE_PRINTF
extern int hpc_verbosity;
#define hpc_debug(V, FORMAT, ...) HPC_SAFE(if (V <= hpc_verbosity) printf(FORMAT, ##__VA_ARGS__);)
#else
#error unsupported
#endif
#else /* HPC_LOG_DEBUG */
#define hpc_debug(format, ...) do{}while(0)
#endif

#if HPC_LOG_ERROR
#if HPC_LOG_USE_LOGMSG
extern void hpc_log_error (const char *format, ...);
#define hpc_error(FORMAT, ...) HPC_SAFE(hpc_log_error("HPC ERROR:%d " FORMAT, __LINE__, ##__VA_ARGS__);)
#elif HPC_LOG_USE_PRINTF
#define hpc_error(FORMAT, ...) HPC_SAFE(printf("HPC ERROR:%d " FORMAT, __LINE__, ##__VA_ARGS__);)
#else
#error unsupported
#endif
#else /* HPC_LOG_ERROR */
#define hpc_error(format, ...) do{}while(0)
#endif

#if HPC_LOG_DEBUG_ISR
extern void hpc_log_debug_isr (int verbosity, const char *format, ...);
#define hpc_debug_isr(V, FORMAT, ...) HPC_SAFE(hpc_log_debug_isr(V, FORMAT, ##__VA_ARGS__);)
#else
#define hpc_debug_isr(V, FORMAT, ...) do{}while(0)
#endif

#ifdef __cplusplus
}
#endif
#endif /* INChpc_log_h */
