/* Copyright (c) 2010-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
14may14,sdt  Fix 'checked_return' warning
13may14,sdt  Rename cafe tasks to tAnalysis* - US37854
11mar14,sdt  Fix 'Null Returns' warning


DESCRIPTION
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/hpc_ep.h>

#include "hpc_utils.h"

#define HPC_DEBUG_VERBOSITY 0
#include "hpc_log.h"

#ifdef __VXWORKS__
#include <taskLib.h>
#include <vxCpuLib.h>
#include <intLib.h>  /* intCount() */
#include <esf.h>
#include <sysLib.h>
#include <logLib.h>

#if (_VX_CPU_FAMILY == _VX_I80X86)
#if defined(_WRS_CONFIG_SMP)
#include <private/vxSmpP.h>
#include <private/windLibP.h>
#else /* _WRS_CONFIG_SMP */
extern int intNest;
#endif /* _WRS_CONFIG_SMP */
#endif /* _VX_CPU_FAMILY == _VX_I80X86 */

#if (_WRS_VXWORKS_MAJOR < 6)					\
	|| (_WRS_VXWORKS_MAJOR == 6) && (_WRS_VXWORKS_MINOR < 9)
#define TASK_ID_ERROR (ERROR)
typedef int _Vx_usr_arg_t;
#endif

#if defined(NDEBUG)
#   define VX_HPC_TASK_OPTIONS  (VX_UNBREAKABLE)
#else
#   define VX_HPC_TASK_OPTIONS  (0)
#endif

#if defined(_WRS_CONFIG_SMP)
static STATUS hpcIterateCpusFunc
    (
    int         numCpus,
    STATUS	(*pIter)(int, void *),
    void *	pData,
    int		waitForCores,
    int		singleCpu
    )
    {
    int cpu;
    STATUS status, rc = OK;
    cpuset_t affinity;

    if (singleCpu)
        {
	cpu = numCpus;
	numCpus++; /* ensure we only need to loop once below */
	}
    else
        cpu = 0;

    for (; cpu < numCpus; cpu++)
        {
        if (waitForCores)
            while (CPUSET_ISSET(vxCpuEnabledGet(), cpu) == 0)
                taskDelay(1);
	CPUSET_ZERO (affinity);
	CPUSET_SET (affinity, cpu);
	if ((status = taskCpuAffinitySet (0, affinity)) != OK)
	    {
	    hpc_error("unable to set cpu affinity %d, error %d errno %d\n",
			    cpu, status, errno);
	    rc = status;
            break;
	    }
	if ((status = (*pIter)(cpu, pData)) != OK)
	    {
/*	    hpc_error("cpu iterator returned error, cpu %d\n", cpu); */
	    rc = status;
            break;
	    }
	}

    /* remove cpu affinity */
    CPUSET_ZERO (affinity);
    if ((status = taskCpuAffinitySet (0, affinity)) != OK)
        {
        hpc_error("unable to remove cpu affinity %d, errno %d\n", status,
        		errno);
        rc = status;
	}
    return rc;
    }
#endif

static STATUS hpcIterateCpus2
    (
    int	   numCpus,
    STATUS (*pIter)(int, void *),
    void * pData,
    int singleCpu
    )
    {
    STATUS status = OK;
#if defined(_WRS_CONFIG_SMP)
     /* The root task on vxWorks was not able to switch CPU affinity at,
     * boot time since the other core are not yet enabled.
     * So a task is spawned which will run this request once the cores
     * are enabled.
     */
    char * tName = taskName(0);
    assert(tName != NULL);
    if (strcmp(tName, "tRootTask") == 0)
        {
		taskSpawn("tAnalysisHpcTmp", 123, VX_HPC_TASK_OPTIONS,
		          sizeof(long) * 2048,
			  (FUNCPTR)hpcIterateCpusFunc, numCpus,
			  (_Vx_usr_arg_t)pIter, (_Vx_usr_arg_t)pData, 1,
			  singleCpu, 0, 0, 0, 0, 0);
        }
    else
        {
	status = hpcIterateCpusFunc(numCpus, pIter, pData, 0, singleCpu);
        }
#else /* defined(_WRS_CONFIG_SMP) */
    if ((status = (*pIter)(0, pData)) != OK)
        {
/*	hpc_error("cpu iterator returned error %s\n", status); */
	}
#endif /* defined(_WRS_CONFIG_SMP) */

    return status;
    }

STATUS hpcExecuteOnSingleCpu
    (
    int cpu,
    STATUS (*pIter)(int, void *),
    void * pData
    )
    {
    return hpcIterateCpus2(cpu, pIter, pData, 1);
    }

STATUS hpcIterateCpus
    (
    int	   numCpus,
    STATUS (*pIter)(int, void *),
    void * pData
    )
    {
    return hpcIterateCpus2(numCpus, pIter, pData, 0);
    }

static HPC_TASK_ID_TYPE hpc_check_period_task_id = TASK_ID_ERROR;
static volatile int hpc_check_period_task_delete_flag;
static SEM_ID hpc_check_period_task_delete_semid;

static void tAnalysisHpc
    (
    int (*pChkEvtPeriodFunc)(void),
    int delay
    )
    {
    int work_to_do;
    (void)taskSuspend(0);
    if (hpc_check_period_task_delete_flag)
	goto exit;
    while (1)
        {
	work_to_do = (*pChkEvtPeriodFunc)();
	if (hpc_check_period_task_delete_flag)
	    goto exit;
        if (work_to_do)
	    (void)taskDelay(delay);
        else
	    taskSuspend(0);
	if (hpc_check_period_task_delete_flag)
	    goto exit;
        }
exit:
    (void)semGive(hpc_check_period_task_delete_semid);
    }

STATUS hpcChkEvtPeriodTaskStart
    (
    int (*pChkEvtPeriodFunc)(void)
    )
    {
    hpc_check_period_task_delete_semid = semBCreate(SEM_Q_PRIORITY, SEM_EMPTY);
    if (hpc_check_period_task_delete_semid == NULL)
	return ERROR;
    hpc_check_period_task_delete_flag = 0;
    hpc_check_period_task_id = taskSpawn("tAnalysisHpc", 50,
                                         VX_HPC_TASK_OPTIONS,
                                         sizeof(long) * 1024,
					 (FUNCPTR)tAnalysisHpc,
					 (_Vx_usr_arg_t)pChkEvtPeriodFunc,
					 sysClkRateGet() / 2,
					 0, 0, 0, 0, 0, 0, 0, 0);
    if (hpc_check_period_task_id == TASK_ID_ERROR)
	return ERROR;
    else
	return OK;
    }

void hpcChkEvtPeriodTaskResume(void)
    {
    (void)taskResume(hpc_check_period_task_id);
    }

void hpcChkEvtPeriodTaskDelete(void)
    {
    STATUS rc;
    if (hpc_check_period_task_id == TASK_ID_ERROR)
	return;
    hpc_check_period_task_delete_flag = 1;

again:
    hpcChkEvtPeriodTaskResume();
    rc = semTake(hpc_check_period_task_delete_semid, sysClkRateGet() / 30);
    if (rc == ERROR && errno == S_objLib_OBJ_TIMEOUT)
        goto again;

    (void)semDelete(hpc_check_period_task_delete_semid);
    hpc_check_period_task_id = TASK_ID_ERROR;
    }

#if (_VX_CPU_FAMILY == _VX_I80X86)
#if _WRS_CONFIG_LP64
struct pm_esf_regs
    {
    /* values pushed by pushal (64bit) instruction in hpcI86PerfMonVector() */
    unsigned long reg_r15;
    unsigned long reg_r14;
    unsigned long reg_r13;
    unsigned long reg_r12;
    unsigned long reg_r11;
    unsigned long reg_r10;
    unsigned long reg_r9;
    unsigned long reg_r8;
    unsigned long reg_rdi;
    unsigned long reg_rsi;
    unsigned long reg_rbp;
    unsigned long reg_rsp;
    unsigned long reg_rbx;
    unsigned long reg_rdx;
    unsigned long reg_rcx;
    unsigned long reg_rax;

    unsigned long error;		/* pushed by intEnter */
    ESF0 esf;				/* values saved by hardware and */
					/* copied to interrupt stack by */
					/* intEnter */
    unsigned char *sup_stack;		/* pointer to supervisor stack */
};
#else /* _WRS_CONFIG_LP64 */
/* Refer to hpcI86PerfMonVector() to see how this stack frame is created.
 * For the nested interrupt case, there is no stack switch from supervisor
 * to interrupt stack. In this case, the address of sup_stack is the SP
 * for the interrupted interrupt. */
struct pm_esf_regs
    {
    /* values pushed by pushal instruction in hpcI86PerfMonVector() */
    unsigned long reg_edi;		/* EDI */
    unsigned long reg_esi;		/* ESI */
    unsigned long reg_ebp;		/* EBP */
    unsigned long reg_esp;		/* ESP */
    unsigned long reg_ebx;		/* EBX */
    unsigned long reg_edx;		/* EDX */
    unsigned long reg_ecx;		/* ECX */
    unsigned long reg_eax;		/* EAX */

    unsigned int error;			/* pushed by intEnter */
    ESF0 esf;				/* values saved by hardware and */
					/* copied to interrupt stack by */
					/* intEnter */
    unsigned char *sup_stack;		/* pointer to supervisor stack */
};
#endif /* _WRS_CONFIG_LP64 */

void hpcGetRegSet(void *pRegCtx, REG_SET *pRegs)
{
    struct pm_esf_regs *pEsf = pRegCtx;
    int nestCnt;

    /* get nested count (different than intCount() used to fake interrupt
     * context when running watchdog handler) */

#ifdef _WRS_CONFIG_SMP
    UINT cpuId = _WRS_CPU_INDEX_GET();

    nestCnt = _WRS_WIND_VARS_ARCH_GET(cpuId, intNest);
#else
    nestCnt = intNest;
#endif /* _WRS_CONFIG_SMP */
    
    if (nestCnt == 1)
        {
        /* sup_stack points to the supervisor stack. We adjust the value
         * by the amount shown to allow for errno and ESF0. */
#ifdef _WRS_CONFIG_LP64
        pRegs->reg_sp = (unsigned long) pEsf->sup_stack + ESF0_NBYTES + 8;
#else
        pRegs->reg_sp = (unsigned long) pEsf->sup_stack + ESF0_NBYTES + 4;
#endif
        }
    else
        {
        /* for nested interrupts, the address of sup_stack is the SP
         * for the interrupted interrupt. */
        pRegs->reg_sp = (unsigned long) &pEsf->sup_stack;
        }

    pRegs->pc = pEsf->esf.pc;
#ifdef _WRS_CONFIG_LP64
    pRegs->reg_fp = pEsf->reg_rbp;
#else
    pRegs->reg_fp = pEsf->reg_ebp;
#endif
}

#elif (_VX_CPU_FAMILY==_VX_PPC)

void hpcGetRegSet(void * pRegCtx, REG_SET *pRegs)
{
    ESFPPC * pEsf = pRegCtx;

    pRegs->reg_sp = pEsf->regSet.reg_sp;
    pRegs->pc = pEsf->regSet.reg_pc;
    pRegs->lr = pEsf->regSet.lr;
    pRegs->gpr[0] = pEsf->regSet.gpr[0];
    pRegs->gpr[11] = pEsf->regSet.gpr[11];
    pRegs->gpr[12] = pEsf->regSet.gpr[12];
}

#else

void hpcGetRegSet(void * pRegCtx, REG_SET *pRegs)
{
}
#endif
#endif /* __VXWORKS__ */


#if HPC_LOG_DEBUG || HPC_LOG_DEBUG_ISR
int hpc_verbosity = HPC_DEBUG_VERBOSITY;
#endif

#if (HPC_LOG_DEBUG && defined(HPC_LOG_USE_LOGMSG)) || HPC_LOG_DEBUG_ISR
void hpc_log_debug_log_msg(int verbosity, const char *format, va_list v_args)
{
	int arg1, arg2, arg3, arg4, arg5, arg6;

	if (verbosity <= hpc_verbosity) {
		arg1 = va_arg(v_args, int);
		arg2 = va_arg(v_args, int);
		arg3 = va_arg(v_args, int);
		arg4 = va_arg(v_args, int);
		arg5 = va_arg(v_args, int);
		arg6 = va_arg(v_args, int);
		(void)logMsg((char *)format, arg1, arg2, arg3, arg4, arg5, arg6);
	}
}
#endif

#if HPC_LOG_DEBUG && defined(HPC_LOG_USE_LOGMSG)
void hpc_log_debug(int verbosity, const char *format, ...)
{
	   va_list args;
	   va_start(args, format);
	   hpc_log_debug_log_msg(verbosity, format, args);
	   va_end(args);
}
#endif

#if HPC_LOG_DEBUG_ISR
void hpc_log_debug_isr (int verbosity, const char *format, ...)
{
	va_list args;
	va_start(args, format);
	hpc_log_debug_log_msg(verbosity, format, args);
	va_end(args);
}
#endif

#if HPC_LOG_ERROR && defined(HPC_LOG_USE_LOGMSG)
void hpc_log_error_log_msg(const char *format, va_list v_args)
{
	int arg1, arg2, arg3, arg4, arg5, arg6;

	arg1 = va_arg(v_args, int);
	arg2 = va_arg(v_args, int);
	arg3 = va_arg(v_args, int);
	arg4 = va_arg(v_args, int);
	arg5 = va_arg(v_args, int);
	arg6 = va_arg(v_args, int);
	(void)logMsg((char *)format, arg1, arg2, arg3, arg4, arg5, arg6);
}

void hpc_log_error(const char *format, ...)
{
	va_list args;
	va_start(args, format);
	hpc_log_error_log_msg(format, args);
	va_end(args);
}
#endif
