/* Copyright (c) 2010-2011 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#ifndef INChpc_utils_h
#define INChpc_utils_h

#ifdef __cplusplus
extern "C" {
#endif

/* Constants for x86 CPU registers */
#define IA32_PMC0        		0xC1
#define IA32_PERFEVTSEL0 		0x186
#define IA32_PERF_GLOBAL_STATUS		0x38E	/* Global Performance Counter Status */
#define IA32_PERF_GLOBAL_CTRL		0x38F	/* Global Performance Counter Control */
#define IA32_PERF_GLOBAL_OVF_CTRL	0x390	/* Global Performance Counter Overflow Control */
/* Fixed-Function Performance Counter 0, Counts Instr_Retired.Any */
#define IA32_FIXED_CTR0			0x309
/* Unique Fixed-Function-Counter Control Register. */
#define IA32_FIXED_CTR_CTRL		0x38D

/* Registers added for Nehalen Uncore events */
#define MSR_UNCORE_PERF_GLOBAL_CTRL	0x391
#define MSR_UNCORE_PERF_GLOBAL_STATUS	0x392
#define MSR_UNCORE_PERF_GLOBAL_OVF_CTRL	0x393
#define MSR_UNCORE_FIXED_CTR0		0x394
#define MSR_UNCORE_FIXED_CTR_CTRL	0x395
#define MSR_UNCORE_ADDR_OPCODE_MATCH	0x396
#define MSR_UNCORE_PMC0			0x3B0
#define MSR_UNCORE_PERFEVTSEL0		0x3C0

#define MSR_IA32_DEBUG_CTL		0x1d9

#if defined(__VXWORKS__)
#include <vxworks/cafe_vx_timestamp.h>  /* for cafe_vx_timestamp64() */
#include <regs.h>
#include <intLib.h>

#if (_WRS_VXWORKS_MAJOR < 6)					\
	|| (_WRS_VXWORKS_MAJOR == 6) && (_WRS_VXWORKS_MINOR < 9)
#define HPC_TASK_ID_TYPE int
#else
#define HPC_TASK_ID_TYPE  TASK_ID
#endif

#else /* Linux host (mock) build for testing */

struct _REG_SET;
typedef struct _REG_SET REG_SET;
#define HPC_TASK_ID_TYPE int
extern int cafe_vx_timestamp64(unsigned long long *t);
extern int cafe_vx_timestamp_freq64(unsigned long long *f);
extern HPC_TASK_ID_TYPE taskIdSelf(void);

#endif /* __VXWORKS__ */

extern void initPentiumRegs(void);

extern STATUS hpcIterateCpus(int numCpus, STATUS (*pIter)(int, void *),
                             void * data);
extern void hpcGetRegSet(void * pRegCtx, REG_SET *pRegs);
extern STATUS hpcExecuteOnSingleCpu(int cpu, STATUS (*pIter)(int, void *),
                                    void * pData);
extern STATUS hpcChkEvtPeriodTaskStart(int (*pChkEvtPeriodFunc)(void));
extern void hpcChkEvtPeriodTaskResume(void);
extern void hpcChkEvtPeriodTaskDelete(void);

/* hpc I86 functions */
extern void hpcI86PerfMonVector(void);
extern void i86cpuid(unsigned int *eax, unsigned int *ebx, unsigned int *ecx,
		unsigned int *edx);

#ifdef __cplusplus
}
#endif
#endif /* INChpc_utils_h */
