/* Copyright (c) 2010-2012, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <test/cafe_CUnitTest.h>
#include <cafe/utils.h>
#include <cafe/value_type.h>
#include <hpc/hpc_ep.h>

#include "hpc_test.h"
#include "../hpc_utils.h"

#define NUM_EVT_DEF_TYPES	4	/* event, timestamp, coreid */
/*
 * Needed this variable for icc compiler. The code:
 * CUT_ASSERT_NOT_EQUAL(0, count); was giving the following error:
 * error: assuming signed overflow does not occur when changing X +- C1 cmp C2
 * to X cmp C1 +- C2
 */
static volatile int cut_zero = 0;

static int *cores;
static int orth_data_ts_error;
static int orth_data_coreid_error;

/******************************************************************************
 * Configuration to print some HPC events
 */

static void perf_handler(struct cafe_event_action_info *event_info,
			 void **event_data)
{
	int event = 0;
	unsigned long long *timestamp = event_data[1];
	unsigned int *core_id = event_data[2];
	HPC_COUNTER *count = event_data[3];

#if defined(CAFE_DEBUG) && !defined(__VXWORKS__)
	if (hpc_hw_drv_type != TEST_VXW_HPC_HW_DRV)
		printf("perf_handler[%d]: %s\t\t%llu\n", vxCpuIndexGet(),
		       event_info->event->types[0]->type_str, *count);
#endif

	if (NUM_EVT_DEF_TYPES >= 3)
		if (*core_id > 32) orth_data_coreid_error++;
#ifdef __VXWORKS__
	if (NUM_EVT_DEF_TYPES >= 2)
		if (*timestamp == 0) orth_data_ts_error++;
#else
	if (NUM_EVT_DEF_TYPES >= 2)
		if (*timestamp != 0xaaaaaaaa55555555ULL) orth_data_ts_error++;
#endif
	while (hpc_test_hte[event] != NULL) {
		if (strcmp(event_info->event->types[0]->type_str, *hpc_test_hte[event]) == 0)
			break;
		event++;
	}
	(void)hpc_test_ctr_inc(event, *count);
}

static int perf_counter_print_get_input_event_actions(
	struct cafe_ep_inst *inst,
	struct cafe_event_descr *event,
	struct cafe_event_action_list *actions)
{
	int cafe_rc;
	struct cafe_event_action_info *p;
	cafe_rc = cafe_ep_inst_event_action_info_get(inst, event, &p);
	if (cafe_rc != CAFE_OK || p == NULL)
		return cafe_rc;
	return cafe_add_event_action(inst, actions, perf_handler, p);
}

static struct cafe_ep_spec hpc_print_config_spec = {
	"perf_counter_print",
	NULL,			/* create_hook */
	NULL,			/* delete_hook */
	NULL,			/* start hook */
	NULL,			/* stop hook */
	perf_counter_print_get_input_event_actions,
	NULL,
	NULL,
	NULL,
	NULL,
	0,			/* number of input events */
	NULL,			/* input events */
	0,
	NULL			/* output events */
};

static int register_hpc_print_config_spec(unsigned int *num_events)
{
	int cafe_rc;
	int e;

	while (hpc_test_hte[*num_events] != NULL) (*num_events)++;
	hpc_print_config_spec.n_input_events = *num_events;
	CUT_ASSERT_NOT_EQUAL(0, hpc_print_config_spec.n_input_events);

	hpc_print_config_spec.input_events =
		malloc(sizeof(struct cafe_ep_spec_input_evt) * (*num_events));
	CUT_ASSERT_NOT_NULL(hpc_print_config_spec.input_events);

	for (e = 0; e < (*num_events); e++) {
		hpc_print_config_spec.input_events[e].num_types = NUM_EVT_DEF_TYPES;
		hpc_print_config_spec.input_events[e].type = &(*hpc_test_hte[e]);
	}

	cafe_rc = cafe_ep_spec_register(&hpc_print_config_spec);
	CUT_ASSERT_CAFE_OK(cafe_rc);

	return 0;
}

static int unregister_hpc_print_config_spec(void)
{
	int cafe_rc;
	cafe_rc = cafe_ep_spec_unregister(&hpc_print_config_spec);
	CUT_ASSERT_CAFE_OK(cafe_rc);

	free(hpc_print_config_spec.input_events);
	return 0;
}


static int enable_required_root_output_events(struct cafe_ep_inst *inst)
{
	unsigned int i, num_events = 0;
	int cafe_rc = 0;
	const char **str;

	while (hpc_test_hte[num_events] != NULL)	num_events++;

	for (i = 0; i < num_events; i++) {
		int n_types;
		struct cafe_event_descr *event;
		struct cafe_value_type *type[NUM_EVT_DEF_TYPES];
		str = hpc_test_hte[i];
		for (n_types = 0; n_types < NUM_EVT_DEF_TYPES; n_types++) {
			cafe_rc = cafe_type_parse(inst->session->ns, str[n_types], &type[n_types]);
			if (cafe_rc != CAFE_OK)
				return cafe_rc;
		}

		cafe_rc = cafe_cfg_get_event(inst->session, NUM_EVT_DEF_TYPES, &type[0], &event);
		if (cafe_rc != CAFE_OK)
			continue;

		cafe_rc = cafe_ep_inst_enable_output_event(inst, event);
		if (cafe_rc != CAFE_OK)
			return cafe_rc;
	}
	return CAFE_OK;
}

/******************************************************************************
 * Test code
 */

int hpc_test_cafe1(void)
{
	int cafe_rc, cut_rc, e, ctr_mask = 0;
	unsigned int num_events = 0;
	struct cafe_ep_spec *hpc_spec;
	struct cafe_ep_spec *hpc_print_spec;
	struct cafe_ep_inst *hpc_provider_inst;
	struct cafe_ep_inst *hpc_print_inst;
	struct cafe_instrument *session;

	hpc_test_clear_counters();

	orth_data_ts_error = orth_data_coreid_error = 0;

	CUT_ASSERT_CAFE_OK(hpc_tst_init());
	cafe_rc = cafe_lib_init();
	CUT_ASSERT_CAFE_OK(cafe_rc);

	CAFE_LOCK();

	if (hpc_hw_drv_type != TEST_VXW_HPC_HW_DRV)
		CUT_ASSERT_CAFE_OK(hpc_register_configuration());

	cut_rc = register_hpc_print_config_spec(&num_events);
	CUT_ASSERT_EQUAL(0, cut_rc);

	CUT_ASSERT_CAFE_OK(cafe_instrument_create("hpc_test_cafe1", &session));

	hpc_spec = cafe_ep_spec_find("perf_counter_provider");
	CUT_ASSERT_NOT_NULL(hpc_spec);

	cafe_rc = cafe_ep_inst_create(session, hpc_spec,
				      0, NULL, &hpc_provider_inst);
	CUT_ASSERT_CAFE_OK(cafe_rc);

	/* We use fake event points, so this instance will only
	 * generate event on the session thread. */
	hpc_provider_inst->flags |= CAFE_EP_INST_FLAG_OUTPUT_SESSION_THREAD;

	hpc_print_spec = cafe_ep_spec_find("perf_counter_print");
	CUT_ASSERT_NOT_NULL(hpc_print_spec);

	cafe_rc = cafe_ep_inst_create(session, hpc_print_spec,
				      0, NULL, &hpc_print_inst);
	CUT_ASSERT_CAFE_OK(cafe_rc);

	CUT_ASSERT_CAFE_OK(enable_required_root_output_events(hpc_provider_inst));

	CUT_ASSERT_CAFE_OK(cafe_inst_enable_all_output_events(hpc_print_inst));

	cafe_rc = cafe_ep_inst_connect(hpc_provider_inst, hpc_print_inst);
	CUT_ASSERT_CAFE_OK(cafe_rc);

	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session));

	cores = malloc(sizeof(int) * num_events);
	CUT_ASSERT_NOT_NULL(cores);
	memset(cores, 0, sizeof(int) * num_events);

	cafe_rc = cafe_instrument_start(session);
	CUT_ASSERT_CAFE_OK(cafe_rc);

	/* generate some events */
	for (e = 0; e < num_events; e++) {
		ctr_mask |= 1 << e;
		CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 20*1000000, session, &cores[e], *hpc_test_hte[e], NULL));
	}
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));

	/* check that we got them */
	for (e = 0; e < num_events; e++)
		CUT_ASSERT_EQUAL(0, hpc_test_check_counters(e, 20*1000000, cores[e]));

	if (orth_data_ts_error > 0) printf("orth_data_ts_error %d\n", orth_data_ts_error);
	CUT_ASSERT_EQUAL(0, orth_data_ts_error);
	CUT_ASSERT_EQUAL(0, orth_data_coreid_error);
	CUT_ASSERT_NOT_EQUAL(cut_zero, e);

	CUT_ASSERT_CAFE_OK(cafe_instrument_stop(session));
	CUT_ASSERT_CAFE_OK(cafe_instrument_delete_inst(session));
	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session));

	if (hpc_hw_drv_type != TEST_VXW_HPC_HW_DRV)
		CUT_ASSERT_CAFE_OK(hpc_unregister_configuration());

	cut_rc = unregister_hpc_print_config_spec();
	CUT_ASSERT_EQUAL(0, cut_rc);

	CAFE_UNLOCK();

	cafe_lib_release();
	CUT_ASSERT_CAFE_OK(hpc_tst_finit());

	free(cores);

	return 0;
}
