/* Copyright (c) 2010-2011, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */
/*
modification history
--------------------
03aug14,sdt  Fixed checked-return warnings (US45196)
13mar14,wzc  support arm hpc(US17715)
*/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <cafe/utils.h>
#include <cafe/value_type.h>
#include <test/cafe_CUnitTest.h>
#include <hpc/hpc_ep.h>
#include <cafe/autogenerated/hpc_common.type.h>

#include "hpc_test.h"

#define NUM_TEST_ITERATIONS 1

/*
 * Needed this variable for icc compiler. The code:
 * CUT_ASSERT_NOT_EQUAL(0, count); was giving the following error:
 * error: assuming signed overflow does not occur when changing X +- C1 cmp C2
 * to X cmp C1 +- C2
 */
static volatile int cut_zero = 0;

#define CTR_SA_EV1 0
#define CTR_SA_EV2 1
#define CTR_SB_EV1 2
#define CTR_SB_EV3 3

/******************************************************************************
 * Configuration to print some HPC events
 */

static void perf_handler_a_1(struct cafe_event_action_info *event_info,
			 void **event_data)
{
	HPC_COUNTER *count = event_data[1];
	hpc_debug(100, "perf_handler_a_1... %llu\n", *count);
	(void)hpc_test_ctr_inc(CTR_SA_EV1, *count);
}

static void perf_handler_a_2(struct cafe_event_action_info *event_info,
			 void **event_data)
{
	HPC_COUNTER *count = event_data[1];
	hpc_debug(100, "perf_handler_a_2... %llu\n", *count);
	(void)hpc_test_ctr_inc(CTR_SA_EV2, *count);
}

static void perf_handler_b_1(struct cafe_event_action_info *event_info,
			 void **event_data)
{
	HPC_COUNTER *count = event_data[1];
	hpc_debug(100, "perf_handler_b_1... %llu\n", *count);
	(void)hpc_test_ctr_inc(CTR_SB_EV1, *count);
}

static void perf_handler_b_3(struct cafe_event_action_info *event_info,
			 void **event_data)
{
	HPC_COUNTER *count = event_data[1];
	hpc_debug(100, "perf_handler_b_3... %llu\n", *count);
	(void)hpc_test_ctr_inc(CTR_SB_EV3, *count);
}

static const char * event1_TYPE[] = {hpc_total_cycles_VT_REF, "P[cafe.skipcount,1000000]"};
#if defined(_VX_CPU_FAMILY) && (_VX_CPU_FAMILY==_VX_ARM)
static const char * event2_TYPE[] = {hpc_total_instructions_VT_REF, "P[cafe.skipcount,1000000]"};
static const char * event3_TYPE[] = {hpc_level_1_data_cache_miss_VT_REF, "P[cafe.skipcount,10000]"};
#else
static const char * event2_TYPE[] = {hpc_total_cycles_VT_REF, "P[cafe.skipcount,2000000]"};
static const char * event3_TYPE[] = {hpc_total_cycles_VT_REF, "P[cafe.skipcount,800000]"};
#endif

static struct cafe_ep_spec_input_evt input_events_1[] = {
    CAFE_INPEV_INIT2(event1, perf_handler_a_1),
    CAFE_INPEV_INIT2(event2, perf_handler_a_2)
};

static struct cafe_ep_spec_input_evt input_events_2[] = {
    CAFE_INPEV_INIT2(event1, perf_handler_b_1),
    CAFE_INPEV_INIT2(event3, perf_handler_b_3)
};

static int get_input_event_actions1(
	struct cafe_ep_inst *inst,
	struct cafe_event_descr *event,
	struct cafe_event_action_list *actions)
{
	int cafe_rc;
	struct cafe_event_action_info *p;
	cafe_rc = cafe_ep_inst_event_action_info_get(inst, event, &p);
	if (cafe_rc != CAFE_OK || p == NULL) {
		return cafe_rc;
	}
	return cafe_add_event_action(inst, actions, (CAFE_ACTION_HANDLER)p->impl_data, p);
}


static struct cafe_ep_spec hpc_test_spec1 = {
	"hpc_test_spec_1",
	NULL,			/* create_hook */
	NULL,			/* delete_hook */
	NULL,			/* start hook */
	NULL,			/* stop hook */
	get_input_event_actions1,
	NULL,
	NULL,
	NULL,
	"S[hpc_test_spec_1.workarea,v[sequence,I]]",
	CAFE_NELEMENTS(input_events_1), input_events_1,
	0,
	NULL			/* output events */
};

static struct cafe_ep_spec hpc_test_spec2 = {
	"hpc_test_spec_2",
	NULL,			/* create_hook */
	NULL,			/* delete_hook */
	NULL,			/* start hook */
	NULL,			/* stop hook */
	get_input_event_actions1,
	NULL,
	NULL,
	NULL,
	NULL,
	CAFE_NELEMENTS(input_events_2), input_events_2,
	0,
	NULL			/* output events */
};

static int register_hpc_test_spec1(void)
{
	int cafe_rc;
	cafe_rc = cafe_ep_spec_register(&hpc_test_spec1);
	CUT_ASSERT_CAFE_OK(cafe_rc);
	return 0;
}

static int register_hpc_test_spec2(void)
{
	int cafe_rc;
	cafe_rc = cafe_ep_spec_register(&hpc_test_spec2);
	CUT_ASSERT_CAFE_OK(cafe_rc);
	return 0;
}

static int unregister_hpc_test_spec1(void)
{
	CUT_ASSERT_CAFE_OK(cafe_ep_spec_unregister(&hpc_test_spec1));
	return 0;
}

static int unregister_hpc_test_spec2(void)
{
	CUT_ASSERT_CAFE_OK(cafe_ep_spec_unregister(&hpc_test_spec2));
	return 0;
}

static int enable_required_root_output_events_a(struct cafe_ep_inst *inst)
{
	struct cafe_value_type *type[2];
	struct cafe_event_descr *event;

	CUT_ASSERT_CAFE_OK(cafe_type_parse(inst->session->ns, event1_TYPE[0], &type[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(inst->session->ns, event1_TYPE[1], &type[1]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(inst->session, 2, &type[0], &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	CUT_ASSERT_CAFE_OK(cafe_type_parse(inst->session->ns, event2_TYPE[0], &type[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(inst->session->ns, event2_TYPE[1], &type[1]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(inst->session, 2, &type[0], &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	return 0;
}

static int enable_required_root_output_events_b(struct cafe_ep_inst *inst)
{
	struct cafe_value_type *type[2];
	struct cafe_event_descr *event;

	CUT_ASSERT_CAFE_OK(cafe_type_parse(inst->session->ns, event1_TYPE[0], &type[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(inst->session->ns, event1_TYPE[1], &type[1]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(inst->session, 2, &type[0], &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	CUT_ASSERT_CAFE_OK(cafe_type_parse(inst->session->ns, event3_TYPE[0], &type[0]));
	CUT_ASSERT_CAFE_OK(cafe_type_parse(inst->session->ns, event3_TYPE[1], &type[1]));
	CUT_ASSERT_CAFE_OK(cafe_cfg_get_event(inst->session, 2, &type[0], &event));
	CUT_ASSERT_CAFE_OK(cafe_ep_inst_enable_output_event(inst, event));

	return 0;
}

static int create_session_a(struct cafe_instrument **session,
	struct cafe_ep_inst **root_ep_inst,
	struct cafe_ep_inst **ep_inst)
{
	struct cafe_ep_spec *hpc_root_spec, *hpc_spec1, *hpc_spec2;

	/* locate the specs by name */
	hpc_root_spec = cafe_ep_spec_find("perf_counter_provider");
	CUT_ASSERT_NOT_NULL(hpc_root_spec);

	hpc_spec1 = cafe_ep_spec_find("hpc_test_spec_1");
	CUT_ASSERT_NOT_NULL(hpc_spec1);

	hpc_spec2 = cafe_ep_spec_find("hpc_test_spec_2");
	CUT_ASSERT_NOT_NULL(hpc_spec2);

	CUT_ASSERT_CAFE_OK(cafe_instrument_create("hpc_test_cafe2_a", session));

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(*session, hpc_root_spec, 0, NULL, root_ep_inst));

	/* We use fake event points, so this instance will only
	 * generate event on the session thread. */
	(*root_ep_inst)->flags |= CAFE_EP_INST_FLAG_OUTPUT_SESSION_THREAD;

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(*session, hpc_spec1, 0, NULL, ep_inst));

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_connect(*root_ep_inst, *ep_inst));

	CUT_ASSERT_EQUAL(0, enable_required_root_output_events_a(*root_ep_inst));

	CUT_ASSERT_CAFE_OK(cafe_inst_enable_all_output_events(*ep_inst));

	return 0;
}

static int create_session_b(struct cafe_instrument **session)
{
	struct cafe_ep_inst *hpc_root_inst_b, *hpc_test_inst1_b;
	struct cafe_ep_spec *hpc_root_spec, *hpc_spec1, *hpc_spec2;

	/* locate the specs by name */
	hpc_root_spec = cafe_ep_spec_find("perf_counter_provider");
	CUT_ASSERT_NOT_NULL(hpc_root_spec);

	hpc_spec1 = cafe_ep_spec_find("hpc_test_spec_1");
	CUT_ASSERT_NOT_NULL(hpc_spec1);

	hpc_spec2 = cafe_ep_spec_find("hpc_test_spec_2");
	CUT_ASSERT_NOT_NULL(hpc_spec2);

	CUT_ASSERT_CAFE_OK(cafe_instrument_create("hpc_test_cafe2_b", session));

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(*session, hpc_root_spec, 0, NULL, &hpc_root_inst_b));

	/* We use fake event points, so this instance will only
	 * generate event on the session thread. */
	hpc_root_inst_b->flags |= CAFE_EP_INST_FLAG_OUTPUT_SESSION_THREAD;

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_create(*session, hpc_spec2, 0, NULL, &hpc_test_inst1_b));

	CUT_ASSERT_CAFE_OK(cafe_ep_inst_connect(hpc_root_inst_b, hpc_test_inst1_b));

	CUT_ASSERT_EQUAL(0, enable_required_root_output_events_b(hpc_root_inst_b));

	CUT_ASSERT_CAFE_OK(cafe_inst_enable_all_output_events(hpc_test_inst1_b));

	return 0;
}

/*
 * Tests starts two sessions which share the same event
 */
static int do_test1(void)
{
	int cores;
	struct cafe_ep_inst *hpc_root_inst, *ep_inst;
	struct cafe_instrument *session_a, *session_b;
	HPC_COUNTER tot_count_a_1 = 0, tot_count_a_2 = 0, tot_count_b_1 = 0, tot_count_b_3 = 0;
	int ctr_mask = 1<<CTR_SA_EV1 | 1<<CTR_SA_EV2 | 1<<CTR_SB_EV1 | 1<< CTR_SB_EV3;

	hpc_test_clear_counters();

	CUT_ASSERT_EQUAL(0, create_session_a(&session_a, &hpc_root_inst, &ep_inst));
	CUT_ASSERT_EQUAL(0, create_session_b(&session_b));

	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session_a));
	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session_b));

	CUT_ASSERT_CAFE_OK(cafe_instrument_start(session_a));
	CUT_ASSERT_CAFE_OK(cafe_instrument_start(session_b));

	/* reload is 800000 */
	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 900000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	tot_count_b_3 = 900000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 200000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_EQUAL(0, hpc_test_generate_isr(0));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 700000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	tot_count_a_1 = 900000 + 200000 + 700000;
	tot_count_b_1 = 900000 + 200000 + 700000;
	tot_count_b_3 += 700000 + 200000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 800001, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	tot_count_a_2 = 900000 + 200000 + 700000 + 800001;
	tot_count_b_3 += 800001;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	/* inc counters a little before we stop session b */
	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 5000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_EQUAL(0, hpc_test_generate_isr(0));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	/* Stop session b */
	ctr_mask = 1<<CTR_SA_EV1 | 1<<CTR_SA_EV2;
	CUT_ASSERT_CAFE_OK(cafe_instrument_stop(session_b));

	/* reload is still 8000000 */
	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 800000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	tot_count_a_1 += 800001 + 5000 + 800000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 100000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_EQUAL(0, hpc_test_generate_isr(0));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 700000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 800000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	tot_count_a_1 += 100000 + 700000 + 800000;
	tot_count_a_2 += 5000 + 800000 + 100000 + 700000 + 800000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	/* Start session b */
	CUT_ASSERT_CAFE_OK(cafe_instrument_start(session_b));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 800000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	tot_count_b_1 += 800001 + 5000 + 800000;
	tot_count_b_3 += 5000 + 800000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 800000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	tot_count_a_1 += 800000 + 800000;
	tot_count_b_3 += 800000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 800000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	tot_count_a_2 += 800000 + 800000 + 800000;
	tot_count_b_1 += 800000 + 800000;
	tot_count_b_3 += 800000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	/* end of test */
	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session_a));
	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session_b));

	return 0;
}

/*
 * Tests starts a sessions. Generates the event. Starts a second sessions which shares the
 * same event. Generates more events.
 */

static int do_test2(void)
{
	int cores;
	struct cafe_ep_inst *hpc_root_inst, *ep_inst;
	struct cafe_instrument *session_a, *session_b;
	HPC_COUNTER tot_count_a_1 = 0, tot_count_a_2 = 0, tot_count_b_1 = 0, tot_count_b_3 = 0;
	int ctr_mask = 0;

	hpc_test_clear_counters();

	CUT_ASSERT_EQUAL(0, create_session_a(&session_a, &hpc_root_inst, &ep_inst));
	CUT_ASSERT_EQUAL(0, create_session_b(&session_b));

	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session_a));
	CUT_ASSERT_CAFE_OK(cafe_instrument_refresh_output_event_actions(session_b));

	CUT_ASSERT_CAFE_OK(cafe_instrument_start(session_a));

	/* reload is 1000000 */
	ctr_mask |= 1<<CTR_SA_EV1 | 1<<CTR_SA_EV2;
	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 1000000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	tot_count_a_1 += 1000000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 1000000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	tot_count_a_1 += 1000000;
	tot_count_a_2 += 1000000 + 1000000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	/* start another session which reduces the reload for the ctr_count */
	CUT_ASSERT_CAFE_OK(cafe_instrument_start(session_b));

	/* reload is now 800000 */
	ctr_mask |= 1<<CTR_SB_EV1 | 1<<CTR_SB_EV3;
	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 400000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_EQUAL(0, hpc_test_generate_isr(0));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 400000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	tot_count_b_3 += 400000 + 400000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 800000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	tot_count_a_1 += 400000 + 400000 + 800000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	tot_count_b_1 += 400000 + 400000 + 800000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	tot_count_b_3 += 800000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_EQUAL(0, hpc_test_generate_event(ctr_mask, 800000, session_a, &cores, hpc_total_cycles_VT_REF, NULL));
	CUT_ASSERT_NOT_EQUAL(0, hpc_test_generate_isr(1));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV1, tot_count_a_1, cores));
	tot_count_a_2 += 400000 + 400000 + 800000 + 800000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SA_EV2, tot_count_a_2, cores));
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV1, tot_count_b_1, cores));
	tot_count_b_3 += 800000;
	CUT_ASSERT_EQUAL(0, hpc_test_check_counters(CTR_SB_EV3, tot_count_b_3, cores));

	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session_a));
	CUT_ASSERT_CAFE_OK(cafe_instrument_delete(session_b));

	return 0;
}

/******************************************************************************
 * Test code2. Test multiple events of the same type share the same hw/ctr_count
 * resource.
 */

int hpc_test_cafe2(void)
{
	unsigned int i;
	CUT_ASSERT_CAFE_OK(hpc_tst_init());
	CUT_ASSERT_CAFE_OK(cafe_lib_init());

	CAFE_LOCK();

	if (hpc_hw_drv_type != TEST_VXW_HPC_HW_DRV) {
		CUT_ASSERT_CAFE_OK(hpc_register_configuration());
	}

	CUT_ASSERT_EQUAL(0, register_hpc_test_spec1());
	CUT_ASSERT_EQUAL(0, register_hpc_test_spec2());

	for (i = 0; i < NUM_TEST_ITERATIONS; i++) {
		CUT_ASSERT_EQUAL(0, do_test1());
		CUT_ASSERT_EQUAL(0, do_test2());
	}

	if (hpc_hw_drv_type != TEST_VXW_HPC_HW_DRV) {
		CUT_ASSERT_CAFE_OK(hpc_unregister_configuration());
	}

	CUT_ASSERT_EQUAL(0, unregister_hpc_test_spec1());
	CUT_ASSERT_EQUAL(0, unregister_hpc_test_spec2());

	CAFE_UNLOCK();

	CUT_ASSERT_CAFE_OK(hpc_tst_finit());
	cafe_lib_release();

	return 0;
}
