/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <test/cafe_CUnitTest.h>
#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/hpcHwDrvTest.h>
#include <hpc/drv_evt_cfg/ppc/e500DrvEvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500CoreEvtCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500CoreDrvCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500DeviceDrvCfg.h>

#include "hpc_test_api.h"

static int num_cores;
static hpc_hw_drv_test_e500_type test_type;
static struct e500EventDescription *evt_desc_device;
static struct driverInstE500 *drv_inst_device;
static unsigned int num_events_device;

static int set_core_events(void)
{
	switch (test_type) {
	case HPC_HW_DRV_TEST_MPC8540:
	case HPC_HW_DRV_TEST_MPC8548:
	case HPC_HW_DRV_TEST_P2020:
		/* Event: Com:10 */
		hpc_test_drv_evt[0].u.e500.evtCode = 10;
		hpc_test_drv_evt[0].u.e500.ctrMask = ~0; /* All counters */
		hpc_test_drv_evt[0].reload = 10000;

		/* Event: Com:1 */
		hpc_test_drv_evt[1].u.e500.evtCode = 1;
		hpc_test_drv_evt[1].u.e500.ctrMask = ~0; /* All counters */
		hpc_test_drv_evt[1].reload = 10000;
		hpc_test_drv_num_evts = 2;
		return 0;
	}
	return 1;
}

static int set_device_events(void)
{
	switch (test_type) {
	case HPC_HW_DRV_TEST_P2020:
		/* Event: C0 System Cycles (64-bit counter) */
		hpc_test_drv_evt[0].u.e500.evtCode = 0 + 64; /* Device counter specific events are offset by 64 */
		hpc_test_drv_evt[0].u.e500.ctrMask = 1<<0; /* Counter 0 */
		hpc_test_drv_evt[0].reload = 1000000;

		/* Counter specific event: C4:121 level2 data cache hit */
		hpc_test_drv_evt[1].u.e500.evtCode = 121;
		hpc_test_drv_evt[1].u.e500.ctrMask = (1<<4); /* Counter 4 */
		hpc_test_drv_evt[1].reload = 1000;

		/* Reference event: Ref:23 level2 inst cache miss */
		hpc_test_drv_evt[2].u.e500.evtCode = 121;
		hpc_test_drv_evt[2].u.e500.ctrMask = ~1; /* not counter 0 */
		hpc_test_drv_evt[2].reload = 1000;
		hpc_test_drv_num_evts = 3;
		return 0;

	case HPC_HW_DRV_TEST_MPC8540:
		/* Event: C0 System Cycles (64-bit counter) */
		hpc_test_drv_evt[0].u.e500.evtCode = 0 + 64; /* Device counter specific events are offset by 64 */
		hpc_test_drv_evt[0].u.e500.ctrMask = 1; /* Counter 0 */
		hpc_test_drv_evt[0].reload = 1000000;

		/* Event Ref:10 Cycles a read is returning data from DDR SDRAM */
		hpc_test_drv_evt[1].u.e500.evtCode = 10;
		hpc_test_drv_evt[1].u.e500.ctrMask = ~0x1; /* Not counter 0 */
		hpc_test_drv_evt[1].reload = 10000;
		hpc_test_drv_num_evts = 2;
		return 0;

	case HPC_HW_DRV_TEST_MPC8548:
		/* Event: C0 System Cycles (64-bit counter) */
		hpc_test_drv_evt[0].u.e500.evtCode = 0 + 64; /* counter specific events are offset by 64 */
		hpc_test_drv_evt[0].u.e500.ctrMask = 1; /* Counter 0 */
		hpc_test_drv_evt[0].reload = 1000000;

		/* Event C2:59 Core instruction accesses to L2 that miss */
		hpc_test_drv_evt[1].u.e500.evtCode = 59 + 64;
		hpc_test_drv_evt[1].u.e500.ctrMask = (1<<2);
		hpc_test_drv_evt[1].reload = 500;

		/* Event C4:57 Core data accesses to L2 that miss */
		hpc_test_drv_evt[2].u.e500.evtCode = 57 + 64;
		hpc_test_drv_evt[2].u.e500.ctrMask = (1<<4);
		hpc_test_drv_evt[2].reload = 500;

		/* Event Ref:22 Core instruction accesses to L2 that hit */
		hpc_test_drv_evt[3].u.e500.evtCode = 22;
		hpc_test_drv_evt[3].u.e500.ctrMask = ~0x1; /* Not counter 0 */
		hpc_test_drv_evt[3].reload = 500;

		/* Event Ref:23 Core data accesses to L2 that hit */
		hpc_test_drv_evt[4].u.e500.evtCode = 23;
		hpc_test_drv_evt[4].u.e500.ctrMask = ~0x1; /* Not counter 0 */
		hpc_test_drv_evt[4].reload = 500;

		hpc_test_drv_num_evts = 5;
		return 0;
	}
	return 1;
}

static int hpc_drv_api_test_ppc_e500(void)
{
	/* test core counters */
	CUT_ASSERT_EQUAL(0, set_core_events());

	CUT_ASSERT_EQUAL(OK, hpcHwE500DriverInit());
	CUT_ASSERT_EQUAL(OK, hpcCafeE500CoreDrvCfgInit(malloc, free, num_cores, hpcDrvEvtDescE500, hpcE500NumEvents, &hpcHwDrvE500Core));
	hpc_test_drv = hpcCafeE500DrvCfgGet();
	hpc_test_drv_funcs = NULL;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	CUT_ASSERT_EQUAL(OK, hpcCafeE500CoreDrvCfgFinit(hpcCafeE500DrvCfgGet()));
	CUT_ASSERT_EQUAL(OK, hpcHwE500DriverFinit());

	/* Tests for device counters */
	CUT_ASSERT_EQUAL(0, set_device_events());

	CUT_ASSERT_EQUAL(OK, hpcHwE500DriverInit());
	CUT_ASSERT_EQUAL(OK, hpcCafeE500DeviceDrvCfgInit(malloc, free, evt_desc_device, num_events_device, drv_inst_device, NULL));
	hpc_test_drv = hpcCafeE500DeviceDrvCfgGet();
	hpc_test_drv_funcs = NULL;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	CUT_ASSERT_EQUAL(OK, hpcCafeE500DeviceDrvCfgFinit(hpcCafeE500DeviceDrvCfgGet()));
	CUT_ASSERT_EQUAL(OK, hpcHwE500DriverFinit());

	return 0;
}

void hpcHwDrvApiTestPpcE500Init(int cores,
	hpc_hw_drv_test_e500_type type,
	struct e500EventDescription *pEvtDescDevice,
	struct driverInstE500 *pDrvInstDevice,
	const unsigned int numEventsDevice)
{
	hpc_driver_api_test_func = hpc_drv_api_test_ppc_e500;

	num_cores = cores;
	test_type = type;
	evt_desc_device = pEvtDescDevice;
	drv_inst_device = pDrvInstDevice;
	num_events_device = numEventsDevice;
}
