/* Copyright (c) 2010-2011, 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
09sep14,wzc  support arm hpc events.(US17715)
28mar14,sdt  fixed build issue in tcf-tools env.
13mar14,wzc  support arm hpc(US17715)
*/

#include <test/cafe_CUnitTest.h>
#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/hpc_drv/ppc/e500Drv.h>
#include <hpc/drv_evt_cfg/ppc/e500CoreDrvCfg.h>
#include <hpc/drv_evt_cfg/ppc/e500CoreEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPmEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86core2EvtCfg.h>
#include <hpc/hpc_drv/arm/armDrv.h>
#include <hpc/drv_evt_cfg/arm/armDrvCfg.h>
#include <hpc/drv_evt_cfg/arm/armCtxA8EvtCfg.h>
#include <hpc/drv_evt_cfg/arm/armCtxA9EvtCfg.h>
#include <hpc/drv_evt_cfg/arm/armCtxA15EvtCfg.h>

#include "hpc_test_api.h"
#include "../tests_mock/hpcHwDriverTest.h"
#include "../hpc_utils.h"
#include "../tests_mock/e500DrvTest.h"
#include "../tests_mock/i86ArchPm1DrvTest.h"
#include "../tests_mock/i86ArchPm2DrvTest.h"
#include "../tests_mock/armDrvTest.h"

CUT_BEGIN_TEST_HARNESS(hpcHwDrvUnitTestSuiteGeneric)
{
	STATUS status;

	CUT_RUN_SUITE_TESTS(hpc_driver_utils_suite);

	/* arm driver test */
	/* Event: cycle clock */
	hpc_test_drv_evt[0].u.arm.evtCode = 0x11;
	hpc_test_drv_evt[0].u.arm.ctrMask = 0x80000000; /* Cycle clock counters */
	hpc_test_drv_evt[0].reload = 0x10000;

	/* Event: Com:1 */
	hpc_test_drv_evt[1].u.arm.evtCode = 3;
	hpc_test_drv_evt[1].u.arm.ctrMask = 0x7fffffff; /* All counters */
	hpc_test_drv_evt[1].reload = 0x10000;
	hpc_test_drv_num_evts = 2;

	CUT_ASSERT_EQUAL(OK, hpcHwArmDriverInit());
	CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgInit(malloc, free, 1, hpcDrvEvtDescArmCtxA8, hpcArmCtxA8NumEvents, &hpcHwDrvArmCtxA8));
	hpc_test_drv = hpcCafeArmDrvCfgGet();
	hpc_test_drv_funcs = &hpcHwDrvArmTestFuncs;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgFinit(hpcCafeArmDrvCfgGet()));
	CUT_ASSERT_EQUAL(OK, hpcHwArmDriverFinit());

	CUT_ASSERT_EQUAL(OK, hpcHwArmDriverInit());
	CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgInit(malloc, free, 1, hpcDrvEvtDescArmCtxA9, hpcArmCtxA9NumEvents, &hpcHwDrvArmCtxA9));
	hpc_test_drv = hpcCafeArmDrvCfgGet();
	hpc_test_drv_funcs = &hpcHwDrvArmTestFuncs;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgFinit(hpcCafeArmDrvCfgGet()));
	CUT_ASSERT_EQUAL(OK, hpcHwArmDriverFinit());
	
	CUT_ASSERT_EQUAL(OK, hpcHwArmDriverInit());
	CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgInit(malloc, free, 1, hpcDrvEvtDescArmCtxA15, hpcArmCtxA15NumEvents, &hpcHwDrvArmCtxA15));
	hpc_test_drv = hpcCafeArmDrvCfgGet();
	hpc_test_drv_funcs = &hpcHwDrvArmTestFuncs;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	CUT_ASSERT_EQUAL(OK, hpcCafeArmDrvCfgFinit(hpcCafeArmDrvCfgGet()));
	CUT_ASSERT_EQUAL(OK, hpcHwArmDriverFinit());

	/* e500 driver test */
	/* Event: Com:10 */
	hpc_test_drv_evt[0].u.e500.evtCode = 10;
	hpc_test_drv_evt[0].u.e500.ctrMask = ~0; /* All counters */
	hpc_test_drv_evt[0].reload = 10000;

	/* Event: Com:1 */
	hpc_test_drv_evt[1].u.e500.evtCode = 1;
	hpc_test_drv_evt[1].u.e500.ctrMask = ~0; /* All counters */
	hpc_test_drv_evt[1].reload = 10000;
	hpc_test_drv_num_evts = 2;

	CUT_ASSERT_EQUAL(OK, hpcHwE500DriverInit());
	CUT_ASSERT_EQUAL(OK, hpcCafeE500CoreDrvCfgInit(malloc, free, 1, hpcDrvEvtDescE500, hpcE500NumEvents, &hpcHwDrvE500Core));
	hpc_test_drv = hpcCafeE500DrvCfgGet();
	hpc_test_drv_funcs = &hpcHwDrvE500CoreTestFuncs;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	CUT_ASSERT_EQUAL(OK, hpcCafeE500CoreDrvCfgFinit(hpcCafeE500DrvCfgGet()));
	CUT_ASSERT_EQUAL(OK, hpcHwE500DriverFinit());

	/* x86 arch V1 driver */
        initPentiumRegs();

        /* INSTRUCTION_RETIRED: Instruction retired */
	hpc_test_drv_evt[0].u.i86Arch1.eventNum = 0xC0;
	hpc_test_drv_evt[0].u.i86Arch1.eventUmask = 0x0;
	hpc_test_drv_evt[0].u.i86Arch1.restrictedCtrMask = 0;
	hpc_test_drv_evt[0].reload = 1000000;
	hpc_test_drv_num_evts = 1;

	status = hpcI86ArchPerfMonInit(malloc, free, 0, 0);
	CUT_ASSERT_EQUAL(OK, status);
	hpc_test_drv = i86ArchPmV1Configure(2, HPC_NUM_I86_CORE2_EVENTS, x86core2EvtDesc);
	CUT_ASSERT_NOT_NULL(hpc_test_drv);
	status = hpcHwI86ArchPm1DriverInit();
	CUT_ASSERT_EQUAL(OK, status);
	hpc_test_drv_funcs = &hpcHwDrvX86ArchV1TestFuncs;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	status = hpcHwI86ArchPm1DriverFinit();
	CUT_ASSERT_EQUAL(OK, status);
	status = i86ArchPmV1Free(hpc_test_drv);

	/* x86 arch V2 driver */
        initPentiumRegs();

        /* CPU_CLK_UNHALTED-CORE: Unhalted core cycles */
	hpc_test_drv_evt[0].u.i86Arch2.counter = 0;
	hpc_test_drv_evt[0].reload = 1000000;
	/* INSTRUCTION_RETIRED: Instruction retired */
	hpc_test_drv_evt[1].u.i86Arch2.counter = 1;
	hpc_test_drv_evt[1].reload = 1000000;
	/* CPU_CLK_UNHALTED-REF: Unhalted reference cycles Measures bus cycle */
	hpc_test_drv_evt[2].u.i86Arch2.counter = 2;
	hpc_test_drv_evt[2].reload = 1000000;
	hpc_test_drv_num_evts = 3;

	status = hpcI86ArchPerfMonInit(malloc, free, 0, 0);
	CUT_ASSERT_EQUAL(OK, status);
	hpc_test_drv = i86ArchPmV2Configure(2);
	CUT_ASSERT_NOT_NULL(hpc_test_drv);
	status = hpcHwI86ArchPm2DriverInit();
	CUT_ASSERT_EQUAL(OK, status);
	hpc_test_drv_funcs = &hpcHwDrvX86ArchV2TestFuncs;
	CUT_RUN_SUITE_TESTS(hpc_driver_api_suite);
	status = hpcHwI86ArchPm2DriverFinit();
	CUT_ASSERT_EQUAL(OK, status);
	status = i86ArchPmV2Free(hpc_test_drv);
}
CUT_END_TEST_HARNESS
