/* Copyright (c) 2010 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdio.h>
#include <stdlib.h>

#include <test/cafe_CUnitTest.h>

#include <hpc/hpc_ep.h>
#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPmEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86core2EvtCfg.h>

#include "../drv_evt_cfg/i86/i86ArchPmEvtCfgP.h"

/* Array index for a 2 dimensional array */
#define A2D(ROW,COLUMN,NCOLUMNS) (ROW * NCOLUMNS + COLUMN)

#define ARCH_V1_NUM_EVENTS_SET(N)	((0xff & N) << 24)
#define ARCH_V1_NUM_CTR_BITS_SET(B)	((0xff & B) << 16)
#define ARCH_V1_NUM_CTRS_SET(C)		((0xff & C) << 8)
#define ARCH_PM_VERSION_SET(V)		(0xff & V)

#define ARCH_V1_Events				(0xff)
#define ARCH_V1_Branch_Mispredicts_Retired	(1 << 6)
#define ARCH_V1_Branch_Instructions_Retired	(1 << 5)
#define ARCH_V1_Last_Level_Cache_Misses		(1 << 4)
#define ARCH_V1_Last_Level_Cache_References	(1 << 3)
#define ARCH_V1_Reference_Cycles		(1 << 2)
#define ARCH_V1_Instructions_Retired		(1 << 1)
#define ARCH_V1_Core_Cycles			(1 << 0)


#define ARCH_V2_NUM_CTRS_SET(C)		(0x1f & C)
#define ARCH_V2_NUM_CTR_BITS_SET(B)	((0xff & B) << 5)

static int test_malloc_count;
static int test_free_count;
static int test_malloc_error;

static void * test_malloc(size_t size)
{
	void * p;
	test_malloc_count++;
	if (test_malloc_error) {
		test_malloc_error = 0;
		return NULL;
	}
	p = malloc(size);
	return p;
}

static void test_free(void *m)
{
	test_free_count++;
	free(m);
}

static int archPerfMonV1ConfigureTest1(void)
{
	struct hpcDriverConfig * v1_drv_config;
        unsigned int eax, ebx, num_cores, core, num_counters, counter, idx2d;
        unsigned int event, num_events;
        STATUS rc;
        struct driverInstArchV1 * pInst;
	struct i86Arch1EventDescription *pI86EventDesc;

        test_malloc_count = test_free_count = test_malloc_error = 0;

	rc = hpcI86ArchPerfMonInit(test_malloc, test_free, 0, 0);
	CUT_ASSERT_EQUAL(OK, rc);

        num_counters = 5;
        num_events = 7;
        eax = ARCH_PM_VERSION_SET(1) |
		ARCH_V1_NUM_CTRS_SET(num_counters) |
		ARCH_V1_NUM_CTR_BITS_SET(40) |
		ARCH_V1_NUM_EVENTS_SET(num_events);

        ebx = ARCH_V1_Events &
		~(ARCH_V1_Branch_Mispredicts_Retired |
		ARCH_V1_Branch_Instructions_Retired |
		ARCH_V1_Last_Level_Cache_Misses |
		ARCH_V1_Last_Level_Cache_References |
		ARCH_V1_Reference_Cycles |
		ARCH_V1_Instructions_Retired |
		ARCH_V1_Core_Cycles);

        num_cores = 2;
        i86ArchPmTest.setCpuidValues(eax, ebx, 0, 0);
	v1_drv_config = i86ArchPmV1Configure(num_cores, 0, NULL);
	CUT_ASSERT_NOT_NULL(v1_drv_config);
	CUT_ASSERT_EQUAL(1, test_malloc_count);
	CUT_ASSERT_EQUAL(0, test_free_count);
	CUT_ASSERT_EQUAL(6 + 1, v1_drv_config->numEvents);
	CUT_ASSERT_EQUAL(2, v1_drv_config->numCpuCores);

	/* check the allocated memory is valid */
	pInst = v1_drv_config->pDrvInst;
	for (core = 0; core < num_cores; core++)
		for (counter = 0; counter < num_counters; counter++) {
			idx2d = A2D(core, counter, num_counters);
			pInst->ppDrvHwCtrData[idx2d] = NULL;
		}

	CUT_ASSERT_NOT_NULL(pInst->pDriverName);
	CUT_ASSERT_EQUAL(40, pInst->numCounterBits);
	CUT_ASSERT_EQUAL(5, pInst->numCounters);

	/* Check the events exist */
	pI86EventDesc = (struct i86Arch1EventDescription *)v1_drv_config->pEventDesc;
	for (event = 0; event < num_events; event++) {
/*		printf("event = %d %s\n", event, pI86EventDesc[event].eventVtStr); */
		CUT_ASSERT_NOT_NULL(&pI86EventDesc[event].eventVtStr);
	}


	rc = i86ArchPmV1Free(v1_drv_config);
	CUT_ASSERT_EQUAL(OK, rc);
	CUT_ASSERT_EQUAL(1, test_malloc_count);
	CUT_ASSERT_EQUAL(1, test_free_count);

	return 0;
}

static int archPerfMonV1ConfigureTest2(void)
{
	struct hpcDriverConfig * v1_drv_config;
        unsigned int eax, ebx, num_cores, num_counters;
        STATUS rc;

        test_malloc_count = test_free_count = test_malloc_error = 0;

	rc = hpcI86ArchPerfMonInit(test_malloc, test_free, 0, 0);
	CUT_ASSERT_EQUAL(OK, rc);

        ebx =  ARCH_V1_Events & ~(ARCH_V1_Branch_Mispredicts_Retired |
		ARCH_V1_Branch_Instructions_Retired |
		ARCH_V1_Last_Level_Cache_Misses |
		ARCH_V1_Last_Level_Cache_References |
		ARCH_V1_Reference_Cycles |
		ARCH_V1_Instructions_Retired |
		ARCH_V1_Core_Cycles);

	/* Test for zero counters error */
        num_counters = 0;
        eax = ARCH_PM_VERSION_SET(1) |
		ARCH_V1_NUM_CTRS_SET(num_counters) |
		ARCH_V1_NUM_CTR_BITS_SET(40) |
		ARCH_V1_NUM_EVENTS_SET(7);

        num_cores = 2;
        i86ArchPmTest.setCpuidValues(eax, ebx, 0, 0);
	v1_drv_config = i86ArchPmV1Configure(num_cores, 0, NULL);
	CUT_ASSERT_NULL(v1_drv_config);
	CUT_ASSERT_EQUAL(0, test_malloc_count);
	CUT_ASSERT_EQUAL(0, test_free_count);

	/* Test for zero cores error */
        num_counters = 5;
        eax = ARCH_PM_VERSION_SET(1) |
		ARCH_V1_NUM_CTRS_SET(num_counters) |
		ARCH_V1_NUM_CTR_BITS_SET(40) |
		ARCH_V1_NUM_EVENTS_SET(7);

        num_cores = 0;
        i86ArchPmTest.setCpuidValues(eax, ebx, 0, 0);
	v1_drv_config = i86ArchPmV1Configure(num_cores, 0, NULL);
	CUT_ASSERT_NULL(v1_drv_config);
	CUT_ASSERT_EQUAL(0, test_malloc_count);
	CUT_ASSERT_EQUAL(0, test_free_count);

	/* Test for zero counter bit width error */
        num_counters = 5;
        eax = ARCH_PM_VERSION_SET(1) |
		ARCH_V1_NUM_CTRS_SET(num_counters) |
		ARCH_V1_NUM_CTR_BITS_SET(0) |
		ARCH_V1_NUM_EVENTS_SET(7);

        num_cores = 2;
        i86ArchPmTest.setCpuidValues(eax, ebx, 0, 0);
	v1_drv_config = i86ArchPmV1Configure(num_cores, 0, NULL);
	CUT_ASSERT_NULL(v1_drv_config);
	CUT_ASSERT_EQUAL(0, test_malloc_count);
	CUT_ASSERT_EQUAL(0, test_free_count);

	/* Test for malloc error */
	test_malloc_error = 1;
        num_counters = 5;
        eax = ARCH_PM_VERSION_SET(1) |
		ARCH_V1_NUM_CTRS_SET(num_counters) |
		ARCH_V1_NUM_CTR_BITS_SET(40) |
		ARCH_V1_NUM_EVENTS_SET(7);

        num_cores = 2;
        i86ArchPmTest.setCpuidValues(eax, ebx, 0, 0);
	v1_drv_config = i86ArchPmV1Configure(num_cores, 0, NULL);
	CUT_ASSERT_NULL(v1_drv_config);
	CUT_ASSERT_EQUAL(1, test_malloc_count);
	CUT_ASSERT_EQUAL(0, test_free_count);

	/* check its still working */
        num_counters = 5;
        eax = ARCH_PM_VERSION_SET(1) |
		ARCH_V1_NUM_CTRS_SET(num_counters) |
		ARCH_V1_NUM_CTR_BITS_SET(40) |
		ARCH_V1_NUM_EVENTS_SET(7);

        num_cores = 2;
        i86ArchPmTest.setCpuidValues(eax, ebx, 0, 0);
	v1_drv_config = i86ArchPmV1Configure(num_cores, 0, NULL);
	CUT_ASSERT_NOT_NULL(v1_drv_config);
	CUT_ASSERT_EQUAL(2, test_malloc_count);
	CUT_ASSERT_EQUAL(0, test_free_count);

	rc = i86ArchPmV1Free(v1_drv_config);
	CUT_ASSERT_EQUAL(OK, rc);
	CUT_ASSERT_EQUAL(2, test_malloc_count);
	CUT_ASSERT_EQUAL(1, test_free_count);

	return 0;
}

static int archPerfMonV1ConfigureTest3(void)
{
	struct hpcDriverConfig * v1_drv_config;
        unsigned int eax, ebx, num_cores, num_counters;
        unsigned int event, num_events, num_non_arch_events;
        STATUS rc;
        struct driverInstArchV1 * pInst;
	struct i86Arch1EventDescription *pI86EventDesc;

        test_malloc_count = test_free_count = test_malloc_error = 0;

	rc = hpcI86ArchPerfMonInit(test_malloc, test_free, 0, 0);
	CUT_ASSERT_EQUAL(OK, rc);

        num_counters = 5;
        num_events = 4;
        num_non_arch_events = HPC_NUM_I86_CORE2_EVENTS;
        eax = ARCH_PM_VERSION_SET(1) |
		ARCH_V1_NUM_CTRS_SET(num_counters) |
		ARCH_V1_NUM_CTR_BITS_SET(40) |
		ARCH_V1_NUM_EVENTS_SET(num_events);

        ebx =  ARCH_V1_Events & ~(ARCH_V1_Branch_Mispredicts_Retired |
		ARCH_V1_Branch_Instructions_Retired |
		ARCH_V1_Instructions_Retired |
		ARCH_V1_Reference_Cycles);

        num_cores = 2;
        i86ArchPmTest.setCpuidValues(eax, ebx, 0, 0);
	v1_drv_config = i86ArchPmV1Configure(num_cores,
			HPC_NUM_I86_CORE2_EVENTS, x86core2EvtDesc);
	CUT_ASSERT_NOT_NULL(v1_drv_config);
	CUT_ASSERT_EQUAL(1, test_malloc_count);
	CUT_ASSERT_EQUAL(0, test_free_count);
	CUT_ASSERT_EQUAL(num_non_arch_events + num_events, v1_drv_config->numEvents);
	CUT_ASSERT_EQUAL(2, v1_drv_config->numCpuCores);

	/* Check the events exist */
	pI86EventDesc = (struct i86Arch1EventDescription *)v1_drv_config->pEventDesc;
	for (event = 0; event < (num_non_arch_events + num_events); event++) {
/*		printf("event = %d %s\n", event, pI86EventDesc[event].eventVtStr); */
		CUT_ASSERT_NOT_NULL(&pI86EventDesc[event].eventVtStr);
	}

	pInst = v1_drv_config->pDrvInst;
	CUT_ASSERT_NOT_NULL(pInst->pDriverName);
	CUT_ASSERT_EQUAL(40, pInst->numCounterBits);
	CUT_ASSERT_EQUAL(5, pInst->numCounters);

	rc = i86ArchPmV1Free(v1_drv_config);
	CUT_ASSERT_EQUAL(OK, rc);
	CUT_ASSERT_EQUAL(1, test_malloc_count);
	CUT_ASSERT_EQUAL(1, test_free_count);

	return 0;
}

static int archPerfMonV2ConfigureTest1(void)
{
	struct hpcDriverConfig * v2_drv_config;
        struct driverInstArchV2 * pInst;
	unsigned int eax, edx;
	int num_cores;
	STATUS rc;

        test_malloc_count = test_free_count = test_malloc_error = 0;

	rc = hpcI86ArchPerfMonInit(test_malloc, test_free, 0, 0);
	CUT_ASSERT_EQUAL(OK, rc);

        num_cores = 2;
        eax = ARCH_PM_VERSION_SET(2);
        edx = ARCH_V2_NUM_CTRS_SET(3) | ARCH_V2_NUM_CTR_BITS_SET(40);
        i86ArchPmTest.setCpuidValues(eax, 0, 0, edx);
	v2_drv_config = i86ArchPmV2Configure(num_cores);
	CUT_ASSERT_NOT_NULL(v2_drv_config);
	CUT_ASSERT_EQUAL(3+2, v2_drv_config->numEvents);
	CUT_ASSERT_EQUAL(2, v2_drv_config->numCpuCores);
	CUT_ASSERT_EQUAL(1, test_malloc_count);
	CUT_ASSERT_EQUAL(0, test_free_count);

	pInst = v2_drv_config->pDrvInst;
	CUT_ASSERT_NOT_NULL(pInst->pDriverName);
	CUT_ASSERT_EQUAL(40, pInst->numCounterBits);
	CUT_ASSERT_EQUAL(3, pInst->numCounters);

	rc = i86ArchPmV2Free(v2_drv_config);
	CUT_ASSERT_EQUAL(OK, rc);
	CUT_ASSERT_EQUAL(1, test_malloc_count);
	CUT_ASSERT_EQUAL(1, test_free_count);

	return 0;
}

static int archPerfMonV2ConfigureTest2(void)
{
	struct hpcDriverConfig * v2_drv_config;
	unsigned int eax, edx;
	int num_cores;
	STATUS rc;

        test_malloc_count = test_free_count = test_malloc_error = 0;

	rc = hpcI86ArchPerfMonInit(test_malloc, test_free, 0, 0);
	CUT_ASSERT_EQUAL(OK, rc);

        num_cores = 2;
        eax = ARCH_PM_VERSION_SET(2);
        edx = ARCH_V2_NUM_CTRS_SET(4) | ARCH_V2_NUM_CTR_BITS_SET(40);
        i86ArchPmTest.setCpuidValues(eax, 0, 0, edx);
	v2_drv_config = i86ArchPmV2Configure(num_cores);
	CUT_ASSERT_NULL(v2_drv_config);

	return 0;
}

static int archPerfMonV2ConfigureTest3(void)
{
	struct hpcDriverConfig * v2_drv_config;
	unsigned int eax, edx;
	int num_cores;
	STATUS rc;

        test_malloc_count = test_free_count = test_malloc_error = 0;

	rc = hpcI86ArchPerfMonInit(test_malloc, test_free, 0, 0);
	CUT_ASSERT_EQUAL(OK, rc);

        num_cores = 2;
        eax = ARCH_PM_VERSION_SET(2);
        edx = ARCH_V2_NUM_CTRS_SET(3) | ARCH_V2_NUM_CTR_BITS_SET(40);
	test_malloc_error = 1;
        i86ArchPmTest.setCpuidValues(eax, 0, 0, edx);
	v2_drv_config = i86ArchPmV2Configure(num_cores);
	CUT_ASSERT_NULL(v2_drv_config);
	CUT_ASSERT_EQUAL(1, test_malloc_count);
	CUT_ASSERT_EQUAL(0, test_free_count);

	return 0;
}

CUT_BEGIN_SUITE(i86_arch_pm_suite)
	CUT_RUN_TEST(archPerfMonV1ConfigureTest1);
	CUT_RUN_TEST(archPerfMonV1ConfigureTest2);
	CUT_RUN_TEST(archPerfMonV1ConfigureTest3);
	CUT_RUN_TEST(archPerfMonV2ConfigureTest1); /* WRS_TODO add test for malloc fails */
	CUT_RUN_TEST(archPerfMonV2ConfigureTest2);
	CUT_RUN_TEST(archPerfMonV2ConfigureTest3);
CUT_END_SUITE
