/* Copyright (c) 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

/*
modification history
--------------------
09sep14,wzc  support arm hpc events.(US17715)
15aug14,wzc  fix jenkins test failure.
28mar14,sdt  fixed build issue in tcf-tools env.
13mar14,wzc  support arm hpc(US17715)
*/

/*
DESCRIPTION
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/hpc_drv/arm/armDrv.h>
#include <hpc/drv_evt_cfg/arm/armDrvCfg.h>

#include "../hpc_log.h"
#include "../hpc_utils.h"
#include "hpcHwDriverTest.h"
#include "../drv_evt_cfg/arm/armDrvCfgP.h"

/* Array index for a 2 dimensional array */
#define A2D(ROW,COLUMN,NCOLUMNS) (ROW * NCOLUMNS + COLUMN)

struct iterData
    {
    struct driverInstArm *   pInst;
    struct armSettings *     pSettings;
    unsigned int	      n;
    STATUS		      status;
    };

static STATUS hpcHwDrvTestInstCtrIncIter
    (
    int    cpu,
    void * pIterData
    )
    {
    struct iterData * pData = pIterData;
    struct driverInstArm * pInst = pData->pInst;
    struct armSettings * pArmSettings = pData->pSettings;
    int c;
    struct hpcHwCtrData * pHwCtrData;
    unsigned int value;
    int event;

/* printf("INC event %u cpu %d n = %d\n", pE500Settings->evtCode, cpu, pData->n); */

    for (c = 0; c < pInst->numCounters; c++)
        {
        /* TODO: check if the counter is running (check hw register) */
        int idx2d = A2D(cpu, c, pData->pInst->numCounters);
        pHwCtrData = pData->pInst->ppDrvHwCtrData[idx2d];

        if ((pHwCtrData) == NULL)
            continue;

        /* Check if counter is counting specified event */
        pInst->getCounterEvent(c, &event);

        if (event == (int)pArmSettings->evtCode)
            {
            pInst->getEvtCount(c, &value);
            value += pData->n;
            pInst->setEvtCount(c, value);
            setArmPmuCounterOverflow(c);
            return OK;
            }
        }
    hpc_error("can't find counter to increment %u\n", pArmSettings->evtCode);
    pData->status = ERROR;
    return ERROR;
    }

static STATUS hpcHwDrvInstTestCtrInc3
    (
    void *		  pDrvInst,
    union eventSettings * pEvtSettings,
    unsigned int	  n,
    struct hpcDriverConfig * pDrvCfg
    )
    {
    struct iterData data;

    data.pInst = pDrvInst;
    data.pSettings = &pEvtSettings->arm;
    data.n = n;
    data.status = OK;

    hpcIterateCpus(pDrvCfg->numCpuCores, hpcHwDrvTestInstCtrIncIter, &data);

    return data.status;
    }

static STATUS hpcHwDrvTestCtrInc2
    (
    struct cafe_namespace *  pNs,
    const char *	     pEventName,
    unsigned int	     n,
    struct hpcDriverConfig * pDrvConfig
    )
    {
    int e;
    struct cafe_value_type *type;
    union eventSettings drvSettings;
    struct hpcEventSettings hpcSettings;
    struct cafe_value_type *pBaseType;
    struct cafe_vm_ident *ident;
    struct armEventDescription *evtDesc;

    memset(&hpcSettings, 0, sizeof(hpcSettings));
    e = cafe_type_parse(pNs, pEventName, &type);
    if (e != CAFE_OK) {
        fprintf(stderr, "%s: Error parsing event type \"%s\": %s\n",
        __FUNCTION__, pEventName, cafe_strerror(e));
        return ERROR;
        }
    pBaseType = cafe_base_type(type);
    ident = pBaseType->ident;

    evtDesc = (struct armEventDescription *)pDrvConfig->pEventDesc;

    for (e = 0; e < pDrvConfig->numEvents; e++)
        {
        if (strstr(evtDesc->eventVtStr, ident->name) != NULL)
            {
            pDrvConfig->pEventSettingsGet(type, &drvSettings, &hpcSettings,
                (const struct hpcEventDescriptor *)evtDesc);
            return hpcHwDrvInstTestCtrInc3(pDrvConfig->pDrvInst, &drvSettings, n, pDrvConfig);
            }
        evtDesc++;
        }
    return ERROR;
    }

/* E500 core events test driver interface */

static STATUS hpcHwDrvTestCtrInc
    (
    struct cafe_namespace * pNs,
    const char *	    pEventName,
    unsigned int	    n,
    int * 		    pCores
    )
    {
    struct hpcDriverConfig * pDrvCfg = hpcCafeArmDrvCfgGet();
    *pCores = pDrvCfg->numCpuCores;
    return hpcHwDrvTestCtrInc2(pNs, pEventName, n, pDrvCfg);
    }

static STATUS hpcHwDrvInstTestCtrInc
    (
    void *		  pDrvInst,
    union eventSettings * pEvtSettings,
    unsigned int	  n
    )
    {
    struct iterData data;
    struct hpcDriverConfig * pDrvCfg = hpcCafeArmDrvCfgGet();

    data.pInst = pDrvInst;
    data.pSettings = &pEvtSettings->arm;
    data.n = n;
    data.status = OK;

    hpcIterateCpus(pDrvCfg->numCpuCores, hpcHwDrvTestInstCtrIncIter, &data);

    return data.status;
    }

static STATUS armIsrIter
    (
    int cpu,
    void * pData
    )
    {
    int * pProcessed = pData;
    *pProcessed += armEventsPrivTest.armPmuIsr(NULL);
    return OK;
    }

static int armIsr
    (
    void * pRegs
    )
    {
    int processed = 0;
    struct hpcDriverConfig * pDrvCfg = hpcCafeArmDrvCfgGet();
    hpcIterateCpus(pDrvCfg->numCpuCores, armIsrIter, &processed);
    return processed;
    }

struct hpcHwDrvTestFuncs hpcHwDrvArmTestFuncs =
    {
    hpcHwDrvInstTestCtrInc,
    hpcHwDrvTestCtrInc,
    armIsr
    };
