/* Copyright (c) 2011-2012 Wind River Systems, Inc.
 *
 * The right to copy, distribute or otherwise make use of this software
 * may be licensed only pursuant to the terms of an applicable Wind River
 * license agreement.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <hpc/hpc_drv/hpcHwDrvApi.h>
#include <hpc/drv_evt_cfg/hpcDrvEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86NehalemUncEvtCfg.h>
#include <hpc/drv_evt_cfg/i86/i86ArchPmEvtCfg.h>

#include "../hpc_log.h"
#include "../hpc_utils.h"
#include "hpcHwDriverTest.h"
#include "../drv_evt_cfg/i86/hpcI86PerfIsr.h"

/* Array index for a 2 dimensional array */
#define A2D(ROW,COLUMN,NCOLUMNS) (ROW * NCOLUMNS + COLUMN)

/******************************************************************************/
/* Test interface for general purpose uncore driver */
/******************************************************************************/

struct iterData
    {
    struct driverInstArchV1 * pInst;
    struct i86Arch1Settings * pSettings;
    unsigned int	      n;
    STATUS		      status;
    };

static STATUS hpcHwDrvTestInstCtrIncIter
    (
    int    cpu,
    void * pIterData
    )
    {
    struct iterData * pData = pIterData;
    int c, bit, overflow = 0;
    long long int perfEvtSel, count, pmGlobalStatus;
    unsigned int event, mask;
    unsigned long long pmcRollover=0;

    for (bit = 0; bit < pData->pInst->numCounterBits; bit++)
	pmcRollover |= (1ULL << bit);

/* printf("INC uncore %u cpu %d ctrs %d value %u\n", pData->pSettings->eventNum, cpu, pData->pInst->numCounters, pData->n); */

    for (c = 0; c < pData->pInst->numCounters; c++)
	{
        pentiumMsrGet (MSR_UNCORE_PERFEVTSEL0 + c, &perfEvtSel);

        event = (unsigned int)perfEvtSel & 0x0ff;
        mask = (perfEvtSel >> 8) & 0xff;
        if (event == pData->pSettings->eventNum && mask == pData->pSettings->eventUmask)
            {
            pentiumMsrGet (MSR_UNCORE_PMC0 + c, &count);
            count += pData->n;
            if (count >= pmcRollover)
                {
          	overflow = 1;
              	count = count - pmcRollover;
                }
            pentiumMsrSet (MSR_UNCORE_PMC0 + c, &count);
            /* Set the overflow status bit */
            if (overflow)
                {
               	pentiumMsrGet(MSR_UNCORE_PERF_GLOBAL_STATUS, &pmGlobalStatus);
                pmGlobalStatus |= 1 << c;
                pentiumMsrSet(MSR_UNCORE_PERF_GLOBAL_STATUS, &pmGlobalStatus);
                }
            return OK;
            }
        }
    hpc_error("can't find counter to increment\n");
    pData->status = ERROR;
    return ERROR;
    }

static STATUS hpcHwDrvInstTestCtrInc
    (
    void *		  pDrvInst,
    union eventSettings * pEvtSettings,
    unsigned int	  n
    )
    {
    struct iterData data;
    struct hpcDriverConfig * pDrv = hpcI86NehalemUncDrvCfgGet();

    data.pInst = pDrvInst;
    data.pSettings = &pEvtSettings->i86Arch1;
    data.n = n;
    data.status = OK;

    hpcIterateCpus(pDrv->numCpuCores, hpcHwDrvTestInstCtrIncIter, &data);

    return data.status;
    }

static STATUS hpcHwDrvTestCtrInc2
    (
    struct cafe_namespace * pNs,
    const char *	pEventName,
    unsigned int	n,
    struct hpcDriverConfig * pDrvConfig
    )
    {
    int e;
    struct cafe_value_type *type;
    union eventSettings drvSettings;
    struct hpcEventSettings hpcSettings;
    struct cafe_value_type *pBaseType;
    struct cafe_vm_ident *ident;
    struct i86Arch1EventDescription *evtDesc;

    memset(&hpcSettings, 0, sizeof(hpcSettings));
    e = cafe_type_parse(pNs, pEventName, &type);
    if (e != CAFE_OK)
	    return ERROR;

    pBaseType = cafe_base_type(type);
    ident = pBaseType->ident;

    evtDesc = (struct i86Arch1EventDescription *)pDrvConfig->pEventDesc;

    for (e = 0; e < pDrvConfig->numEvents; e++)
        {
        if (strstr(evtDesc->eventVtStr, ident->name) != NULL)
            {
            pDrvConfig->pEventSettingsGet(type, &drvSettings, &hpcSettings,
					  (const struct hpcEventDescriptor *)evtDesc);
            return hpcHwDrvInstTestCtrInc(pDrvConfig->pDrvInst, &drvSettings, n);
            }
        evtDesc++;
        }
    return ERROR;
    }

static STATUS hpcHwDrvTestCtrInc
    (
    struct cafe_namespace * 	pNs,
    const char *		pEventName,
    unsigned int		n,
    int *			pCores
    )
    {
    struct hpcDriverConfig * pDrv = hpcI86NehalemUncDrvCfgGet();
    *pCores = pDrv->numCpuCores;
    return hpcHwDrvTestCtrInc2(pNs, pEventName, n, pDrv);
    }

static STATUS hpcI86PerfMonHandlerTestIter
    (
    int cpu,
    void * pData
    )
    {
    int * pProcessed = pData;
    *pProcessed += hpcI86PerfMonHandler(NULL);
    return OK;
    }

static int hpcI86PerfMonHandlerTest
    (
    void * pRegs
    )
    {
    int processed = 0;
    struct hpcDriverConfig * pDrv = hpcI86NehalemUncDrvCfgGet();
    hpcIterateCpus(pDrv->numCpuCores, hpcI86PerfMonHandlerTestIter, &processed);
    return processed;
    }

struct hpcHwDrvTestFuncs hpcHwDrvX86NehalemUncTestFuncs =
    {
    hpcHwDrvInstTestCtrInc, /* called by driver API test */
    hpcHwDrvTestCtrInc, /* Called by hpc test */
    hpcI86PerfMonHandlerTest
    };

/******************************************************************************/
/* Test interface for fixed uncore driver */
/******************************************************************************/

struct iterDataFxd
    {
    struct driverInstArchV2 * pInst;
    struct i86Arch2Settings * pSettings;
    unsigned int	      n;
    STATUS		      status;
    };

static STATUS hpcHwDrvTestInstCtrIncFxdIter
    (
    int    cpu,
    void * pIterData
    )
    {
    struct iterDataFxd * pData = pIterData;
    int bit, overflow = 0;
    long long int count, pmGlobalStatus;
    unsigned long long pmcRollover=0;
    int c = pData->pSettings->counter;

    for (bit = 0; bit < pData->pInst->numCounterBits; bit++)
	pmcRollover |= (1ULL << bit);

/* printf("INC Uncore fixed cpu %d ctr %d value %u\n", cpu, pData->pSettings->counter, pData->n); */

    if (1)
        {
        pentiumMsrGet (MSR_UNCORE_FIXED_CTR0 + c, &count);
        count += pData->n;
        if (count >= pmcRollover)
           {
           overflow = 1;
           count = count - pmcRollover;
           }
        pentiumMsrSet (MSR_UNCORE_FIXED_CTR0 + c, &count);
        /* Set the overflow status bit */
        if (overflow)
            {
            pentiumMsrGet(MSR_UNCORE_PERF_GLOBAL_STATUS, &pmGlobalStatus);
            pmGlobalStatus |= 0x100000000ULL << c;
            pentiumMsrSet(MSR_UNCORE_PERF_GLOBAL_STATUS, &pmGlobalStatus);
            }
        return OK;
        }
    hpc_error("can't find counter to increment\n");
    pData->status = ERROR;
    return ERROR;
    }

static STATUS hpcHwDrvInstTestCtrIncFxd
    (
    void *		  pDrvInst,
    union eventSettings * pEvtSettings,
    unsigned int	  n
    )
    {
    struct iterDataFxd data;
    struct hpcDriverConfig * pDrv = hpcI86NehalemUncFixedDrvCfgGet();

    data.pInst = pDrvInst;
    data.pSettings = &pEvtSettings->i86Arch2;
    data.n = n;
    data.status = OK;

    hpcIterateCpus(pDrv->numCpuCores, hpcHwDrvTestInstCtrIncFxdIter, &data);

    return data.status;
    }

static STATUS hpcHwDrvTestCtrInc2Fxd
    (
    struct cafe_namespace * pNs,
    const char *	pEventName,
    unsigned int	n,
    struct hpcDriverConfig * pDrvConfig
    )
    {
    int e;
    struct cafe_value_type *type;
    union eventSettings drvSettings;
    struct hpcEventSettings hpcSettings;
    struct cafe_value_type *pBaseType;
    struct cafe_vm_ident *ident;
    struct i86Arch2EventDescription *evtDesc;

    memset(&hpcSettings, 0, sizeof(hpcSettings));
    e = cafe_type_parse(pNs, pEventName, &type);
    if (e != CAFE_OK)
	    return ERROR;

    pBaseType = cafe_base_type(type);
    ident = pBaseType->ident;

    evtDesc = (struct i86Arch2EventDescription *)pDrvConfig->pEventDesc;

    for (e = 0; e < pDrvConfig->numEvents; e++)
        {
        if (strstr(evtDesc->eventVtStr, ident->name) != NULL)
            {
            pDrvConfig->pEventSettingsGet(type, &drvSettings, &hpcSettings,
					  (const struct hpcEventDescriptor *)evtDesc);
            return hpcHwDrvInstTestCtrIncFxd(pDrvConfig->pDrvInst, &drvSettings, n);
            }
        evtDesc++;
        }
    return ERROR;
    }

static STATUS hpcHwDrvTestCtrIncFxd
    (
    struct cafe_namespace * 	pNs,
    const char *		pEventName,
    unsigned int		n,
    int	*			pCores
    )
    {
    struct hpcDriverConfig * pDrv = hpcI86NehalemUncFixedDrvCfgGet();
    *pCores = pDrv->numCpuCores;
    return hpcHwDrvTestCtrInc2Fxd(pNs, pEventName, n, pDrv);
    }

static STATUS hpcI86PerfMonHandlerTestFxdIter
    (
    int cpu,
    void * pData
    )
    {
    int * pProcessed = pData;
    *pProcessed += hpcI86PerfMonHandler(NULL);
    return OK;
    }

static int hpcI86PerfMonHandlerTestFxd
    (
    void * pRegs
    )
    {
    int processed = 0;
    struct hpcDriverConfig * pDrv = hpcI86NehalemUncFixedDrvCfgGet();
    hpcIterateCpus(pDrv->numCpuCores, hpcI86PerfMonHandlerTestFxdIter, &processed);
    return processed;
    }

struct hpcHwDrvTestFuncs hpcHwDrvX86NehalemUncFxdTestFuncs =
    {
    hpcHwDrvInstTestCtrIncFxd,	/* called by driver API test */
    hpcHwDrvTestCtrIncFxd,	/* Called by hpc test */
    hpcI86PerfMonHandlerTestFxd
    };
